/*==========================================================================*/
/* MsgTestInit.c contains the program initialisation routines.              */
/*==========================================================================*/
/* (C) Copyright Tim Armstrong 2003, 2004, 2005, 2006, 2007, 2008           */
/*==========================================================================*/
/* Includes                                                                 */
/*==========================================================================*/
#ifndef __TJAINC_MSGTEST_H
#include "MsgTest.h"
#endif /* __TJAINC_MSGTEST_H */

#ifndef __TJAINC_MSGTESTSTATICAV_H
#include "MsgTestStaticAV.h"
#endif /* __TJAINC_MSGTESTSTATICAV_H */

/*==========================================================================*/
/* Static variables                                                         */
/*==========================================================================*/
/* Constants */
#ifdef WIN32
static char* kpszDirFileConcat =      "\\\0";
#else
static char* kpszDirFileConcat =      "/\0";
#endif /* WIN32 */
static char* kpszDOT =                ".";
static char* kpszEmptyString =        "\0";


/* Value keywords */
static char* kpszLOG =                "LOG\0";
static char* kpszBUFFNAME =           "TestBuff\0";
static char* kpszMQSCBUFFNAME =       "MqscBuff\0";
static char* kpszMQMDNAME =           "TestMQMD\0";
static char* kpszRFH2NAME =           "TestRFH2\0";
static char* kpszFILEIN =             "FileIn.txt\0";
static char* kpszFILEOUT =            "FileOut.txt\0";
static char* kpszQGET =               "MSGTEST.OUT\0";
static char* kpszQPUT =               "MSGTEST.IN\0";
static char* kpszDIRVAR =             "%Dir%\0";
static char* kpszONE =                "1\0";
static char* kpszZERO =               "0\0";
static char* kpszTEN =                "10\0";
static char* kpszNONE =               "NONE\0";
static char* kpszMINUSONE =           "-1\0";
static char* kpszNUMFMT =             "%d\0";
static char* kpszDOUBLEFMT =          "%lf\0";
static char* kpszSTRFMT =             "%s\0";
static char* kpszROOT =               "Root\0";
static char* kpszDEFCONN =            "DefConn\0";
static char* kpszTESTMSGSET =         "TestMsgSet\0";
static char* kpszAUTO =               "AUTO\0";
static char* kpszSYSADMINCOMQ =       "SYSTEM.ADMIN.COMMAND.QUEUE\0";
static char* kpszSYSMQSCREPLYQ =      "SYSTEM.MQSC.REPLY.QUEUE\0";
static char* kpszMQSCREPLYDYNAMICQ =  "MSGTEST.MQSC.REPLY.QUEUE.*\0";
static char* kpszMAXMSGLENGTH =       "MaxMsgLength\0";


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     InitCBs                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Initialise MsgTest's control block structures                        */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     argc    Counter of the number of arguments supplied to the program   */
/*     argv    Pointer to list of argument pointers supplied to the program */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     SUCCESS or FAILURE                                                   */
/*==========================================================================*/
long
InitCBs(int argc, char** argv)
{
    ControlInfoBlk*  lpCIB;
    DefaultsInfoBlk* lpDIB;
    VarInfoBlk*      lpVIB;
    long             lIdx;
    time_t            oTimeT;
    struct tm*        poTimeTm ;

    /* Initialise the main control block */
    memset(kpMCB, 0x00, sizeof(MainCtrlBlk));
    time(&oTimeT);
    poTimeTm = localtime(&oTimeT);
    strftime(kpMCB->lpszTimeBuff, LOG_BUFF_LEN, "%H:%M:%S", poTimeTm );

    /* Tell the world we are starting up */
    ConsoleMsg(1, kpMCB->lpszTimeBuff, __DATE__, __TIME__);

    /* Initialise the linked list anchors */
    kpMCB->lpTIBActiveHead = NULL;
    kpMCB->lpTIBActiveTail = NULL;
    kpMCB->lpTIBReadyHead = NULL;
    kpMCB->lpTIBReadyTail = NULL;
    kpMCB->lpVIBHead = NULL;
    kpMCB->lpVIBTail = NULL;
    kpMCB->lpFBRHead = NULL;
    kpMCB->lpFBRTail = NULL;
    kpMCB->lpIFBHead = NULL;
    kpMCB->lpIFBTail = NULL;
    kpMCB->lpIIBHead = NULL;
    kpMCB->lpIIBTail = NULL;
    kpMCB->lpCIB = NULL;
    kpMCB->lpDIB = NULL;
    kpMCB->lpCDBHead = NULL;
    kpMCB->lpCDBTail = NULL;
    kpMCB->oDummyRootMSLL.eMSLLBlockType = P_ROOT;
    kpMCB->oDummyRootMSLL.lpMSLLNext = NULL;
    kpMCB->oDummyRootMSLL.lpMSLLPrev = NULL;
    kpMCB->oDummyRootMSLL.lpszNameOrType = CloneString(kpszROOT);
    kpMCB->oDummyRootMSLL.lNameOrTypeLen = StrlenNull(kpszROOT);

    /* Script file related stuff */
    kpMCB->lpszScriptFileName = NULL;
    kpMCB->lpszScriptBuff = NULL;
    kpMCB->lpszScriptName = NULL;
    kpMCB->lScriptXmlLen = 0;

    /* The main control block's processing flag */
    kpMCB->lProcessing = SUCCESS;

    /* Initialise test totals */
    kpMCB->lTotalTests = 0;
    kpMCB->lSuccessfulTests = 0;
    kpMCB->lFailedTests = 0;
    kpMCB->lIncompleteTests = 0;

    /* Initialise script basic statistics totals */
    kpMCB->lScriptTotalGetFile = 0;
    kpMCB->lScriptSuccessfulGetFile = 0;
    kpMCB->lScriptFailedGetFile = 0;
    kpMCB->lScriptTotalPutFile = 0;
    kpMCB->lScriptSuccessfulPutFile = 0;
    kpMCB->lScriptFailedPutFile = 0;
    kpMCB->lScriptTotalGetMsg = 0;
    kpMCB->lScriptSuccessfulGetMsg = 0;
    kpMCB->lScriptFailedGetMsg = 0;
    kpMCB->lScriptTotalPutMsg = 0;
    kpMCB->lScriptSuccessfulPutMsg = 0;
    kpMCB->lScriptFailedPutMsg = 0;
    kpMCB->lScriptTotalCompare = 0;
    kpMCB->lScriptSuccessfulCompare = 0;
    kpMCB->lScriptFailedCompare = 0;
    kpMCB->bScriptHasGetFile = FALSE;
    kpMCB->bScriptHasPutFile = FALSE;
    kpMCB->bScriptHasGetMsg = FALSE;
    kpMCB->bScriptHasPutMsg = FALSE;
    kpMCB->bScriptHasCompare = FALSE;
    kpMCB->bScriptHasMqscCommand = FALSE;
    kpMCB->bConnectionRequired = FALSE;

    /* Initialise the ConcurrencyWait flag and WaitTotal */
    kpMCB->bConcurrencyWait = TRUE;
    kpMCB->lWaitTotal = 0;

    /* Initialise the GmoWait flag */
    kpMCB->bGmoWait = FALSE;

    /* Initialise the MessageRate flag */
    kpMCB->bMessageRate = FALSE;

    /* The current element depth the parsing routines have reached */
    kpMCB->lScriptParseDepth = 0;

    /* Initialise the file errno copy variable */
    kpMCB->lFileErrno = 0;

    /* Initialise the ReasonCode counters and arrays */
    ClearRC();

    /* Allocate the array that holds the elements names hierarchy  */
    /* as we parse the script file                                 */
    for(lIdx = 0; lIdx < MAX_SCRIPT_PARSE_DEPTH + 1; lIdx++)
    {
        kpMCB->lplpszElementName[lIdx] = malloc(NAME_BUFF_LEN + 4);
        if(! kpMCB->lplpszElementName[lIdx])
        {
            ConsoleMsg(3, NAME_BUFF_LEN + 4, "ElementNameBuffers", __LINE__, __FILE__);
            return FAILURE;
        }
        memset(kpMCB->lplpszElementName[lIdx], 0x00, NAME_BUFF_LEN + 4);
    }

    /* Initialise the array that holds the elements type hierarchy */
    /* as we parse the script file                                 */
    for(lIdx = 0; lIdx < MAX_SCRIPT_PARSE_DEPTH + 1; lIdx++)
    {
        kpMCB->eElementType[lIdx] = E_UNKNOWN;
    }

    /* Allocate the buffer to hold the current Name attribute */
    kpMCB->lpszCurrNameAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrNameAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB NameBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrNameAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current Format attribute */
    kpMCB->lpszCurrFormatAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrFormatAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB FormatBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrFormatAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current OutputFormat attribute */
    kpMCB->lpszCurrOutputFormatAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrOutputFormatAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB OutputFormatBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrOutputFormatAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current MQMD attribute  */
    kpMCB->lpszCurrMQMDAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrMQMDAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB MQMDBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrMQMDAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current RFH2 attribute  */
    kpMCB->lpszCurrRFH2Attr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrRFH2Attr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB RFH2Buffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrRFH2Attr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current R1 attribute */
    kpMCB->lpszCurrR1Attr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrR1Attr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB R1Buffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrR1Attr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current R2 attribute */
    kpMCB->lpszCurrR2Attr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrR2Attr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB R2Buffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrR2Attr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current From attribute */
    kpMCB->lpszCurrFromAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrFromAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB FromBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrFromAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current To attribute */
    kpMCB->lpszCurrToAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrToAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB ToBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrToAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current Incr attribute */
    kpMCB->lpszCurrIncrAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrIncrAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB IncrBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrIncrAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current Cond attribute */
    kpMCB->lpszCurrCondAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrCondAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB CondBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrCondAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current Mode attribute */
    kpMCB->lpszCurrModeAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrModeAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB ModeBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrModeAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current Interval attribute */
    kpMCB->lpszCurrIntervalAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrIntervalAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB IntervalBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrIntervalAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current Pos attribute */
    kpMCB->lpszCurrPosAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrPosAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB PosBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrPosAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current Len attribute */
    kpMCB->lpszCurrLenAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrLenAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB LenBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrLenAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current Type attribute */
    kpMCB->lpszCurrTypeAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrTypeAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB TypeBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrTypeAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current MsgSep attribute */
    kpMCB->lpszCurrMsgSepAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrMsgSepAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB MsgSepBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrMsgSepAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current SegSep attribute */
    kpMCB->lpszCurrSegSepAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrSegSepAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB SegSepBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrSegSepAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current ElementSep attribute */
    kpMCB->lpszCurrElementSepAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrElementSepAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB ElementSepBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrElementSepAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current MinOccurs attribute */
    kpMCB->lpszCurrMinOccursAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrMinOccursAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB MinOccursBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrMinOccursAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current MaxOccurs attribute */
    kpMCB->lpszCurrMaxOccursAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrMaxOccursAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB MaxOccursBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrMaxOccursAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current CCSID attribute */
    kpMCB->lpszCurrCCSIDAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrCCSIDAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB CCSIDBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrCCSIDAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current Encoding attribute */
    kpMCB->lpszCurrEncodingAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrEncodingAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB EncodingBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrEncodingAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current MsgSetName attribute */
    kpMCB->lpszCurrMsgSetNameAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrMsgSetNameAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB MsgSetNameBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrMsgSetNameAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current MsgNum attribute */
    kpMCB->lpszCurrMsgNumAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrMsgNumAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB MsgNumBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrMsgNumAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current Offset attribute */
    kpMCB->lpszCurrOffsetAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrOffsetAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB OffsetBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrOffsetAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current Justify attribute */
    kpMCB->lpszCurrJustifyAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrJustifyAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB JustifyBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrJustifyAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current PadChar attribute */
    kpMCB->lpszCurrPadCharAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrPadCharAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB PadCharBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrPadCharAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current EndAlign attribute */
    kpMCB->lpszCurrEndAlignAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrEndAlignAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB EndAlignBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrEndAlignAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current CmqValues attribute */
    kpMCB->lpszCurrCmqValuesAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrCmqValuesAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB CmqValuesBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrCmqValuesAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current Truncate attribute */
    kpMCB->lpszCurrTruncateAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrTruncateAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB TruncateBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrTruncateAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current List attribute */
    kpMCB->lpszCurrListAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrListAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB ListBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrListAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current data variable attribute */
    kpMCB->lpszCurrDataVarAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrDataVarAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB DataVarBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrDataVarAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current connection attribute */
    kpMCB->lpszCurrConnectionAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrConnectionAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB ConnectionBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrConnectionAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current of attribute */
    kpMCB->lpszCurrOfAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrOfAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB OfBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrOfAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current in attribute */
    kpMCB->lpszCurrInAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrInAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB InBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrInAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current delimiter attribute */
    kpMCB->lpszCurrDelimiterAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrDelimiterAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB DelimiterBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrDelimiterAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current file extension attribute */
    kpMCB->lpszCurrFileExtensionAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrFileExtensionAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB FileExtensionBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrFileExtensionAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the buffer to hold the current remainder attribute */
    kpMCB->lpszCurrRemAttr = malloc(ATTR_BUFF_LEN + 4);
    if(! kpMCB->lpszCurrRemAttr)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "MCB RemBuffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszCurrRemAttr, 0x00, ATTR_BUFF_LEN + 4);

    /* Allocate the error work buffer */
    kpMCB->lpszErrorWorkBuff = malloc(ERROR_WORK_BUFF_LEN + 4);
    if(! kpMCB->lpszErrorWorkBuff)
    {
        ConsoleMsg(3, ATTR_BUFF_LEN + 4, "Error Work Buffer", __LINE__, __FILE__);
        return FAILURE;
    }
    memset(kpMCB->lpszErrorWorkBuff, 0x00, ERROR_WORK_BUFF_LEN + 4);

    /* Initialise reply buffer pointer and length. It will be set after the */
    /* session connections have been established if necessary               */
    kpMCB->lpszMqscReplyBuff = NULL;
    kpMCB->lMqscReplyBuffLen = 0;

    /* Set initialisation complete flag to FALSE */
    kpMCB->bInitComplete = FALSE;
    
    /* Set the For loop type to unknown */
    kpMCB->eForType = F_UNKNOWN;

    /* Set the logging suppression flag to FALSE */
    kpMCB->bLoggingSuppressed = FALSE;

    /* Set up the message rate report flag and counters */
    kpMCB->bReportMessageRate = FALSE;
    kpMCB->lMessageRateAchieved = 0;
    kpMCB->lMessageRateMissed = 0;

    /* Initialise the parsed character data pointer */
    kpMCB->lParseCharDataLen = 0;
    kpMCB->lpszParseCharDataBuff = NULL;

    /* Allocate the MCB's SPB for holding secondary parser information */
    kpMCB->lpSPB = malloc(sizeof(SecondaryParseBlk));
    if(kpMCB->lpSPB)
    {
        /* Initialise the secondary parse block fields */
        memset(kpMCB->lpSPB, 0x00, sizeof(SecondaryParseBlk));
        kpMCB->lpSPB->lpszCtrlBlkFileName = NULL;
        kpMCB->lpSPB->lpszCtrlBlkBuff = NULL;
        kpMCB->lpSPB->lCtrlBlkXmlLen = 0;
        kpMCB->lpSPB->lCtrlBlkParseDepth = 0;

        /* Allocate the array that holds the elements names hierarchy  */
        /* for secondary file parsing                                  */
        for(lIdx = 0; lIdx < MAX_CTRLBLK_PARSE_DEPTH + 1; lIdx++)
        {
            kpMCB->lpSPB->lplpszElementName[lIdx] = malloc(NAME_BUFF_LEN + 4);
            if(! kpMCB->lpSPB->lplpszElementName[lIdx])
            {
                ConsoleMsg(3, NAME_BUFF_LEN + 4, "SPB ElementNameBuffers", __LINE__, __FILE__);
                return FAILURE;
            }
            memset(kpMCB->lpSPB->lplpszElementName[lIdx], 0x00, NAME_BUFF_LEN + 4);
        }

        /* Initialise the array that holds the elements type hierarchy */
        /* for secondary file parsing                                  */
        for(lIdx = 0; lIdx < MAX_CTRLBLK_PARSE_DEPTH + 1; lIdx++)
        {
            kpMCB->lpSPB->eElementType[lIdx] = E_UNKNOWN;
        }
    }
    else
    {
        ConsoleMsg(3, sizeof(SecondaryParseBlk), "MCB SecondaryParseBlk", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
        return FAILURE;
    }

    /* Allocate the MCB's MSPB for holding message set parser information */
    kpMCB->lpMSPB = malloc(sizeof(MsgSetParseBlk));
    if(kpMCB->lpMSPB)
    {
        /* Initialise the message set parse block fields */
        memset(kpMCB->lpMSPB, 0x00, sizeof(MsgSetParseBlk));
        kpMCB->lpMSPB->lpszMsgSetBuff = NULL;
        kpMCB->lpMSPB->lMsgSetDataLen = 0;
        kpMCB->lpMSPB->lMsgSetParseDepth = 0;
        kpMCB->lpMSPB->lpCurrELB = NULL;
        kpMCB->lpMSPB->lpCurrSDB = NULL;
        kpMCB->lpMSPB->lpCurrSGB = NULL;
        kpMCB->lpMSPB->lpCurrMDB = NULL;
        kpMCB->lpMSPB->lpCurrMSTN = NULL;
        kpMCB->lpMSPB->lpCurrMSH = NULL;

        /* Allocate the array that holds the elements names hierarchy  */
        /* for message set file parsing                                */
        for(lIdx = 0; lIdx < MAX_MSGSET_PARSE_DEPTH + 1; lIdx++)
        {
            kpMCB->lpMSPB->lplpszElementName[lIdx] = malloc(NAME_BUFF_LEN + 4);
            if(! kpMCB->lpMSPB->lplpszElementName[lIdx])
            {
                ConsoleMsg(3, NAME_BUFF_LEN + 4, "MSPB ElementNameBuffers", __LINE__, __FILE__);
                return FAILURE;
            }
            memset(kpMCB->lpMSPB->lplpszElementName[lIdx], 0x00, NAME_BUFF_LEN + 4);
        }

        /* Initialise the array that holds the elements type hierarchy */
        /* for message set file parsing                                */
        for(lIdx = 0; lIdx < MAX_MSGSET_PARSE_DEPTH + 1; lIdx++)
        {
            kpMCB->lpMSPB->eMsgSetType[lIdx] = P_UNKNOWN;
        }
    }
    else
    {
        ConsoleMsg(3, sizeof(MsgSetParseBlk), "MCB MsgSetParseBlk", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
        return FAILURE;
    }

    /* Allocate the MCB's LIB for holding parsed List information */
    kpMCB->lpLIB = malloc(sizeof(ListInfoBlk));
    if(kpMCB->lpLIB)
    {
        /* Initialise the List info block fields */
        memset(kpMCB->lpLIB, 0x00, sizeof(ListInfoBlk));
        kpMCB->lpLIB->lpLEBHead = NULL;
        kpMCB->lpLIB->lpLEBTail = NULL;
        kpMCB->lpLIB->lpLEBPointerArray = NULL;
        kpMCB->lpLIB->lpszListFileName = NULL;
        kpMCB->lpLIB->lpszDir = NULL;
        kpMCB->lpLIB->lpszResListFileName = NULL;
        kpMCB->lpLIB->lpszResDir = NULL;
        kpMCB->lpLIB->lListElementCount = 0;
        kpMCB->lpLIB->bDynamicLIB = FALSE;
    }
    else
    {
        ConsoleMsg(3, sizeof(ListInfoBlk), "MCB ListInfoBlk", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
        return FAILURE;
    }

    /* Clone the Null LIB just created for use in variable initialisation */
    kpMCB->lpNullLIB = NULL;
    CloneLIB(&kpMCB->lpNullLIB, kpMCB->lpLIB);
    if(FAILURE == kpMCB->lProcessing)
    {
        return FAILURE;
    }

    /* Allocate the MCB's MIB for holding parsed MQMD information */
    kpMCB->lpMIB = malloc(sizeof(MQMDInfoBlk));
    if(kpMCB->lpMIB)
    {
        /* Initialise the MQMD info block fields */
        memset(kpMCB->lpMIB, 0x00, sizeof(MQMDInfoBlk));
        kpMCB->lpMIB->lpszVersion = NULL;
        kpMCB->lpMIB->lpszReport = NULL;
        kpMCB->lpMIB->lpszMsgType = NULL;
        kpMCB->lpMIB->lpszExpiry = NULL;
        kpMCB->lpMIB->lpszFeedback = NULL;
        kpMCB->lpMIB->lpszEncoding = NULL;
        kpMCB->lpMIB->lpszCodedCharSetId = NULL;
        kpMCB->lpMIB->lpszFormat = NULL;
        kpMCB->lpMIB->lpszPriority = NULL;
        kpMCB->lpMIB->lpszPersistence = NULL;
        kpMCB->lpMIB->lpszMsgId = NULL;
        kpMCB->lpMIB->lpszCorrelId = NULL;
        kpMCB->lpMIB->lpszBackoutCount = NULL;
        kpMCB->lpMIB->lpszReplyToQ = NULL;
        kpMCB->lpMIB->lpszReplyToQMgr = NULL;
        kpMCB->lpMIB->lpszUserIdentifier = NULL;
        kpMCB->lpMIB->lpszAccountingToken = NULL;
        kpMCB->lpMIB->lpszApplIdentityData = NULL;
        kpMCB->lpMIB->lpszPutApplType = NULL;
        kpMCB->lpMIB->lpszPutApplName = NULL;
        kpMCB->lpMIB->lpszPutDate = NULL;
        kpMCB->lpMIB->lpszPutTime = NULL;
        kpMCB->lpMIB->lpszApplOriginData = NULL;
        kpMCB->lpMIB->lpszGroupId = NULL;
        kpMCB->lpMIB->lpszMsgSeqNumber = NULL;
        kpMCB->lpMIB->lpszOffset = NULL;
        kpMCB->lpMIB->lpszMsgFlags = NULL;
        kpMCB->lpMIB->lpszOriginalLength = NULL;
        kpMCB->lpMIB->lpszMQMDFileName = NULL;
        kpMCB->lpMIB->lpszDir = NULL;
        kpMCB->lpMIB->lpszResMQMDFileName = NULL;
        kpMCB->lpMIB->lpszResDir = NULL;
        kpMCB->lpMIB->bDynamicMIB = FALSE;
    }
    else
    {
        ConsoleMsg(3, sizeof(MQMDInfoBlk), "MCB MQMDInfoBlk", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
        return FAILURE;
    }

    /* Clone the Null MIB just created for use in variable initialisation */
    kpMCB->lpNullMIB = NULL;
    CloneMIB(&kpMCB->lpNullMIB, kpMCB->lpMIB);
    if(FAILURE == kpMCB->lProcessing)
    {
        return FAILURE;
    }

    /* Allocate the MCB's RIB for holding parsed RFH2 information */
    kpMCB->lpRIB = malloc(sizeof(RFH2InfoBlk));
    if(kpMCB->lpRIB)
    {
        /* Initialise the RFH2 info block fields */
        memset(kpMCB->lpRIB, 0x00, sizeof(RFH2InfoBlk));
        kpMCB->lpRIB->lpFIBHead = NULL;
        kpMCB->lpRIB->lpFIBTail = NULL;
        kpMCB->lpRIB->lpszFormat = NULL;
        kpMCB->lpRIB->lpszRFH2FileName = NULL;
        kpMCB->lpRIB->lpszDir = NULL;
        kpMCB->lpRIB->lpszResRFH2FileName = NULL;
        kpMCB->lpRIB->lpszResDir = NULL;
        kpMCB->lpRIB->lRFH2FolderDataLen = 0;
        kpMCB->lpRIB->lRFH2FolderCount = 0;
        kpMCB->lpRIB->bDynamicRIB = FALSE;
        kpMCB->lpRIB->lEncoding = MQENC_NATIVE;
        kpMCB->lpRIB->lCodedCharSetId = 0;
        kpMCB->lpRIB->bUseRIBEncodingAndCCSID = FALSE;
    }
    else
    {
        ConsoleMsg(3, sizeof(RFH2InfoBlk), "MCB RFH2InfoBlk", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
        return FAILURE;
    }

    /* Clone the Null RIB just created for use in variable initialisation */
    kpMCB->lpNullRIB = NULL;
    CloneRIB(&kpMCB->lpNullRIB, kpMCB->lpRIB);
    if(FAILURE == kpMCB->lProcessing)
    {
        return FAILURE;
    }

    /* Allocate the MCB's CSE for holding command sub element information */
    kpMCB->lpCSE = malloc(sizeof(CSE));
    if(kpMCB->lpCSE)
    {
        memset(kpMCB->lpCSE, 0x00, sizeof(CSE));
        /* Initialise the command sub element string pointers */
        kpMCB->lpCSE->lpszBuffName = NULL;
        kpMCB->lpCSE->lpszFileName = NULL;
        kpMCB->lpCSE->lpszListFileName = NULL;
        kpMCB->lpCSE->lpszRegexFileName = NULL;
        kpMCB->lpCSE->lpszMQMDFileName = NULL;
        kpMCB->lpCSE->lpszRFH2FileName = NULL;
        kpMCB->lpCSE->lpszDir = NULL;
        kpMCB->lpCSE->lpszData = NULL;
        kpMCB->lpCSE->lpszQName = NULL;
        kpMCB->lpCSE->lpszMqscReplyDynamicQName = NULL;
        kpMCB->lpCSE->lpszQMgrName = NULL;
        kpMCB->lpCSE->lpszDepth = NULL;
        kpMCB->lpCSE->lpszInterval = NULL;
        kpMCB->lpCSE->lpszIntervalMillis = NULL;
        kpMCB->lpCSE->lpszTime = NULL;
        kpMCB->lpCSE->lpszCommand = NULL;
        kpMCB->lpCSE->lpszMaskFileName = NULL;
        kpMCB->lpCSE->lpszCmdMaxWait = NULL;
        kpMCB->lpCSE->lpszGmoOptions = NULL;
        kpMCB->lpCSE->lpszPmoOptions = NULL;
        kpMCB->lpCSE->lpszGmoMatchOptions = NULL;
        kpMCB->lpCSE->lpszGmoWait = NULL;
        kpMCB->lpCSE->lpszMsgId = NULL;
        kpMCB->lpCSE->lpszCorrelId = NULL;
        kpMCB->lpCSE->lpszGroupId = NULL;
        kpMCB->lpCSE->lpszMsgSeqNumber = NULL;
        kpMCB->lpCSE->lpszFormat = NULL;
        kpMCB->lpCSE->lpszCompRFH2VarName1 = NULL;
        kpMCB->lpCSE->lpszCompRFH2VarName2 = NULL;

        /* Initialise the command sub element flags */
        kpMCB->lpCSE->bIgnoreRC = FALSE;
        kpMCB->lpCSE->bConvert = FALSE;
        kpMCB->lpCSE->bClear = FALSE;
        kpMCB->lpCSE->bLogAndContinue = FALSE;
        kpMCB->lpCSE->bIntervalInMsgId = FALSE;
        kpMCB->lpCSE->bIntervalInCorrelId = FALSE;
        kpMCB->lpCSE->bSyncpoint = FALSE;
        kpMCB->lpCSE->bSetIdentityContext = FALSE;
        kpMCB->lpCSE->bSetAllContext = FALSE;
        kpMCB->lpCSE->bRemoveDLH = FALSE;
        kpMCB->lpCSE->bRemoveXQH = FALSE;
        kpMCB->lpCSE->bAppend = FALSE;

        /* Default is no browse i.e. use destructive MQGET's */
        kpMCB->lpCSE->eBrowseType = B_NONE;

        /* Default is no grouping for MQPUT's */
        kpMCB->lpCSE->eGroupType = G_NONE;

        kpMCB->lpCSE->lpIIB = NULL;
        kpMCB->lpCSE->lpMRB = NULL;
    }
    else
    {
        ConsoleMsg(3, sizeof(CSE), "MCB CSE", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
        return FAILURE;
    }

    /* Initialise the local integer encoding value */
    kpMCB->lLocalIntegerEncoding = MQENC_NATIVE & MQENC_INTEGER_MASK;

    /* Get the frequency of the counter we are using */
#ifdef WIN32
    /* If running under windows get the performance frequency */
    QueryPerformanceFrequency((LARGE_INTEGER*)&kpMCB->i64CountFrequency);
#else
    /* Otherwise we use gettimeofday and a timeval with micro-seconds */
    kpMCB->i64CountFrequency = 1000000;
#endif /* WIN32 */
    kpMCB->i64OneHalfMilliSecond = kpMCB->i64CountFrequency / 2000;

    /* If there are any extra parameters after the script name */
    /* store them as variables with names %1%, %2% ... %n%     */
    for(lIdx = 2; lIdx < argc; lIdx++)
    {
        memset(kpMCB->lpszCurrNameAttr, 0x00, ATTR_BUFF_LEN);
        sprintf(kpMCB->lpszCurrNameAttr, "%d\0", lIdx - 1);
        lpVIB = CreateVarInfoBlk(kpMCB->lpszCurrNameAttr);
        if(lpVIB)
        {
            lpVIB->eVarType = V_STR;
            lpVIB->lpszVarFormat = CloneString(kpszSTRFMT);
            lpVIB->lpszCurrStrValue = CloneString(argv[lIdx]);
            AddToTailOfList((LinkedListBlk**)&kpMCB->lpVIBHead,
                            (LinkedListBlk**)&kpMCB->lpVIBTail,
                            (LinkedListBlk*)lpVIB);
        }
        else
        {
            ConsoleMsg(33, kpMCB->lpszCurrNameAttr);
            kpMCB->lProcessing = FAILURE;
            return FAILURE;
        }
    }
    memset(kpMCB->lpszCurrNameAttr, 0x00, ATTR_BUFF_LEN);

    /* Initialise the script file name and script name variables */
    /* including the %ScriptName% automatic variable             */
    InitScriptAutoVars(argv);

    /* Allocate and initialise the control information block */
    kpMCB->lpCIB = malloc(sizeof(ControlInfoBlk));
    if(kpMCB->lpCIB)
    {
        lpCIB = kpMCB->lpCIB;
        memset(lpCIB, 0x00, sizeof(ControlInfoBlk));
        lpCIB->lpszConnQMgrName = NULL;
        lpCIB->lpszConnChannelName = NULL;
        lpCIB->lpszConnHostName = NULL;
        lpCIB->lpszConnPortNum = NULL;
        lpCIB->lpszConnKeyRepository = NULL;
        lpCIB->lpszConnSSLCipherSpec = NULL;
        lpCIB->lpszConnSSLPeerName = NULL;
        lpCIB->lpszConnMaxMsgLength = NULL;
        lpCIB->lpszTestLogFileName = NULL;
        lpCIB->lpszTestLogDir = NULL;
        lpCIB->bTestLogAppend = FALSE;
        lpCIB->lpszTestReportFileName = NULL;
        lpCIB->lpszTestReportDir = NULL;
        lpCIB->bTestReportAppend = FALSE;
        lpCIB->lpszTestUserLogFileName = NULL;
        lpCIB->lpszTestUserLogDir = NULL;
        lpCIB->bTestUserLogAppend = FALSE;
        lpCIB->lpszTestIntervalFilePrefix = NULL;
        lpCIB->lpszTestIntervalDelimiter = NULL;
        lpCIB->lpszTestIntervalFileExtension = NULL;
        lpCIB->lpszTestIntervalDir = NULL;
        lpCIB->bTestIntervalAppend = FALSE;
        lpCIB->lpszFilePrefix = NULL;
        lpCIB->lpszParserName = NULL;
        lpCIB->lpMSFLHead = NULL;
        lpCIB->lpMSFLTail = NULL;
        lpCIB->lpCurrCDB = NULL;
        lpCIB->lTestMaxWait = 60;
        lpCIB->lNextLogMsgInterval = 5;
        lpCIB->lConcurrency = 1;
        lpCIB->lConcurrencyWait = 20;
        lpCIB->lpszOnError = NULL;
        lpCIB->lpszConvTableDir = NULL;
        lpCIB->lpszLocalEncoding = NULL;
        lpCIB->lpszLocalCodedCharSetId = NULL;
        lpCIB->lpTestLogFile = NULL;
        lpCIB->lpTestReportFile = NULL;
        lpCIB->lpTestUserLogFile = NULL;
    }
    else
    {
        ConsoleMsg(3, sizeof(ControlInfoBlk), "ControlInfoBlk", __LINE__, __FILE__);
        return FAILURE;
    }

    /* Allocate and initialise the defaults information block */
    kpMCB->lpDIB = malloc(sizeof(DefaultsInfoBlk));
    if(kpMCB->lpDIB)
    {
        lpDIB = kpMCB->lpDIB;
        memset(lpDIB, 0x00, sizeof(DefaultsInfoBlk));
        lpDIB->lpVIBHead = NULL;
        lpDIB->lpVIBTail = NULL;
        lpDIB->lpGFB = NULL;
        lpDIB->lpPFB = NULL;
        lpDIB->lpIDB = NULL;
        lpDIB->lpOVB = NULL;
        lpDIB->lpEXB = NULL;
        lpDIB->lpSSB = NULL;
        lpDIB->lpGMB = NULL;
        lpDIB->lpPMB = NULL;
        lpDIB->lpSLB = NULL;
        lpDIB->lpWDB = NULL;
        lpDIB->lpWTB = NULL;
        lpDIB->lpCMB = NULL;
        lpDIB->lpCOB = NULL;
        lpDIB->lpBOB = NULL;
        lpDIB->lpSYB = NULL;
    }
    else
    {
        ConsoleMsg(3, sizeof(DefaultsInfoBlk), "DefaultsInfoBlk", __LINE__, __FILE__);
        return FAILURE;
    }

    return SUCCESS;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ResolveCIBVars                                                       */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Resolve any program argument variables in the CIB.                   */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     N/A                                                                  */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     SUCCESS or FAILURE                                                   */
/*==========================================================================*/
long
ResolveCIBVars()
{
    ConnectionDefBlk*  lpCDB;
    ControlInfoBlk*    lpCIB;
    MsgSetFileLoadBlk* lpMSFL;
    
    /* Initialise local variables */
    lpCIB = kpMCB->lpCIB;

    /* If the CIB queue manager name is not null or empty then we are */
    /* looking at an old style connection definition so create a      */
    /* "Default" CDB and set its values to those in the CIB           */
    if(0 < StrlenNull(lpCIB->lpszConnQMgrName))
    {
        /* See if the connection already exists. If it does then its an */
        /* error otherwise create a new connection definiton block      */
        lpCDB = FindConnectionDefBlk(kpMCB->lpCDBHead, kpMCB->lpszCurrNameAttr);
        if(NULL == lpCDB)
        {
            lpCDB = CreateConnectionDefBlk(kpszDEFCONN);
            if(! lpCDB)
            {
                kpMCB->lProcessing = FAILURE;
                return FAILURE;
            }
            AddToTailOfList((LinkedListBlk**)&kpMCB->lpCDBHead,
                            (LinkedListBlk**)&kpMCB->lpCDBTail,
                            (LinkedListBlk*)lpCDB);
        }
        else
        {
            ConsoleMsg(193);
            kpMCB->lProcessing = FAILURE;
            return FAILURE;
        }
        SetCDBToCIBValues(lpCDB);
    }

    /* Resolve any program invocation variables used in the CDB */
    lpCDB = kpMCB->lpCDBHead;
    while(NULL != lpCDB)
    {
        ResolvePgmValue(&lpCDB->lpszConnQMgrName, &lpCDB->lpszConnQMgrName);
        ResolvePgmValue(&lpCDB->lpszConnChannelName, &lpCDB->lpszConnChannelName);
        ResolvePgmValue(&lpCDB->lpszConnHostName, &lpCDB->lpszConnHostName);
        ResolvePgmValue(&lpCDB->lpszConnPortNum, &lpCDB->lpszConnPortNum);
        ResolvePgmValue(&lpCDB->lpszConnKeyRepository, &lpCDB->lpszConnKeyRepository);
        ResolvePgmValue(&lpCDB->lpszConnSSLCipherSpec, &lpCDB->lpszConnSSLCipherSpec);
        ResolvePgmValue(&lpCDB->lpszConnSSLPeerName, &lpCDB->lpszConnSSLPeerName);
        ResolvePgmValue(&lpCDB->lpszConnMaxMsgLength, &lpCDB->lpszConnMaxMsgLength);
        lpCDB = (ConnectionDefBlk*)lpCDB->lpCDBNext;
    }

    /* Resolve any program invocation variables used in the CIB */
    if(lpCIB->lpszTestLogFileName)
    {
        ResolvePgmValue(&lpCIB->lpszTestLogFileName, &lpCIB->lpszTestLogFileName);
    }
    else
    {
        lpCIB->lpszTestLogFileName = CloneString(kpszNONE);
    }
    ResolvePgmValue(&lpCIB->lpszTestLogDir, &lpCIB->lpszTestLogDir);
    if(lpCIB->lpszTestReportFileName)
    {
        ResolvePgmValue(&lpCIB->lpszTestReportFileName, &lpCIB->lpszTestReportFileName);
    }
    else
    {
        lpCIB->lpszTestReportFileName = CloneString(kpszNONE);
    }
    ResolvePgmValue(&lpCIB->lpszTestReportDir, &lpCIB->lpszTestReportDir);
    if(lpCIB->lpszTestUserLogFileName)
    {
        ResolvePgmValue(&lpCIB->lpszTestUserLogFileName, &lpCIB->lpszTestUserLogFileName);
    }
    if(lpCIB->lpszTestUserLogDir)
    {
        ResolvePgmValue(&lpCIB->lpszTestUserLogDir, &lpCIB->lpszTestUserLogDir);
    }
    ResolvePgmValue(&lpCIB->lpszTestIntervalFilePrefix, &lpCIB->lpszTestIntervalFilePrefix);
    ResolvePgmValue(&lpCIB->lpszTestIntervalDelimiter, &lpCIB->lpszTestIntervalDelimiter);
    ResolvePgmValue(&lpCIB->lpszTestIntervalFileExtension, &lpCIB->lpszTestIntervalFileExtension);
    ResolvePgmValue(&lpCIB->lpszTestIntervalDir, &lpCIB->lpszTestIntervalDir);
    ResolvePgmValue(&lpCIB->lpszOnError, &lpCIB->lpszOnError);
    ResolvePgmValue(&lpCIB->lpszConvTableDir, &lpCIB->lpszConvTableDir);
    ResolvePgmValue(&lpCIB->lpszLocalEncoding, &lpCIB->lpszLocalEncoding);
    ResolvePgmValue(&lpCIB->lpszLocalCodedCharSetId, &lpCIB->lpszLocalCodedCharSetId);

    /* Resolve any program invocation variables used in */
    /* the MsgSetFileLoadBlk's attached to the CIB      */
    lpMSFL = kpMCB->lpCIB->lpMSFLHead;
    while(NULL != lpMSFL)
    {
        ResolvePgmValue(&lpMSFL->lpszFileName, &lpMSFL->lpszFileName);
        ResolvePgmValue(&lpMSFL->lpszDir, &lpMSFL->lpszDir);
        lpMSFL = (MsgSetFileLoadBlk*)lpMSFL->lpMSFLNext;
    }

    /* Did an error occur during variable resolution */
    if(FAILURE == kpMCB->lProcessing)
    {
        return FAILURE;
    }

    return SUCCESS;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     PrepareTests                                                         */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Having successfully parsed the script file go through the linked     */
/*     list of tests and their associated command blocks                    */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     N/A                                                                  */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     SUCCESS or FAILURE                                                   */
/*==========================================================================*/
long
PrepareTests()
{
    DefaultsInfoBlk*  lpDIB;
    TestInfoBlk*      lpTIB;
    VarInfoBlk*       lpVIB;
    CommandCtrlBlk*   lpCCB;
    GetFileBlk*       lpGFBD;
    PutFileBlk*       lpPFBD;
    InlineDataBlk*    lpIDBD;
    OverlayBlk*       lpOVBD;
    ExtractBlk*       lpEXBD;
    SubstringBlk*     lpSSBD;
    GetMsgBlk*        lpGMBD;
    PutMsgBlk*        lpPMBD;
    SelectBlk*        lpSLBD;
    WaitDepthBlk*     lpWDBD;
    WaitTimeBlk*      lpWTBD;
    CompareBlk*       lpCMBD;
    CommitBlk*        lpCOBD;
    BackoutBlk*       lpBOBD;
    SystemBlk*        lpSYBD;
    GetFileBlk*       lpGFBI;
    PutFileBlk*       lpPFBI;
    InlineDataBlk*    lpIDBI;
    OverlayBlk*       lpOVBI;
    ExtractBlk*       lpEXBI;
    SubstringBlk*     lpSSBI;
    GetMsgBlk*        lpGMBI;
    PutMsgBlk*        lpPMBI;
    SelectBlk*        lpSLBI;
    WaitDepthBlk*     lpWDBI;
    WaitTimeBlk*      lpWTBI;
    CompareBlk*       lpCMBI;
    CommitBlk*        lpCOBI;
    BackoutBlk*       lpBOBI;
    SystemBlk*        lpSYBI;
    MqscCommandBlk*   lpSCBI;
    PositionBlk*      lpPSBI;
    ListInfoBlk*      lpLIB;
    MQMDInfoBlk*      lpMIB;
    RFH2InfoBlk*      lpRIB;
    IntervalInfoBlk*  lpIIB;
    long              lIdx;
    long              lTestNum;
    long              lConvNum;
    double            dConvNum;
    char              lpszVarInfo[VAR_INFO_LEN + 4];
    char              lpszVarValueFmtBuff[WORK_BUFF_LEN + 4];

    /* Get a pointer to the defaults info block */
    lpDIB = kpMCB->lpDIB;
    lpGFBD = lpDIB->lpGFB;
    lpPFBD = lpDIB->lpPFB;
    lpIDBD = lpDIB->lpIDB;
    lpOVBD = lpDIB->lpOVB;
    lpEXBD = lpDIB->lpEXB;
    lpSSBD = lpDIB->lpSSB;
    lpGMBD = lpDIB->lpGMB;
    lpPMBD = lpDIB->lpPMB;
    lpSLBD = lpDIB->lpSLB;
    lpWDBD = lpDIB->lpWDB;
    lpWTBD = lpDIB->lpWTB;
    lpCMBD = lpDIB->lpCMB;
    lpCOBD = lpDIB->lpCOB;
    lpBOBD = lpDIB->lpBOB;
    lpSYBD = lpDIB->lpSYB;

    /* Display program invocation variables */
    LogMsg(49, "Script invocation");
    memset(lpszVarInfo, 0x00, VAR_INFO_LEN + 4);
    lpVIB = kpMCB->lpVIBHead;
    if(! lpVIB)
    {
        sprintf(lpszVarInfo, "None defined");
        LogMsg(EXTRA_LINE, lpszVarInfo);
    }
    while(NULL != lpVIB)
    {
        switch(lpVIB->eVarType)
        {
        case V_STR:
            sprintf(lpszVarInfo, "%%%s%% string assigned value \"%s\"\0",
                    lpVIB->lpszVarName, lpVIB->lpszCurrStrValue);
            break;
        default:
            sprintf(lpszVarInfo, "Unexpected var=%s type=%d\0",
                    lpVIB->lpszVarName, lpVIB->eVarType);
            LogMsg(EXTRA_LINE_ERR, lpszVarInfo);
            kpMCB->lProcessing = FAILURE;
            return FAILURE;
            break;
        }
        LogMsg(EXTRA_LINE, lpszVarInfo);
        lpVIB = (VarInfoBlk*)lpVIB->lpVIBNext;
    }

    /* Display default variables */
    LogMsg(49, "<Default>");
    memset(lpszVarInfo, 0x00, VAR_INFO_LEN + 4);
    lpVIB = lpDIB->lpVIBHead;
    if(! lpVIB)
    {
        sprintf(lpszVarInfo, "None defined");
        LogMsg(EXTRA_LINE, lpszVarInfo);
    }
    while(NULL != lpVIB)
    {
        switch(lpVIB->eVarType)
        {
        case V_NUM:
            lConvNum = ConvNum(NULL, lpVIB->lpszCurrStrValue, lpVIB->lpszVarName);
            memset(lpszVarValueFmtBuff, 0x00, WORK_BUFF_LEN + 4);
            strcpy(lpszVarValueFmtBuff, "%%%s%% number was assigned the value \"");
            strcat(lpszVarValueFmtBuff, lpVIB->lpszVarFormat);
            strcat(lpszVarValueFmtBuff, "\" Format=\"%s\"\0");
            sprintf(lpszVarInfo, lpszVarValueFmtBuff, lpVIB->lpszVarName,
                    lConvNum, lpVIB->lpszVarFormat);
            break;
        case V_DOUBLE:
            dConvNum = ConvDouble(NULL, lpVIB->lpszCurrStrValue, lpVIB->lpszVarName);
            memset(lpszVarValueFmtBuff, 0x00, WORK_BUFF_LEN + 4);
            strcpy(lpszVarValueFmtBuff, "%%%s%% number was assigned the value \"");
            strcat(lpszVarValueFmtBuff, lpVIB->lpszVarFormat);
            strcat(lpszVarValueFmtBuff, "\" Format=\"%s\"\0");
            sprintf(lpszVarInfo, lpszVarValueFmtBuff, lpVIB->lpszVarName,
                    dConvNum, lpVIB->lpszVarFormat);
            break;
        case V_STR:
            sprintf(lpszVarInfo, "%%%s%% string assigned value \"%s\"\0",
                    lpVIB->lpszVarName, lpVIB->lpszCurrStrValue);
            break;
        case V_RANDOMNUM:
            lConvNum = ConvNum(NULL, lpVIB->lpszCurrStrValue, lpVIB->lpszVarName);
            memset(lpszVarValueFmtBuff, 0x00, WORK_BUFF_LEN + 4);
            strcpy(lpszVarValueFmtBuff, "%%%s%% random number assigned seed value \"");
            strcat(lpszVarValueFmtBuff, lpVIB->lpszVarFormat);
            strcat(lpszVarValueFmtBuff, "\" Format=\"%s\"\0");
            sprintf(lpszVarInfo, lpszVarValueFmtBuff, lpVIB->lpszVarName,
                    lConvNum, lpVIB->lpszVarFormat);
            break;
        case V_LIST:
            lpLIB = lpVIB->lpCtrlBlk;
            sprintf(lpszVarInfo, "List=%s created\0",
                    lpVIB->lpszVarName);
            break;
        case V_MQMD:
            lpMIB = lpVIB->lpCtrlBlk;
            sprintf(lpszVarInfo, "MQMD=%s created\0",
                    lpVIB->lpszVarName);
            break;
        case V_RFH2:
            lpRIB = lpVIB->lpCtrlBlk;
            sprintf(lpszVarInfo, "RFH2=%s created\0",
                    lpVIB->lpszVarName);
            break;
        default:
            sprintf(lpszVarInfo, "Unexpected var=%s type=%d\0",
                    lpVIB->lpszVarName, lpVIB->eVarType);
            LogMsg(EXTRA_LINE_ERR, lpszVarInfo);
            kpMCB->lProcessing = FAILURE;
            return FAILURE;
            break;
        }
        LogMsg(EXTRA_LINE, lpszVarInfo);
        lpVIB = (VarInfoBlk*)lpVIB->lpVIBNext;
    }

    /* Process the interval info blocks */
    lpIIB = kpMCB->lpIIBHead;
    while(NULL != lpIIB)
    {
        /* Check each has a start and end */
        if((TRUE == lpIIB->bHaveStartCmd) && (FALSE == lpIIB->bHaveEndCmd))
        {
            LogMsg(96, lpIIB->lpszIntervalName, "<Interval End>");
            kpMCB->lProcessing = FAILURE;
        }
        if((FALSE == lpIIB->bHaveStartCmd) && (TRUE == lpIIB->bHaveEndCmd))
        {
            LogMsg(96, lpIIB->lpszIntervalName, "<Interval Start>");
            kpMCB->lProcessing = FAILURE;
        }

        /* Open the interval info recording file if requested */
        if(0 < StrlenNull(kpMCB->lpCIB->lpszTestIntervalFilePrefix))
        {
            if(0 != memcmp(kpszNONE, kpMCB->lpCIB->lpszTestIntervalFilePrefix, strlen(kpszNONE) + 1))
            {
                OpenIntervalFile(lpIIB);
            }
        }

        /* Next interval */
        lpIIB = (IntervalInfoBlk*)lpIIB->lpIIBNext;
    }
    if(FAILURE == kpMCB->lProcessing)
    {
        return FAILURE;
    }

    /* Process each test in the ready list */
    lTestNum = 1;
    lpTIB = kpMCB->lpTIBReadyHead;
    while(NULL != lpTIB)
    {
        /* Assign each test a number and set its initial state          */
        lpTIB->lTestNum = lTestNum;
        lpTIB->bCmdResult = SUCCESS;
        lpTIB->bFinallyResetFailedTest = FALSE;
        /* Process all the command blocks replacing any missing values  */
        /* with their defaults from the defaults info block. If there   */
        /* is no defaults info block then assign program defaults. Also */
        /* if any of the values in the defaults block are missing then  */
        /* use the program defaults.                                    */
        lpCCB = lpTIB->lpCCBHead;
        while(NULL != lpCCB)
        {
            switch(lpCCB->eCommandType)
            {
            case C_GETFILE:
                lpTIB->bTestHasGetFile = TRUE;
                lpGFBI = (GetFileBlk*)lpCCB;
                if(lpGFBD)
                {
                    ChangeIfStr1Empty(&lpGFBI->lpszMQMDVarName,
                                      &lpGFBD->lpszMQMDVarName);
                    ChangeIfStr1Empty(&lpGFBI->lpszRFH2VarName,
                                      &lpGFBD->lpszRFH2VarName);
                    ChangeIfStr1Empty(&lpGFBI->lpszBuffName,
                                      &lpGFBD->lpszBuffName);
                    ChangeIfStr1Empty(&lpGFBI->lpszFileName,
                                      &lpGFBD->lpszFileName);
                    ChangeIfStr1Empty(&lpGFBI->lpszMQMDFileName,
                                      &lpGFBD->lpszMQMDFileName);
                    ChangeIfStr1Empty(&lpGFBI->lpszRFH2FileName,
                                      &lpGFBD->lpszRFH2FileName);
                    ChangeIfStr1Empty(&lpGFBI->lpszDir,
                                      &lpGFBD->lpszDir);
                    ChangeIfStr1Empty(&lpGFBI->lpszCmdMaxWait,
                                      &lpGFBD->lpszCmdMaxWait);
                }
                ChangeIfStr1Empty(&lpGFBI->lpszMQMDVarName,
                                  &kpszMQMDNAME);
                ChangeIfStr1Empty(&lpGFBI->lpszRFH2VarName,
                                  &kpszRFH2NAME);
                ChangeIfStr1Empty(&lpGFBI->lpszBuffName,
                                  &kpszBUFFNAME);
                ChangeIfStr1Empty(&lpGFBI->lpszFileName,
                                  &kpszFILEIN);
                ChangeIfStr1Empty(&lpGFBI->lpszMQMDFileName,
                                  NULL);
                ChangeIfStr1Empty(&lpGFBI->lpszRFH2FileName,
                                  NULL);
                ChangeIfStr1Empty(&lpGFBI->lpszDir,
                                  &kpszDIRVAR);
                ChangeIfStr1Empty(&lpGFBI->lpszCmdMaxWait,
                                  &kpszMINUSONE);
                break;
            case C_PUTFILE:
                lpTIB->bTestHasPutFile = TRUE;
                lpPFBI = (PutFileBlk*)lpCCB;
                if(lpPFBD)
                {
                    ChangeIfStr1Empty(&lpPFBI->lpszMQMDVarName,
                                      &lpPFBD->lpszMQMDVarName);
                    ChangeIfStr1Empty(&lpPFBI->lpszRFH2VarName,
                                      &lpPFBD->lpszRFH2VarName);
                    ChangeIfStr1Empty(&lpPFBI->lpszBuffName,
                                      &lpPFBD->lpszBuffName);
                    ChangeIfStr1Empty(&lpPFBI->lpszFileName,
                                      &lpPFBD->lpszFileName);
                    ChangeIfStr1Empty(&lpPFBI->lpszMQMDFileName,
                                      &lpPFBD->lpszMQMDFileName);
                    ChangeIfStr1Empty(&lpPFBI->lpszRFH2FileName,
                                      &lpPFBD->lpszRFH2FileName);
                    ChangeIfStr1Empty(&lpPFBI->lpszDir,
                                      &lpPFBD->lpszDir);
                    if(TRUE == lpPFBD->bAppend)
                    {
                        lpPFBI->bAppend = TRUE;
                    }
                    /* For the DataVar attribute NULL=no-override so only */
                    /* call ChangeIfStr1Empty if default is not NULL      */
                    if(lpPFBD->lpszDataVarName)
                    {
                        ChangeIfStr1Empty(&lpPFBI->lpszDataVarName,
                                          &lpPFBD->lpszDataVarName);
                    }
                }
                ChangeIfStr1Empty(&lpPFBI->lpszMQMDVarName,
                                  &kpszMQMDNAME);
                ChangeIfStr1Empty(&lpPFBI->lpszRFH2VarName,
                                  &kpszRFH2NAME);
                ChangeIfStr1Empty(&lpPFBI->lpszBuffName,
                                  &kpszBUFFNAME);
                ChangeIfStr1Empty(&lpPFBI->lpszMQMDFileName,
                                  NULL);
                ChangeIfStr1Empty(&lpPFBI->lpszRFH2FileName,
                                  NULL);
                /* If both the MQMDFileName and the RFH2FileName are blank */
                /* the set the default output file name to "FileOut.txt"   */
                /* otherwise permit the user to only output MQMD of RFH2   */
                /* information to their respective files                   */
                if((0 == StrlenNull(lpPFBI->lpszMQMDFileName)) &&
                   (0 == StrlenNull(lpPFBI->lpszRFH2FileName)))
                {
                    ChangeIfStr1Empty(&lpPFBI->lpszFileName,
                                      &kpszFILEOUT);
                }
                else
                {
                    ChangeIfStr1Empty(&lpPFBI->lpszFileName,
                                      NULL);
                }
                ChangeIfStr1Empty(&lpPFBI->lpszDir,
                                  &kpszDIRVAR);
                break;
            case C_INLINEDATA:
                lpIDBI = (InlineDataBlk*)lpCCB;
                if(lpIDBD)
                {
                    ChangeIfStr1Empty(&lpIDBI->lpszBuffName,
                                      &lpIDBD->lpszBuffName);
                    ChangeIfStr1Empty(&lpIDBI->lpszData,
                                      &lpIDBD->lpszData);
                }
                ChangeIfStr1Empty(&lpIDBI->lpszBuffName,
                                  &kpszBUFFNAME);
                ChangeIfStr1Empty(&lpIDBI->lpszData,
                                  NULL);
                break;
            case C_OVERLAY:
                lpOVBI = (OverlayBlk*)lpCCB;
                if(lpOVBD)
                {
                    ChangeIfStr1Empty(&lpOVBI->lpszPos,
                                      &lpOVBD->lpszPos);
                    ChangeIfStr1Empty(&lpOVBI->lpszLen,
                                      &lpOVBD->lpszLen);
                    ChangeIfStr1Empty(&lpOVBI->lpszBuffName,
                                      &lpOVBD->lpszBuffName);
                    ChangeIfStr1Empty(&lpOVBI->lpszData,
                                      &lpOVBD->lpszData);
                }
                ChangeIfStr1Empty(&lpOVBI->lpszPos,
                                  &kpszONE);
                ChangeIfStr1Empty(&lpOVBI->lpszLen,
                                  &kpszZERO);
                ChangeIfStr1Empty(&lpOVBI->lpszBuffName,
                                  &kpszBUFFNAME);
                ChangeIfStr1Empty(&lpOVBI->lpszData,
                                  NULL);
                break;
            case C_EXTRACT:
                lpEXBI = (ExtractBlk*)lpCCB;
                if(lpEXBD)
                {
                    ChangeIfStr1Empty(&lpEXBI->lpszVarName,
                                      &lpEXBD->lpszVarName);
                    ChangeIfStr1Empty(&lpEXBI->lpszPos,
                                      &lpEXBD->lpszPos);
                    ChangeIfStr1Empty(&lpEXBI->lpszLen,
                                      &lpEXBD->lpszLen);
                    ChangeIfStr1Empty(&lpEXBI->lpszBuffName,
                                      &lpEXBD->lpszBuffName);
                }
                ChangeIfStr1Empty(&lpEXBI->lpszPos,
                                  &kpszONE);
                ChangeIfStr1Empty(&lpEXBI->lpszLen,
                                  &kpszZERO);
                ChangeIfStr1Empty(&lpEXBI->lpszBuffName,
                                  &kpszBUFFNAME);
                break;
            case C_SUBSTRING:
                lpSSBI = (SubstringBlk*)lpCCB;
                if(lpSSBD)
                {
                    ChangeIfStr1Empty(&lpSSBI->lpszVarName,
                                      &lpSSBD->lpszVarName);
                    ChangeIfStr1Empty(&lpSSBI->lpszFromVarName,
                                      &lpSSBD->lpszFromVarName);
                    ChangeIfStr1Empty(&lpSSBI->lpszPos,
                                      &lpSSBD->lpszPos);
                    ChangeIfStr1Empty(&lpSSBI->lpszLen,
                                      &lpSSBD->lpszLen);
                }
                ChangeIfStr1Empty(&lpSSBI->lpszPos,
                                  &kpszONE);
                ChangeIfStr1Empty(&lpSSBI->lpszLen,
                                  &kpszZERO);
                break;
            case C_GETMSG:
                lpTIB->bTestHasGetMsg = TRUE;
                lpGMBI = (GetMsgBlk*)lpCCB;
                if(lpGMBD)
                {
                    ChangeIfStr1Empty(&lpGMBI->lpszMQMDVarName,
                                      &lpGMBD->lpszMQMDVarName);
                    ChangeIfStr1Empty(&lpGMBI->lpszRFH2VarName,
                                      &lpGMBD->lpszRFH2VarName);
                    ChangeIfStr1Empty(&lpGMBI->lpszConnectionName,
                                      &lpGMBD->lpszConnectionName);
                    ChangeIfStr1Empty(&lpGMBI->lpszBuffName,
                                      &lpGMBD->lpszBuffName);
                    ChangeIfStr1Empty(&lpGMBI->lpszQName,
                                      &lpGMBD->lpszQName);
                    ChangeIfStr1Empty(&lpGMBI->lpszCmdMaxWait,
                                      &lpGMBD->lpszCmdMaxWait);
                    ChangeIfStr1Empty(&lpGMBI->lpszGmoOptions,
                                      &lpGMBD->lpszGmoOptions);
                    ChangeIfStr1Empty(&lpGMBI->lpszGmoMatchOptions,
                                      &lpGMBD->lpszGmoMatchOptions);
                    ChangeIfStr1Empty(&lpGMBI->lpszGmoWait,
                                      &lpGMBD->lpszGmoWait);
                    /* For the MsgId and CorrelId special override variables */
                    /* NULL=no-override and EmptyString=MQMI_NONE/MQCI_NONE  */
                    /* so only call ChangeIfStr1Empty if default is not NULL */
                    if(lpGMBD->lpszMsgId)
                    {
                        ChangeIfStr1Empty(&lpGMBI->lpszMsgId,
                                          &lpGMBD->lpszMsgId);
                    }
                    if(lpGMBD->lpszCorrelId)
                    {
                        ChangeIfStr1Empty(&lpGMBI->lpszCorrelId,
                                          &lpGMBD->lpszCorrelId);
                    }
                    if(lpGMBD->lpszGroupId)
                    {
                        ChangeIfStr1Empty(&lpGMBI->lpszGroupId,
                                          &lpGMBD->lpszGroupId);
                    }
                    if(lpGMBD->lpszMsgSeqNumber)
                    {
                        ChangeIfStr1Empty(&lpGMBI->lpszMsgSeqNumber,
                                          &lpGMBD->lpszMsgSeqNumber);
                    }
                }
                ChangeIfStr1Empty(&lpGMBI->lpszMQMDVarName,
                                  &kpszMQMDNAME);
                ChangeIfStr1Empty(&lpGMBI->lpszRFH2VarName,
                                  &kpszRFH2NAME);
                ChangeIfStr1Empty(&lpGMBI->lpszConnectionName,
                                  &kpszDEFCONN);
                ChangeIfStr1Empty(&lpGMBI->lpszBuffName,
                                  &kpszBUFFNAME);
                ChangeIfStr1Empty(&lpGMBI->lpszQName,
                                  &kpszQGET);
                ChangeIfStr1Empty(&lpGMBI->lpszCmdMaxWait,
                                  &kpszMINUSONE);
                ChangeIfStr1Empty(&lpGMBI->lpszGmoWait,
                                  &kpszZERO);
                /* Check that only one of the interval options is being used */
                if((TRUE == lpGMBI->bIntervalInMsgId) && (TRUE == lpGMBI->bIntervalInCorrelId))
                {
                    LogMsg(112, "<GetMsg>\0");
                    kpMCB->lProcessing = FAILURE;
                    return FAILURE;
                }
                /* Browse automatic mode can only be used inside a loop */
                if((B_BROWSEAUTO == lpGMBI->eBrowseType) && (NULL == lpGMBI->lpBreakContinueFSB))
                {
                    LogMsg(154);
                    kpMCB->lProcessing = FAILURE;
                    return FAILURE;
                }
                /* Not allowed to use <GmoOptions> in combination with certain other GMO flags */
                if((0 < StrlenNull(lpGMBI->lpszGmoOptions)) &&
                  (TRUE == lpGMBI->bSyncpoint))
                {
                    LogMsg(201);
                    kpMCB->lProcessing = FAILURE;
                    return FAILURE;
                }
                break;
            case C_PUTMSG:
                lpTIB->bTestHasPutMsg = TRUE;
                lpPMBI = (PutMsgBlk*)lpCCB;
                if(lpPMBD)
                {
                    ChangeIfStr1Empty(&lpPMBI->lpszMQMDVarName,
                                      &lpPMBD->lpszMQMDVarName);
                    ChangeIfStr1Empty(&lpPMBI->lpszRFH2VarName,
                                      &lpPMBD->lpszRFH2VarName);
                    ChangeIfStr1Empty(&lpPMBI->lpszConnectionName,
                                      &lpPMBD->lpszConnectionName);
                    ChangeIfStr1Empty(&lpPMBI->lpszBuffName,
                                      &lpPMBD->lpszBuffName);
                    ChangeIfStr1Empty(&lpPMBI->lpszQName,
                                      &lpPMBD->lpszQName);
                    ChangeIfStr1Empty(&lpPMBI->lpszQMgrName,
                                      &lpPMBD->lpszQMgrName);
                    ChangeIfStr1Empty(&lpPMBI->lpszPmoOptions,
                                      &lpPMBD->lpszPmoOptions);
                    /* For the DataVar attribute, MsgId, CorrelId and Format*/
                    /* special override variables NULL=no-override and      */
                    /* EmptyString=MQMI_NONE or MQCI_NONE or MQFMT_NONE so  */
                    /* only call ChangeIfStr1Empty if default is not NULL   */
                    if(lpPMBD->lpszDataVarName)
                    {
                        ChangeIfStr1Empty(&lpPMBI->lpszDataVarName,
                                          &lpPMBD->lpszDataVarName);
                    }
                    if(lpPMBD->lpszMsgId)
                    {
                        ChangeIfStr1Empty(&lpPMBI->lpszMsgId,
                                          &lpPMBD->lpszMsgId);
                    }
                    if(lpPMBD->lpszCorrelId)
                    {
                        ChangeIfStr1Empty(&lpPMBI->lpszCorrelId,
                                          &lpPMBD->lpszCorrelId);
                    }
                    if(lpPMBD->lpszGroupId)
                    {
                        ChangeIfStr1Empty(&lpPMBI->lpszGroupId,
                                          &lpPMBD->lpszGroupId);
                    }
                    if(lpPMBD->lpszMsgSeqNumber)
                    {
                        ChangeIfStr1Empty(&lpPMBI->lpszMsgSeqNumber,
                                          &lpPMBD->lpszMsgSeqNumber);
                    }
                    if(lpPMBD->lpszFormat)
                    {
                        ChangeIfStr1Empty(&lpPMBI->lpszFormat,
                                          &lpPMBD->lpszFormat);
                    }
                }
                ChangeIfStr1Empty(&lpPMBI->lpszMQMDVarName,
                                  &kpszMQMDNAME);
                /* Was an RFH2 variable specified for MQPUT */
                if(0 < strlen(lpPMBI->lpszRFH2VarName))
                {
                    lpPMBI->bSuppressRFH2 = FALSE;
                }
                ChangeIfStr1Empty(&lpPMBI->lpszRFH2VarName,
                                  &kpszRFH2NAME);
                ChangeIfStr1Empty(&lpPMBI->lpszConnectionName,
                                  &kpszDEFCONN);
                ChangeIfStr1Empty(&lpPMBI->lpszBuffName,
                                  &kpszBUFFNAME);
                ChangeIfStr1Empty(&lpPMBI->lpszQName,
                                  &kpszQPUT);
                /* V110 code mod no longer defaults to CIB queue manager name */
                /* to permit proper handling of cluster queues. Also brings   */
                /* into line with the documentation                           */
                ChangeIfStr1Empty(&lpPMBI->lpszQMgrName, &kpszEmptyString);
                /* Check that only one of the interval options is being used */
                if((TRUE == lpPMBI->bIntervalInMsgId) && (TRUE == lpPMBI->bIntervalInCorrelId))
                {
                    LogMsg(112, "<PutMsg>\0");
                    kpMCB->lProcessing = FAILURE;
                    return FAILURE;
                }
                if((TRUE == lpPMBI->bSetIdentityContext) && (TRUE == lpPMBI->bSetAllContext))
                {
                    LogMsg(195);
                    kpMCB->lProcessing = FAILURE;
                    return FAILURE;
                }
                /* Not allowed to use <PmoOptions> in combination with other PMO flags */
                if((0 < StrlenNull(lpPMBI->lpszPmoOptions)) &&
                   ((TRUE == lpPMBI->bSetAllContext) ||
                    (TRUE == lpPMBI->bSetIdentityContext) ||
                    (TRUE == lpPMBI->bSyncpoint)))
                {
                    LogMsg(200);
                    kpMCB->lProcessing = FAILURE;
                    return FAILURE;
                }
                /* Group automatic mode can only be used inside a loop */
                if((G_GROUPAUTO == lpPMBI->eGroupType) && (NULL == lpPMBI->lpBreakContinueFSB))
                {
                    LogMsg(202);
                    kpMCB->lProcessing = FAILURE;
                    return FAILURE;
                }
                /* Group automatic mode cannot be used inside a ForEver loop */
                if((G_GROUPAUTO == lpPMBI->eGroupType) && (NULL != lpPMBI->lpBreakContinueFSB))
                {
                    if(F_FOREVER == lpPMBI->lpBreakContinueFSB->eForType)
                    {
                        LogMsg(203);
                        kpMCB->lProcessing = FAILURE;
                        return FAILURE;
                    }
                }
                break;
            case C_SELECT:
                lpSLBI = (SelectBlk*)lpCCB;
                if(lpSLBD)
                {
                    ChangeIfStr1Empty(&lpSLBI->lpszMQMDVarName,
                                      &lpSLBD->lpszMQMDVarName);
                    ChangeIfStr1Empty(&lpSLBI->lpszBuffName,
                                      &lpSLBD->lpszBuffName);
                    ChangeIfStr1Empty(&lpSLBI->lpszMsgSetName,
                                      &lpSLBD->lpszMsgSetName);
                    ChangeIfStr1Empty(&lpSLBI->lpszMsgNum,
                                      &lpSLBD->lpszMsgNum);
                }
                ChangeIfStr1Empty(&lpSLBI->lpszMQMDVarName,
                                  &kpszMQMDNAME);
                ChangeIfStr1Empty(&lpSLBI->lpszBuffName,
                                  &kpszBUFFNAME);
                ChangeIfStr1Empty(&lpSLBI->lpszMsgSetName,
                                  &kpszTESTMSGSET);
                ChangeIfStr1Empty(&lpSLBI->lpszMsgNum,
                                  &kpszAUTO);
                break;
            case C_WAITONMSGDEPTH:
                lpWDBI = (WaitDepthBlk*)lpCCB;
                if(lpWDBD)
                {
                    ChangeIfStr1Empty(&lpWDBI->lpszConnectionName,
                                      &lpWDBD->lpszConnectionName);
                    ChangeIfStr1Empty(&lpWDBI->lpszQName,
                                      &lpWDBD->lpszQName);
                    ChangeIfStr1Empty(&lpWDBI->lpszDepth,
                                      &lpWDBD->lpszDepth);
                    ChangeIfStr1Empty(&lpWDBI->lpszCmdMaxWait,
                                      &lpWDBD->lpszCmdMaxWait);
                }
                ChangeIfStr1Empty(&lpWDBI->lpszConnectionName,
                                  &kpszDEFCONN);
                ChangeIfStr1Empty(&lpWDBI->lpszQName,
                                  &kpszQGET);
                ChangeIfStr1Empty(&lpWDBI->lpszDepth,
                                  &kpszONE);
                ChangeIfStr1Empty(&lpWDBI->lpszCmdMaxWait,
                                  &kpszMINUSONE);
                break;
            case C_WAITONTIME:
                lpWTBI = (WaitTimeBlk*)lpCCB;
                if(lpWTBD)
                {
                    ChangeIfStr1Empty(&lpWTBI->lpszInterval,
                                      &lpWTBD->lpszInterval);
                    ChangeIfStr1Empty(&lpWTBI->lpszIntervalMillis,
                                      &lpWTBD->lpszIntervalMillis);
                    ChangeIfStr1Empty(&lpWTBI->lpszTime,
                                      &lpWTBD->lpszTime);
                }
                /* If none of the Interval, IntervalMillis or Time options has */
                /* been requested then the default Interval is 10 seconds      */
                if((0 == StrlenNull(lpWTBI->lpszInterval)) &&
                   (0 == StrlenNull(lpWTBI->lpszIntervalMillis)) &&
                   (0 == StrlenNull(lpWTBI->lpszTime)))
                {
                    lpWTBI->eWaitType = W_INTERVAL;
                    ChangeIfStr1Empty(&lpWTBI->lpszInterval, &kpszTEN);
                }
                /* If we dont know which kind of wait then try the default setting */
                if(W_UNKNOWN == lpWTBI->eWaitType)
                {
                    lpWTBI->eWaitType = lpWTBD->eWaitType;
                }
                /* If it still unknown then default to Interval */
                if(W_UNKNOWN == lpWTBI->eWaitType)
                {
                    lpWTBI->eWaitType = W_INTERVAL;
                }
                break;
            case C_COMPARE:
                lpTIB->bTestHasCompare = TRUE;
                lpCMBI = (CompareBlk*)lpCCB;
                if(lpCMBD)
                {
                    ChangeIfStr1Empty(&lpCMBI->lpszBuffName,
                                      &lpCMBD->lpszBuffName);
                    ChangeIfStr1Empty(&lpCMBI->lpszCompFileName,
                                      &lpCMBD->lpszCompFileName);
                    ChangeIfStr1Empty(&lpCMBI->lpszCompMaskFileName,
                                      &lpCMBD->lpszCompMaskFileName);
                    ChangeIfStr1Empty(&lpCMBI->lpszCompRegexFileName,
                                      &lpCMBD->lpszCompRegexFileName);
                    ChangeIfStr1Empty(&lpCMBI->lpszCompRFH2VarName1,
                                      &lpCMBD->lpszCompRFH2VarName1);
                    ChangeIfStr1Empty(&lpCMBI->lpszCompRFH2VarName2,
                                      &lpCMBD->lpszCompRFH2VarName2);
                    ChangeIfStr1Empty(&lpCMBI->lpszDir,
                                      &lpCMBD->lpszDir);
                    if(TRUE == lpCMBD->bLogAndContinue)
                    {
                        lpCMBI->bLogAndContinue = TRUE;
                    }
                }
                ChangeIfStr1Empty(&lpCMBI->lpszBuffName,
                                  &kpszBUFFNAME);
                ChangeIfStr1Empty(&lpCMBI->lpszCompFileName,
                                  NULL);
                ChangeIfStr1Empty(&lpCMBI->lpszCompMaskFileName,
                                  NULL);
                ChangeIfStr1Empty(&lpCMBI->lpszCompRegexFileName,
                                  NULL);
                ChangeIfStr1Empty(&lpCMBI->lpszCompRFH2VarName1,
                                  NULL);
                ChangeIfStr1Empty(&lpCMBI->lpszCompRFH2VarName2,
                                  NULL);
                ChangeIfStr1Empty(&lpCMBI->lpszDir,
                                  &kpszDIRVAR);
                if(((0 < StrlenNull(lpCMBI->lpszCompFileName)) || (0 < StrlenNull(lpCMBI->lpszCompMaskFileName))) &&
                    (0 < StrlenNull(lpCMBI->lpszCompRegexFileName)))
                {
                    LogMsg(271);
                    kpMCB->lProcessing = FAILURE;
                    return FAILURE;
                }
                break;
            case C_COMMIT:
                lpCOBI = (CommitBlk*)lpCCB;
                if(lpCOBD)
                {
                    ChangeIfStr1Empty(&lpCOBI->lpszConnectionName,
                                      &lpCOBD->lpszConnectionName);
                }
                ChangeIfStr1Empty(&lpCOBI->lpszConnectionName,
                                  &kpszDEFCONN);
                break;
            case C_BACKOUT:
                lpBOBI = (BackoutBlk*)lpCCB;
                if(lpBOBD)
                {
                    ChangeIfStr1Empty(&lpBOBI->lpszConnectionName,
                                      &lpBOBD->lpszConnectionName);
                }
                ChangeIfStr1Empty(&lpBOBI->lpszConnectionName,
                                  &kpszDEFCONN);
                break;
            case C_SYSTEM:
                lpSYBI = (SystemBlk*)lpCCB;
                if(lpSYBD)
                {
                    ChangeIfStr1Empty(&lpSYBI->lpszCommand,
                                      &lpSYBD->lpszCommand);
                }
                ChangeIfStr1Empty(&lpSYBI->lpszCommand,
                                  NULL);
                break;
            case C_MQSCCOMMAND:
                lpSCBI = (MqscCommandBlk*)lpCCB;
                ChangeIfStr1Empty(&lpSCBI->lpszConnectionName,
                                  &kpszDEFCONN);
                ChangeIfStr1Empty(&lpSCBI->lpszBuffName,
                                  &kpszMQSCBUFFNAME);
                ChangeIfStr1Empty(&lpSCBI->lpszQName,
                                  &kpszSYSADMINCOMQ);
                ChangeIfStr1Empty(&lpSCBI->lpszMqscReplyDynamicQName,
                                  &kpszMQSCREPLYDYNAMICQ);
                ChangeIfStr1Empty(&lpSCBI->lpszCmdMaxWait,
                                  &kpszMINUSONE);
                break;
            /* Nothing needs to be done but its not an error */
            case C_FORSTART:
            case C_FOREND:
            case C_IFSTART:
            case C_IFEND:
            case C_ELSEIF:
            case C_ELSE:
            case C_CONTINUE:
            case C_BREAK:
            case C_EXITTEST:
            case C_FAILTEST:
            case C_FINALLYSTART:
            case C_INTERVALSTART:
            case C_INTERVALEND:
            case C_VAR:
            case C_MATH:
            case C_LENGTH:
                break;
            case C_POSITION:
                lpPSBI = (PositionBlk*)lpCCB;
                ChangeIfStr1Empty(&lpPSBI->lpszFrom, &kpszONE);
                break;
            /* Nothing needs to be done but its not an error */
            case C_STDIN:
            case C_STDOUT:
            case C_USERLOG:
                break;
            case C_FINALLYEND:
                if(NULL != lpCCB->lpCCBNext)
                {
                    LogMsg(191);
                    kpMCB->lProcessing = FAILURE;
                }
                break;
            default:
                LogMsg(31, lpCCB->eCommandType);
                kpMCB->lProcessing = FAILURE;
                return FAILURE;
            }
            if(FAILURE == kpMCB->lProcessing)
            {
                return FAILURE;
            }
            /* Command block is ready for initialisation */
            lpCCB->eCommandState = S_INIT;
            /* Next command block in the chain */
            lpCCB = (CommandCtrlBlk*)lpCCB->lpCCBNext;
        }

        /* Point the current command block pointer to the first command */
        lpTIB->lpCCBCurr = lpTIB->lpCCBHead;
        
        /* Log and report the fact that the test is now initialised */
        LogMsg(34, lpTIB->lTestNum, lpTIB->lpszTestName);
        ReportMsg(34, lpTIB->lTestNum, lpTIB->lpszTestName);
        kpMCB->lTotalTests++;
        kpMCB->lIncompleteTests++;
        
        /* Next test */
        lTestNum++;
        lpTIB = (TestInfoBlk*)lpTIB->lpTIBNext;
    }

    /* Check for conditions where because we only pseudo-thread */
    /* we wont allow a concurrency level greater than one       */
    if((kpMCB->lpCIB->lConcurrency > 1) && (TRUE == kpMCB->bGmoWait))
    {
        ConsoleMsg(120);
        LogMsg(120);
        kpMCB->lProcessing = FAILURE;
        return FAILURE;
    }
    if((kpMCB->lpCIB->lConcurrency > 1) && (TRUE == kpMCB->bMessageRate))
    {
        ConsoleMsg(127);
        LogMsg(127);
        kpMCB->lProcessing = FAILURE;
        return FAILURE;
    }

    /* Move up to the concurrency level of tests to the active list */
    lpTIB = kpMCB->lpTIBReadyHead;
    for(lIdx = 0; (lIdx < kpMCB->lpCIB->lConcurrency) && (NULL != lpTIB); lIdx++)
    {
        RemoveFromList((LinkedListBlk**)&kpMCB->lpTIBReadyHead,
                       (LinkedListBlk**)&kpMCB->lpTIBReadyTail,
                       (LinkedListBlk*)lpTIB);
        AddToTailOfList((LinkedListBlk**)&kpMCB->lpTIBActiveHead,
                        (LinkedListBlk**)&kpMCB->lpTIBActiveTail,
                        (LinkedListBlk*)lpTIB);
        InitTimePidAutoVars(lpTIB, lpTIB->lpVIBHead);
        /* Test has been activated log and report the fact */
        LogMsg(35, lpTIB->lTestNum, lpTIB->lpszTestName);
        ReportMsg(35, lpTIB->lTestNum, lpTIB->lpszTestName);
        /* Next test from the head of the chain */
        lpTIB = kpMCB->lpTIBReadyHead;
    }

    return SUCCESS;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     OpenResources                                                        */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Open resources such as connections                                   */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     N/A                                                                  */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     SUCCESS or FAILURE                                                   */
/*==========================================================================*/
long
OpenResources()
{
    ControlInfoBlk*   lpCIB;
    ConnectionDefBlk* lpCDB;
#ifndef __MSGTEST_S
    MQCNO             oConnectOptsDefault = {MQCNO_DEFAULT};
    MQCNO             oConnectOpts;
    MQCD              oClientDefDefault = {MQCD_CLIENT_CONN_DEFAULT};
    MQCD              oClientDef;
    MQSCO             oClientSslDefault = {MQSCO_DEFAULT};
    MQSCO             oClientSsl;
    long              lConnNameLen;
    char              lpszDirAndFile[FILE_BUFF_LEN + 4];
    char              lpszPeerName[MQ_SSL_PEER_NAME_LENGTH + 4];
#endif
    long              bCreatedDefaultCDB;
    long              lMaxMsgLen;
    MQOD              oObjDescDefault = {MQOD_DEFAULT};
    MQOD              oObjDesc;
    MQHOBJ            hObj;
    MQLONG            lSelector[2];
    MQLONG            lIntegerAttrValues[1];
    char              lpszInqQueueManagerName[MQ_Q_MGR_NAME_LENGTH + 4];
    MQLONG            lCloseOpts;
    MQLONG            lCC;
    MQLONG            lRC;

    /* Initialise local variables */
    lpCIB = kpMCB->lpCIB;
    lCC = MQCC_OK;
    lRC = MQRC_NONE;

    /* Get a pointer to the first connection definition block */
    lpCDB = kpMCB->lpCDBHead;

    /* If we don't have a Connection Defintion Block and the script */
    /* requires a connection then create one that will attempt to   */
    /* make a default connection i.e. a blank queue manager name    */
    bCreatedDefaultCDB = FALSE;
    if((NULL == lpCDB) && (TRUE == kpMCB->bConnectionRequired))
    {
        lpCDB = CreateConnectionDefBlk(kpszDEFCONN);
        if(! lpCDB)
        {
            kpMCB->lProcessing = FAILURE;
            return FAILURE;
        }

        bCreatedDefaultCDB = TRUE;
        AddToTailOfList((LinkedListBlk**)&kpMCB->lpCDBHead,
                        (LinkedListBlk**)&kpMCB->lpCDBTail,
                        (LinkedListBlk*)lpCDB);

        lpCDB->lpszConnQMgrName = CloneStringNull(kpszEmptyString);
    }

    while(NULL != lpCDB)
    {
        /* Use defaults to initialise the control blocks */
        memcpy(&oObjDesc, &oObjDescDefault, sizeof(MQOD));
#ifndef __MSGTEST_S
        memcpy(&oConnectOpts, &oConnectOptsDefault, sizeof(MQCNO));
        memcpy(&oClientDef, &oClientDefDefault, sizeof(MQCD));
#endif
        if(0 != memcmp(kpszNONE, lpCDB->lpszConnQMgrName, strlen(kpszNONE) + 1))
        {
#ifndef __MSGTEST_S
            if(FALSE == bCreatedDefaultCDB)
            {
                /* For a client connection prepare the connection options and client definition */
                if(0 < StrlenNull(lpCDB->lpszConnChannelName))
                {
                    strncpy(oClientDef.ChannelName, lpCDB->lpszConnChannelName, MQ_CHANNEL_NAME_LENGTH);
                    if(0 < StrlenNull(lpCDB->lpszConnPortNum))
                    {
                        lConnNameLen = StrlenNull(lpCDB->lpszConnHostName) + StrlenNull(lpCDB->lpszConnPortNum) + 2;
                        if(MQ_CONN_NAME_LENGTH < lConnNameLen)
                        {
                            LogMsg(39, lpCDB->lpszConnHostName, lpCDB->lpszConnPortNum);
                            return FAILURE;
                        }
                        strcpy(oClientDef.ConnectionName, lpCDB->lpszConnHostName);
                        strcat(oClientDef.ConnectionName, "(");
                        strcat(oClientDef.ConnectionName, lpCDB->lpszConnPortNum);
                        strcat(oClientDef.ConnectionName, ")");
                    }
                    else
                    {
                        lConnNameLen = StrlenNull(lpCDB->lpszConnHostName);
                        if(MQ_CONN_NAME_LENGTH < lConnNameLen)
                        {
                            LogMsg(39, lpCDB->lpszConnHostName, kpszEmptyString);
                            return FAILURE;
                        }
                        strcpy(oClientDef.ConnectionName, lpCDB->lpszConnHostName);
                    }
                }

                /* Set the client connection channels maximum message length if supplied */
                if(0 < StrlenNull(lpCDB->lpszConnMaxMsgLength))
                {
                    oClientDef.MaxMsgLength = ConvNum(NULL, lpCDB->lpszConnMaxMsgLength, kpszMAXMSGLENGTH);
                    if(FAILURE == kpMCB->lProcessing)
                    {
                        return FAILURE;
                    }
                }

                oConnectOpts.ClientConnPtr = &oClientDef;
                oConnectOpts.Version = MQCNO_VERSION_2;

                /* If the SSL options KeyRepository, SSLCipherSpec, SSLPeerName have been specified then setup */
                /* the MQSCO object and point the MQCNO connection options structure at it            */
                if((0 < StrlenNull(lpCDB->lpszConnSSLPeerName)) || (0 < StrlenNull(lpCDB->lpszConnKeyRepository)) || (0 < StrlenNull(lpCDB->lpszConnSSLCipherSpec)))
                {
                    /* At least one of the SSL options was specified however we actually need to know KeyRepository and CipherSpec */
                    if((0 == StrlenNull(lpCDB->lpszConnKeyRepository)) || (0 == StrlenNull(lpCDB->lpszConnSSLCipherSpec)))
                    {
                        LogMsg(239);
                        return FAILURE;
                    }

                    /* Generate full path file name for checking the key repository */
                    memset(lpszDirAndFile, 0x00, FILE_BUFF_LEN + 4);
                    strncpy(lpszDirAndFile, lpCDB->lpszConnKeyRepository, FILE_BUFF_LEN);
                    strncat(lpszDirAndFile, ".kdb", FILE_BUFF_LEN); /* Extension for MQ V6 is ".kdb" */
                    if(FAILURE == FileExists(lpszDirAndFile))
                    {
                        /* Try with the MQ V5.3 extension of ".sto" before giving up */
                        memset(lpszDirAndFile, 0x00, FILE_BUFF_LEN + 4);
                        strncpy(lpszDirAndFile, lpCDB->lpszConnKeyRepository, FILE_BUFF_LEN);
                        strncat(lpszDirAndFile, ".sto", FILE_BUFF_LEN); /* Extension for MQ V5.3 is ".sto" */
                        if(FAILURE == FileExists(lpszDirAndFile))
                        {
                            LogMsg(241, lpCDB->lpszConnKeyRepository);
                            return FAILURE;
                        }
                    }

                    /* Set up the MQSCO control block and set the additonal control block chain MQCNO->MQSCO */
                    memcpy(&oClientSsl, &oClientSslDefault, sizeof(MQSCO));
                    strncpy(oClientSsl.KeyRepository, lpCDB->lpszConnKeyRepository, MQ_SSL_KEY_REPOSITORY_LENGTH); 
                    strncpy(oClientSsl.CryptoHardware," ", MQ_SSL_CRYPTO_HARDWARE_LENGTH); 

                    /* Set up the extra values in the MQCD */
                    oClientDef.Version = MQCD_VERSION_7;
                    strncpy(oClientDef.SSLCipherSpec, lpCDB->lpszConnSSLCipherSpec, MQ_SSL_CIPHER_SPEC_LENGTH);
                    memset(lpszPeerName, 0x00, MQ_SSL_PEER_NAME_LENGTH + 4);
                    strncpy(lpszPeerName, lpCDB->lpszConnSSLPeerName, MQ_SSL_PEER_NAME_LENGTH);
                    if(0 == StrlenNull(lpszPeerName))
                    {
                        oClientDef.SSLPeerNamePtr = NULL;
                        oClientDef.SSLPeerNameLength = 0;
                    }
                    else
                    {
                        oClientDef.SSLPeerNamePtr = &lpszPeerName;
                        oClientDef.SSLPeerNameLength = StrlenNull(lpszPeerName);
                    }
                    oClientDef.SSLClientAuth = 0;

                    /* For SSL CNO needs to be Version 4 */
                    oConnectOpts.Version = MQCNO_VERSION_4;

                    /* Establish MQCNO->MQSCO pointer    */
                    oConnectOpts.SSLConfigPtr = &oClientSsl ; 
                }
            }
#endif

            /* Now attempt to connect to the queue manager */
#ifdef __MSGTEST_S
            /* Simple server connection */
            MQCONN(lpCDB->lpszConnQMgrName, &lpCDB->hConn, &lCC, &lRC);
#else
            if(TRUE == bCreatedDefaultCDB)
            {
                /* Basic client connection */
                MQCONN(lpCDB->lpszConnQMgrName, &lpCDB->hConn, &lCC, &lRC);
            }
            else
            {
                /* Client connection with all the options */
                MQCONNX(lpCDB->lpszConnQMgrName, &oConnectOpts, &lpCDB->hConn, &lCC, &lRC);
            }
            /* MQRC_SSL_ALREADY_INITIALIZED                                                                   */
            /*   The SSL information can be specified only on:                                                */
            /*   The first MQCONNX call of the client process, or a subsequent MQCONNX call when all previous */
            /*   connections to the queue manager (using any protocol) have been concluded using MQDISC.      */
            /*                                                                                                */
            /*   These are the only states in which the process-wide SSL environment can be initialised.      */
            /*   If an MQCONNX call is issued specifying SSL information when the SSL environment already     */
            /*   exists, the SSL information on the call is ignored and the connection is made using the      */
            /*   existing SSL environment; the call returns completion code MQCC_WARNING and reason code      */
            /*   MQRC_SSL_ALREADY_INITIALIZED in this case.                                                   */
#endif

            /* Check the result */
            if(MQRC_NONE == lRC)
            {
                if(0 == StrlenNull(lpCDB->lpszConnQMgrName))
                {
                    ConsoleMsg(260, lCC, lRC);
                    LogMsg(260, lCC, lRC);
                }
                else
                {
                    ConsoleMsg(40, lpCDB->lpszConnQMgrName, lCC, lRC);
                    LogMsg(40, lpCDB->lpszConnQMgrName, lCC, lRC);
                }
                lpCDB->bWasAlreadyConnected = FALSE;
            }
            else if(MQRC_SSL_ALREADY_INITIALIZED == lRC)
            {
                LogMsg(240, lpCDB->lpszConnQMgrName, lCC, lRC);
            }
            else if(MQRC_ALREADY_CONNECTED == lRC)
            {
                /* Issue a warning message and set a flag so we */
                /* don't try to disconnect twice                */
                LogMsg(270, lpCDB->lpszConnQMgrName, lCC, lRC);
                lpCDB->bWasAlreadyConnected = FALSE;
            }
            else
            {
                if(0 == StrlenNull(lpCDB->lpszConnQMgrName))
                {
                    LogMsg(261, lCC, lRC);
                }
                else
                {
                    LogMsg(41, lpCDB->lpszConnQMgrName, lCC, lRC);
                }
                return FAILURE;
            }

            /* Now open the queue manager object for inquire */
            oObjDesc.ObjectType = MQOT_Q_MGR;
            MQOPEN(lpCDB->hConn, &oObjDesc,
                   MQOO_INQUIRE + MQOO_FAIL_IF_QUIESCING,
                   &hObj, &lCC, &lRC);
            if(MQRC_NONE == lRC)
            {
                if(0 == StrlenNull(lpCDB->lpszConnQMgrName))
                {
                    LogMsg(262, lCC, lRC);
                }
                else
                {
                    LogMsg(211, lpCDB->lpszConnQMgrName, lCC, lRC);
                }
            }
            else
            {
                if(0 == StrlenNull(lpCDB->lpszConnQMgrName))
                {
                    LogMsg(263, lCC, lRC);
                }
                else
                {
                    LogMsg(212, lpCDB->lpszConnQMgrName, lCC, lRC);
                }
                return FAILURE;
            }

            /* Get the CCSID and Name of the queue manager */
            lSelector[0] = MQIA_CODED_CHAR_SET_ID;
            lSelector[1] = MQCA_Q_MGR_NAME;
            memset(lpszInqQueueManagerName, 0x00, MQ_Q_MGR_NAME_LENGTH + 4);
            MQINQ(lpCDB->hConn, hObj, 2, lSelector,
                  1, lIntegerAttrValues,
                  MQ_Q_MGR_NAME_LENGTH, lpszInqQueueManagerName, &lCC, &lRC);
            if(MQRC_NONE == lRC)
            {
                lpCDB->lpszConnQMgrName = TrimString(lpszInqQueueManagerName);
                if(0 == StrlenNull(lpCDB->lpszConnQMgrName))
                {
                    LogMsg(264, lCC, lRC);
                }
                else
                {
                    LogMsg(213, lpCDB->lpszConnQMgrName, lCC, lRC);
                }
            }
            else
            {
                if(0 == StrlenNull(lpCDB->lpszConnQMgrName))
                {
                    LogMsg(265, lCC, lRC);
                }
                else
                {
                    LogMsg(214, lpCDB->lpszConnQMgrName, lCC, lRC);
                }
                return FAILURE;
            }

            /* Got the CCSID */
            lpCDB->lCCSID = lIntegerAttrValues[0];
            LogMsg(215, lpCDB->lpszConnQMgrName, lpCDB->lCCSID);

            /* Finished our inquiries of the queue manager */
            lCloseOpts = MQCO_NONE;
            MQCLOSE(lpCDB->hConn, &hObj, lCloseOpts, &lCC, &lRC);
            if(MQRC_NONE == lRC)
            {
                LogMsg(216, lpCDB->lpszConnQMgrName, lCC, lRC);
            }
            else
            {
                LogMsg(217, lpCDB->lpszConnQMgrName, lCC, lRC);
                return FAILURE;
            }

            /* If an MqscCommand has been used in the script then get the */
            /* maximum message length for reply buffer allocation         */
            if(TRUE == kpMCB->bScriptHasMqscCommand)
            {
                memcpy(&oObjDesc, &oObjDescDefault, sizeof(MQOD));
                strncpy(oObjDesc.ObjectName, kpszSYSMQSCREPLYQ, MQ_Q_NAME_LENGTH);
                MQOPEN(lpCDB->hConn, &oObjDesc,
                       MQOO_INQUIRE + MQOO_FAIL_IF_QUIESCING,
                       &hObj, &lCC, &lRC);
                if(MQRC_NONE == lRC)
                {
                    LogMsg(242, kpszSYSMQSCREPLYQ, lpCDB->lpszConnQMgrName, lCC, lRC);
                }
                else
                {
                    LogMsg(243, kpszSYSMQSCREPLYQ, lpCDB->lpszConnQMgrName, lCC, lRC);
                    return FAILURE;
                }

                /* Get the MaximumMessageLength of the MQSC command reply model queue */
                lSelector[0] = MQIA_MAX_MSG_LENGTH;
                MQINQ(lpCDB->hConn, hObj, 1, lSelector,
                      1, lIntegerAttrValues,
                      0, NULL, &lCC, &lRC);
                if(MQRC_NONE == lRC)
                {
                    LogMsg(244, kpszSYSMQSCREPLYQ, lpCDB->lpszConnQMgrName, lCC, lRC);
                }
                else
                {
                    LogMsg(245, kpszSYSMQSCREPLYQ, lpCDB->lpszConnQMgrName, lCC, lRC);
                    return FAILURE;
                }
                /* Got the MaxMsgLen */
                lMaxMsgLen = lIntegerAttrValues[0];
                if(kpMCB->lMqscReplyBuffLen < lMaxMsgLen)
                {
                    kpMCB->lMqscReplyBuffLen = lMaxMsgLen;
                }
                LogMsg(246, lpCDB->lpszConnQMgrName, kpszSYSMQSCREPLYQ, lMaxMsgLen);

                /* Finished our inquiries of the MQSC command reply model queue */
                lCloseOpts = MQCO_NONE;
                MQCLOSE(lpCDB->hConn, &hObj, lCloseOpts, &lCC, &lRC);
                if(MQRC_NONE == lRC)
                {
                    LogMsg(247, kpszSYSMQSCREPLYQ, lpCDB->lpszConnQMgrName, lCC, lRC);
                }
                else
                {
                    LogMsg(248, kpszSYSMQSCREPLYQ, lpCDB->lpszConnQMgrName, lCC, lRC);
                    return FAILURE;
                }
            }
        }
        else
        {
            LogMsg(70);
        }
        lpCDB = (ConnectionDefBlk*)lpCDB->lpCDBNext;
    }

    /* Allocate the MQSC command reply buffer if required */
    if(0 < kpMCB->lMqscReplyBuffLen)
    {
        kpMCB->lpszMqscReplyBuff = malloc(kpMCB->lMqscReplyBuffLen + 8);
        if(! kpMCB->lpszMqscReplyBuff)
        {
            LogMsg(3, kpMCB->lMqscReplyBuffLen + 8, "Mqsc Command Reply Buffer", __LINE__, __FILE__);
            return FAILURE;
        }
        memset(kpMCB->lpszMqscReplyBuff, 0x00, kpMCB->lMqscReplyBuffLen + 8);
    }

    return SUCCESS;
}

/*==========================================================================*/
/* FUNCTION:                                                                */
/*     InitScriptAutoVars                                                   */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Set the values of the script level automatic variables               */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     argv    Pointer to list of argument pointers supplied to the program */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
InitScriptAutoVars(char** argv)
{
    char*       lpszScriptName;
    char*       lpszDot;
    char*       lpszScriptNameClone;
    time_t          oCurrTime;

    /* *********************** */
    /* Process the script name */
    /* *********************** */
    kpMCB->lpszScriptFileName = argv[1];

    /* Find the last backslash\slash char and assume it ends the directory path */
    lpszScriptName = strrchr(argv[1], *kpszDirFileConcat);
    if(NULL == lpszScriptName)
    {
        lpszScriptName = argv[1];
    }
    else
    {
        lpszScriptName++;
    }

    /* Change the last full stop to \0 to truncate an assumed file extenstion */
    lpszScriptNameClone = CloneString(lpszScriptName);
    lpszDot = strrchr(lpszScriptNameClone, *kpszDOT);
    if(NULL != lpszDot)
    {
        *lpszDot = '\0';
    }
    kpMCB->lpszScriptName = CloneString(lpszScriptNameClone);
    FreeIfNotNull((void**)&lpszScriptNameClone);

    /* ****************************************** */
    /* Setup the script level automatic variables */
    /* ****************************************** */
    AddAutoStrVar(&kpMCB->lpVIBHead, &kpMCB->lpVIBTail, kpszYEAR4);
    AddAutoStrVar(&kpMCB->lpVIBHead, &kpMCB->lpVIBTail, kpszYEAR2);
    AddAutoStrVar(&kpMCB->lpVIBHead, &kpMCB->lpVIBTail, kpszMONTH3);
    AddAutoStrVar(&kpMCB->lpVIBHead, &kpMCB->lpVIBTail, kpszMONTH2);
    AddAutoStrVar(&kpMCB->lpVIBHead, &kpMCB->lpVIBTail, kpszDAY3);
    AddAutoStrVar(&kpMCB->lpVIBHead, &kpMCB->lpVIBTail, kpszDAY2);
    AddAutoStrVar(&kpMCB->lpVIBHead, &kpMCB->lpVIBTail, kpszHOURS);
    AddAutoStrVar(&kpMCB->lpVIBHead, &kpMCB->lpVIBTail, kpszMINUTES);
    AddAutoStrVar(&kpMCB->lpVIBHead, &kpMCB->lpVIBTail, kpszSECONDS);
    AddAutoStrVar(&kpMCB->lpVIBHead, &kpMCB->lpVIBTail, kpszPID);
    AddAutoStrVar(&kpMCB->lpVIBHead, &kpMCB->lpVIBTail, kpszSCRIPTNAME);

    /* Get the current time value */
    time(&oCurrTime);

    /* Set the script level time related values */
    SetAutoVar(NULL, kpMCB->lpVIBHead, kpszYEAR4, &oCurrTime);
    SetAutoVar(NULL, kpMCB->lpVIBHead, kpszYEAR2, &oCurrTime);
    SetAutoVar(NULL, kpMCB->lpVIBHead, kpszMONTH3, &oCurrTime);
    SetAutoVar(NULL, kpMCB->lpVIBHead, kpszMONTH2, &oCurrTime);
    SetAutoVar(NULL, kpMCB->lpVIBHead, kpszDAY3, &oCurrTime);
    SetAutoVar(NULL, kpMCB->lpVIBHead, kpszDAY2, &oCurrTime);
    SetAutoVar(NULL, kpMCB->lpVIBHead, kpszHOURS, &oCurrTime);
    SetAutoVar(NULL, kpMCB->lpVIBHead, kpszMINUTES, &oCurrTime);
    SetAutoVar(NULL, kpMCB->lpVIBHead, kpszSECONDS, &oCurrTime);
    SetAutoVar(NULL, kpMCB->lpVIBHead, kpszPID, NULL);
    SetAutoVar(NULL, kpMCB->lpVIBHead, kpszSCRIPTNAME, NULL);
}
