/*==========================================================================*/
/* MsgTestShut.c contains the program termination and cleanup routines.     */
/*==========================================================================*/
/* (C) Copyright Tim Armstrong 2003, 2004, 2005, 2006, 2007, 2008           */
/*==========================================================================*/
/* Includes                                                                 */
/*==========================================================================*/
#ifndef __TJAINC_MSGTEST_H
#include "MsgTest.h"
#endif /* __TJAINC_MSGTEST_H */


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeResources                                                        */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Close open resources such as connections, queues, files etc.         */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     N/A                                                                  */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeResources()
{
    VarInfoBlk*        lpVIB;
    VarInfoBlk*        lpVIBFree;
    ConnectionDefBlk*  lpCDB;
    ConnectionDefBlk*  lpCDBFree;
    MsgSetHdr*         lpMSH;
    MsgSetHdr*         lpMSHFree;
    IntervalInfoBlk*   lpIIB;
    IntervalInfoBlk*   lpIIBFree;
    ForBackRefBlk*     lpFBR;
    ForBackRefBlk*     lpFBRFree;
    IfBackRefBlk*      lpIFB;
    IfBackRefBlk*      lpIFBFree;
    MsgSetFileLoadBlk* lpMSFL;
    MsgSetFileLoadBlk* lpMSFLFree;
    MQLONG             lCC;
    MQLONG             lRC;
    long               lIdx;
    time_t             oTimeT;
    struct tm*         poTimeTm ;

    /* Initialise local variables */
    lCC = MQCC_OK;
    lRC = MQRC_NONE;

    /* And get the time we ended */
    time(&oTimeT);
    poTimeTm = localtime(&oTimeT);
    strftime(kpMCB->lpszTimeBuff, LOG_BUFF_LEN, "%H:%M:%S", poTimeTm );

    /* Free var info blocks if any */
    lpVIB = kpMCB->lpVIBHead;
    while(NULL != lpVIB)
    {
        lpVIBFree = lpVIB;
        lpVIB = (VarInfoBlk*)lpVIB->lpVIBNext;
        FreeVIB(&lpVIBFree);
    }

    /* Free connection definition blocks */
    lpCDB = kpMCB->lpCDBHead;
    while(NULL != lpCDB)
    {
        /* Release the connection */
        if((lpCDB->hConn) && (FALSE == lpCDB->bWasAlreadyConnected))
        {
            /* Backout any uncommitted units of work on this connection */
            MQBACK(lpCDB->hConn, &lCC, &lRC);
            if(MQRC_NONE == lRC)
            {
                LogMsg(196, lpCDB->lpszConnQMgrName, lCC, lRC);
            }
            else
            {
                LogMsg(197, lpCDB->lpszConnQMgrName, lCC, lRC);
            }
            /* Now disconnect from the queue manager */
            MQDISC(&lpCDB->hConn, &lCC, &lRC);
            if(MQRC_NONE == lRC)
            {
                LogMsg(42, lpCDB->lpszConnQMgrName, lCC, lRC);
            }
            else
            {
                LogMsg(43, lpCDB->lpszConnQMgrName, lCC, lRC);
            }
        }
        lpCDBFree = lpCDB;
        lpCDB = (ConnectionDefBlk*)lpCDB->lpCDBNext;
        FreeCDB(&lpCDBFree);
    }

    /* Free message set related blocks */
    lpMSH = kpMCB->lpMSHHead;
    while(NULL != lpMSH)
    {
        lpMSHFree = lpMSH;
        lpMSH = (MsgSetHdr*)lpMSH->lpMSHNext;
        FreeMSH(&lpMSHFree);
    }

    /* Free any Interval Info blocks */
    lpIIB = kpMCB->lpIIBHead;
    while(NULL != lpIIB)
    {
        lpIIBFree = lpIIB;
        lpIIB = (IntervalInfoBlk*)lpIIB->lpIIBNext;
        FreeIIB(&lpIIBFree);
    }

    /* Free for loop back reference blocks if any */
    lpFBR = kpMCB->lpFBRHead;
    while(NULL != lpFBR)
    {
        lpFBRFree = lpFBR;
        lpFBR = (ForBackRefBlk*)lpFBR->lpFBRNext;
        FreeFBR(&lpFBRFree);
    }

    /* Free if back reference blocks if any */
    lpIFB = kpMCB->lpIFBHead;
    while(NULL != lpIFB)
    {
        lpIFBFree = lpIFB;
        lpIFB = (IfBackRefBlk*)lpIFB->lpIFBNext;
        FreeIFB(&lpIFBFree);
    }

    /* If we have a defaults info block clean up its resources */
    if(kpMCB->lpDIB)
    {
        /* Free var info blocks if any */
        lpVIB = kpMCB->lpDIB->lpVIBHead;
        while(NULL != lpVIB)
        {
            lpVIBFree = lpVIB;
            lpVIB = (VarInfoBlk*)lpVIB->lpVIBNext;
            FreeVIB(&lpVIBFree);
        }

        /* Clean up the various default command blocks if present */
        if(kpMCB->lpDIB->lpGFB)
        {
            FreeCCB((CommandCtrlBlk**)&kpMCB->lpDIB->lpGFB);
        }
        if(kpMCB->lpDIB->lpPFB)
        {
            FreeCCB((CommandCtrlBlk**)&kpMCB->lpDIB->lpPFB);
        }
        if(kpMCB->lpDIB->lpIDB)
        {
            FreeCCB((CommandCtrlBlk**)&kpMCB->lpDIB->lpIDB);
        }
        if(kpMCB->lpDIB->lpOVB)
        {
            FreeCCB((CommandCtrlBlk**)&kpMCB->lpDIB->lpOVB);
        }
        if(kpMCB->lpDIB->lpEXB)
        {
            FreeCCB((CommandCtrlBlk**)&kpMCB->lpDIB->lpEXB);
        }
        if(kpMCB->lpDIB->lpSSB)
        {
            FreeCCB((CommandCtrlBlk**)&kpMCB->lpDIB->lpSSB);
        }
        if(kpMCB->lpDIB->lpGMB)
        {
            FreeCCB((CommandCtrlBlk**)&kpMCB->lpDIB->lpGMB);
        }
        if(kpMCB->lpDIB->lpPMB)
        {
            FreeCCB((CommandCtrlBlk**)&kpMCB->lpDIB->lpPMB);
        }
        if(kpMCB->lpDIB->lpSLB)
        {
            FreeCCB((CommandCtrlBlk**)&kpMCB->lpDIB->lpSLB);
        }
        if(kpMCB->lpDIB->lpWDB)
        {
            FreeCCB((CommandCtrlBlk**)&kpMCB->lpDIB->lpWDB);
        }
        if(kpMCB->lpDIB->lpWTB)
        {
            FreeCCB((CommandCtrlBlk**)&kpMCB->lpDIB->lpWTB);
        }
        if(kpMCB->lpDIB->lpCMB)
        {
            FreeCCB((CommandCtrlBlk**)&kpMCB->lpDIB->lpCMB);
        }
        if(kpMCB->lpDIB->lpCOB)
        {
            FreeCCB((CommandCtrlBlk**)&kpMCB->lpDIB->lpCOB);
        }
        if(kpMCB->lpDIB->lpBOB)
        {
            FreeCCB((CommandCtrlBlk**)&kpMCB->lpDIB->lpBOB);
        }
        if(kpMCB->lpDIB->lpSYB)
        {
            FreeCCB((CommandCtrlBlk**)&kpMCB->lpDIB->lpSYB);
        }

        /* Free the DIB itself */
        FreeIfNotNull((void**)&kpMCB->lpDIB);
    }

    /* If we have a control info block clean up its resources */
    /* and any MCB resources that require its existence       */
    if(kpMCB->lpCIB)
    {
        /* Close the log file */
        if(kpMCB->lpCIB->lpTestLogFile)
        {
            if(TRUE == kpMCB->bInitComplete)
            {
                if(TRUE == kpMCB->bScriptHasGetFile)
                {
                    LogMsg(163, kpMCB->lScriptTotalGetFile,
                           kpMCB->lScriptSuccessfulGetFile,
                           kpMCB->lScriptFailedGetFile);
                }
                if(TRUE == kpMCB->bScriptHasPutFile)
                {
                    LogMsg(165, kpMCB->lScriptTotalPutFile,
                           kpMCB->lScriptSuccessfulPutFile,
                           kpMCB->lScriptFailedPutFile);
                }
                if(TRUE == kpMCB->bScriptHasGetMsg)
                {
                    LogMsg(167, kpMCB->lScriptTotalGetMsg,
                           kpMCB->lScriptSuccessfulGetMsg,
                           kpMCB->lScriptFailedGetMsg);
                }
                if(TRUE == kpMCB->bScriptHasPutMsg)
                {
                    LogMsg(169, kpMCB->lScriptTotalPutMsg,
                           kpMCB->lScriptSuccessfulPutMsg,
                           kpMCB->lScriptFailedPutMsg);
                }
                if(TRUE == kpMCB->bScriptHasCompare)
                {
                    LogMsg(171, kpMCB->lScriptTotalCompare,
                           kpMCB->lScriptSuccessfulCompare,
                           kpMCB->lScriptFailedCompare);
                }
                LogMsg(62, kpMCB->lTotalTests,
                       kpMCB->lSuccessfulTests,
                       kpMCB->lFailedTests,
                       kpMCB->lIncompleteTests);
                LogMsg(63, kpMCB->lWaitTotal);
            }
            LogMsg(2, kpMCB->lpszTimeBuff);
            fflush(kpMCB->lpCIB->lpTestLogFile);
            fclose(kpMCB->lpCIB->lpTestLogFile);
        }

        /* Close the report file */
        if(kpMCB->lpCIB->lpTestReportFile)
        {
            if(FAILURE == kpMCB->lProcessing)
            {
                ReportMsg(32);
            }
            if(TRUE == kpMCB->bInitComplete)
            {
                if(TRUE == kpMCB->bScriptHasGetFile)
                {
                    ReportMsg(163, kpMCB->lScriptTotalGetFile,
                              kpMCB->lScriptSuccessfulGetFile,
                              kpMCB->lScriptFailedGetFile);
                }
                if(TRUE == kpMCB->bScriptHasPutFile)
                {
                    ReportMsg(165, kpMCB->lScriptTotalPutFile,
                              kpMCB->lScriptSuccessfulPutFile,
                              kpMCB->lScriptFailedPutFile);
                }
                if(TRUE == kpMCB->bScriptHasGetMsg)
                {
                    ReportMsg(167, kpMCB->lScriptTotalGetMsg,
                              kpMCB->lScriptSuccessfulGetMsg,
                              kpMCB->lScriptFailedGetMsg);
                }
                if(TRUE == kpMCB->bScriptHasPutMsg)
                {
                    ReportMsg(169, kpMCB->lScriptTotalPutMsg,
                              kpMCB->lScriptSuccessfulPutMsg,
                              kpMCB->lScriptFailedPutMsg);
                }
                if(TRUE == kpMCB->bScriptHasCompare)
                {
                    ReportMsg(171, kpMCB->lScriptTotalCompare,
                              kpMCB->lScriptSuccessfulCompare,
                              kpMCB->lScriptFailedCompare);
                }
                ReportMsg(62, kpMCB->lTotalTests,
                          kpMCB->lSuccessfulTests,
                          kpMCB->lFailedTests,
                          kpMCB->lIncompleteTests);
                ReportMsg(63, kpMCB->lWaitTotal);
            }
            ReportMsg(2, kpMCB->lpszTimeBuff);
            fflush(kpMCB->lpCIB->lpTestReportFile);
            fclose(kpMCB->lpCIB->lpTestReportFile);
        }

        /* Close the user log file */
        if(kpMCB->lpCIB->lpTestUserLogFile)
        {
            UserLogMsg(2, kpMCB->lpszTimeBuff);
            fflush(kpMCB->lpCIB->lpTestUserLogFile);
            fclose(kpMCB->lpCIB->lpTestUserLogFile);
        }

        /* Free control info block strings */
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszConnQMgrName);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszConnChannelName);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszConnHostName);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszConnPortNum);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszConnKeyRepository);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszConnSSLCipherSpec);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszConnSSLPeerName);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszConnMaxMsgLength);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszTestLogFileName);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszTestLogDir);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszTestReportFileName);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszTestReportDir);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszTestUserLogFileName);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszTestUserLogDir);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszTestIntervalFilePrefix);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszTestIntervalDelimiter);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszTestIntervalFileExtension);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszTestIntervalDir);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszOnError);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszConvTableDir);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszLocalEncoding);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszLocalCodedCharSetId);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszFilePrefix);
        FreeIfNotNull((void**)&kpMCB->lpCIB->lpszParserName);

        /* Free any message set file load blocks */
        lpMSFL = kpMCB->lpCIB->lpMSFLHead;
        while(NULL != lpMSFL)
        {
            lpMSFLFree = lpMSFL;
            lpMSFL = (MsgSetFileLoadBlk*)lpMSFL->lpMSFLNext;
            FreeMSFL(&lpMSFLFree);
        }

        /* And finally the CIB itself */
        FreeIfNotNull((void**)&kpMCB->lpCIB);
    }

    /* Free control blocks associated with the secondary parse block */
    if(kpMCB->lpSPB)
    {
        FreeIfNotNull((void**)&kpMCB->lpSPB->lpszCtrlBlkFileName);
        FreeIfNotNull((void**)&kpMCB->lpSPB->lpszCtrlBlkBuff);
        
        /* Free element name buffers for secondary parse block */
        for(lIdx = 0; lIdx < MAX_CTRLBLK_PARSE_DEPTH + 1; lIdx++)
        {
            FreeIfNotNull((void**)&kpMCB->lpSPB->lplpszElementName[lIdx]);
        }

        /* Free the secondary parse block itself */
        FreeIfNotNull((void**)&kpMCB->lpSPB);
    }

    /* Free control blocks associated with the message set parse block */
    if(kpMCB->lpMSPB)
    { 
        FreeIfNotNull((void**)&kpMCB->lpMSPB->lpszMsgSetBuff);
        
        /* Free element name buffers for message set parse block */
        for(lIdx = 0; lIdx < MAX_MSGSET_PARSE_DEPTH + 1; lIdx++)
        {
            FreeIfNotNull((void**)&kpMCB->lpMSPB->lplpszElementName[lIdx]);
        }

        /* Free the message set parse block itself */
        FreeIfNotNull((void**)&kpMCB->lpMSPB);
    }

    /* Free the CSE control block */
    if(kpMCB->lpCSE)
    {
        /* Call the clear CSE function to release any strings */
        ClearCSE();

        /* Free the command sub element block itself */
        FreeIfNotNull((void**)&kpMCB->lpCSE);
    }

    /* Call the clear ParseCharDataBuff function to release any strings */
    ClearPCDB();

    /* Free the MCB's List info blocks */
    if(kpMCB->lpLIB)
    {
        FreeLIB(&kpMCB->lpLIB);
    }
    if(kpMCB->lpNullLIB)
    {
        FreeLIB(&kpMCB->lpNullLIB);
    }

    /* Free the MCB's MQMD info blocks */
    if(kpMCB->lpMIB)
    {
        FreeMIB(&kpMCB->lpMIB);
    }
    if(kpMCB->lpNullMIB)
    {
        FreeMIB(&kpMCB->lpNullMIB);
    }

    /* Free the MCB's RFH2 info blocks */
    if(kpMCB->lpRIB)
    {
        FreeRIB(&kpMCB->lpRIB);
    }
    if(kpMCB->lpNullRIB)
    {
        FreeRIB(&kpMCB->lpNullRIB);
    }

    /* Free element name buffers */
    for(lIdx = 0; lIdx < MAX_SCRIPT_PARSE_DEPTH + 1; lIdx++)
    {
        FreeIfNotNull((void**)&kpMCB->lplpszElementName[lIdx]);
    }

    /* Free current Name attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrNameAttr);

    /* Free current Format attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrFormatAttr);

    /* Free current OutputFormat attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrOutputFormatAttr);

    /* Free current MQMD attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrMQMDAttr);

    /* Free current RFH2 attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrRFH2Attr);

    /* Free current R1 attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrR1Attr);

    /* Free current R2 attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrR2Attr);

    /* Free current From attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrFromAttr);

    /* Free current To attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrToAttr);

    /* Free current Incr attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrIncrAttr);

    /* Free current Cond attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrCondAttr);

    /* Free current Mode attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrModeAttr);

    /* Free current Interval attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrIntervalAttr);

    /* Free current Pos attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrPosAttr);

    /* Free current Len attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrLenAttr);

    /* Free current Type attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrTypeAttr);

    /* Free current MsgSep attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrMsgSepAttr);

    /* Free current SegSep attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrSegSepAttr);

    /* Free current ElementSep attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrElementSepAttr);

    /* Free current MinOccurs attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrMinOccursAttr);

    /* Free current MaxOccurs attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrMaxOccursAttr);

    /* Free current CCSID attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrCCSIDAttr);

    /* Free current Encoding attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrEncodingAttr);

    /* Free current Field attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrMsgSetNameAttr);

    /* Free current Equal attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrMsgNumAttr);

    /* Free current Offset attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrOffsetAttr);

    /* Free current Justify attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrJustifyAttr);

    /* Free current PadChar attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrPadCharAttr);

    /* Free current EndAlign attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrEndAlignAttr);

    /* Free current CmqValues attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrCmqValuesAttr);

    /* Free current Truncate attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrTruncateAttr);

    /* Free current List attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrListAttr);

    /* Free current Data Variable attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrDataVarAttr);

    /* Free current Connection attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrConnectionAttr);

    /* Free current Of attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrOfAttr);

    /* Free current In attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrInAttr);

    /* Free current Delimiter attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrDelimiterAttr);

    /* Free current FileExtension attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrFileExtensionAttr);

    /* Free current Rem attribute buffer */
    FreeIfNotNull((void**)&kpMCB->lpszCurrRemAttr);

    /* Free the script buffer */
    FreeIfNotNull((void**)&kpMCB->lpszScriptBuff);

    /* Free the MQSC command reply buffer */
    FreeIfNotNull((void**)&kpMCB->lpszMqscReplyBuff);

    /* All done */
    if(TRUE == kpMCB->bInitComplete)
    {
        ConsoleMsg(62, kpMCB->lTotalTests,
                   kpMCB->lSuccessfulTests,
                   kpMCB->lFailedTests,
                   kpMCB->lIncompleteTests);
    }
    ConsoleMsg(2, kpMCB->lpszTimeBuff);

    return;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeBIB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an buff info block and its contents                             */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpBIB Pointer to pointer to the buff info block                    */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeBIB(BuffInfoBlk** lplpBIB)
{
    /* Free the buffer name string */
    FreeIfNotNull((void**)&(*lplpBIB)->lpszBuffName);

    /* Free the buffer */
    FreeIfNotNull((void**)&(*lplpBIB)->lpszBuff);

    /* Finally free the buff info block itself */
    FreeIfNotNull((void**)lplpBIB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeCDB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an connection definition block and its contents                 */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpCDB Pointer to pointer to the connection definition block        */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeCDB(ConnectionDefBlk** lplpCDB)
{
    FreeIfNotNull((void**)&(*lplpCDB)->lpszConnQMgrName);
    FreeIfNotNull((void**)&(*lplpCDB)->lpszConnChannelName);
    FreeIfNotNull((void**)&(*lplpCDB)->lpszConnHostName);
    FreeIfNotNull((void**)&(*lplpCDB)->lpszConnPortNum);
    FreeIfNotNull((void**)&(*lplpCDB)->lpszConnKeyRepository);
    FreeIfNotNull((void**)&(*lplpCDB)->lpszConnSSLCipherSpec);
    FreeIfNotNull((void**)&(*lplpCDB)->lpszConnSSLPeerName);
    FreeIfNotNull((void**)&(*lplpCDB)->lpszConnMaxMsgLength);

    /* Free the connection definition block itself */
    FreeIfNotNull((void**)lplpCDB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeFIB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an Folder info block and its contents                           */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpFIB Pointer to pointer to the folder info block                  */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeFIB(FolderInfoBlk** lplpFIB)
{
    /* Free the folder data strings */
    FreeIfNotNull((void**)&(*lplpFIB)->lpszFolderName);
    FreeIfNotNull((void**)&(*lplpFIB)->lpszFolderData);

    /* Free the Folder info block itself */
    FreeIfNotNull((void**)lplpFIB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeFBR                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free a for loop back reference block                                 */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpFBR Pointer to pointer to the loop back reference block          */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeFBR(ForBackRefBlk** lplpFBR)
{
    /* Free the back reference block itself */
    FreeIfNotNull((void**)lplpFBR);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeIFB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free a if back reference block                                       */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpIFB Pointer to pointer to the back reference block               */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeIFB(IfBackRefBlk** lplpIFB)
{
    /* Free the back reference block itself */
    FreeIfNotNull((void**)lplpIFB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeLEB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an ListElement info block and its contents                      */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpLEB Pointer to pointer to the list element info block            */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeLEB(ListElementInfoBlk** lplpLEB)
{
    /* Free the ListElement value string */
    FreeIfNotNull((void**)&(*lplpLEB)->lpszListElementValue);

    /* Free the ListElement info block itself */
    FreeIfNotNull((void**)lplpLEB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeLIB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an List info block and its contents                             */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpLIB Pointer to pointer to the list info block                    */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeLIB(ListInfoBlk** lplpLIB)
{
    /* Call the LIB clear function */
    ClearLIB(*lplpLIB);
    
    /* Free the List info block itself */
    FreeIfNotNull((void**)lplpLIB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeMIB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an MQMD info block and its contents                             */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpMIB Pointer to pointer to the MQMD info block                    */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeMIB(MQMDInfoBlk** lplpMIB)
{
    /* Free the MQMD info strings */
    ClearMIB(*lplpMIB);

    /* Free the MQMD info block itself */
    FreeIfNotNull((void**)lplpMIB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeMSH                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an message set header block and its contents                    */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpMSH Pointer to pointer to the message set header block           */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeMSH(MsgSetHdr** lplpMSH)
{
    MsgDefBlk*     lpMDB;
    MsgDefBlk*     lpMDBFree;
    SegmentDefBlk* lpSDB;
    SegmentDefBlk* lpSDBFree;

    FreeIfNotNull((void**)&(*lplpMSH)->lpszMsgSetName);

    /* Free the message set trees */
    /* ??? TA todo */

    /* Free any associated message definition blocks */
    lpMDB = (*lplpMSH)->lpMDBHead;
    while(NULL != lpMDB)
    {
        lpMDBFree = lpMDB;
        lpMDB = (MsgDefBlk*)lpMDB->lpMDBNext;
        FreeMDB(&lpMDBFree);
    }

    /* Free any associated segment definition blocks */
    lpSDB = (*lplpMSH)->lpSDBHead;
    while(NULL != lpSDB)
    {
        lpSDBFree = lpSDB;
        lpSDB = (SegmentDefBlk*)lpSDB->lpSDBNext;
        FreeSDB(&lpSDBFree);
    }

    /* Free the message set header block itself */
    FreeIfNotNull((void**)lplpMSH);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeMDB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an message definition block and its contents                    */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpMDB Pointer to pointer to the message definition block           */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeMDB(MsgDefBlk** lplpMDB)
{
    MsgSetLinkedListBlk* lpMSLL;
    MsgSetLinkedListBlk* lpMSLLFree;

    /* Free the folder data strings */
    FreeIfNotNull((void**)&(*lplpMDB)->lpszType);
    FreeIfNotNull((void**)&(*lplpMDB)->lpszOutputFormat);
    FreeIfNotNull((void**)&(*lplpMDB)->lpszElementSep);
    FreeIfNotNull((void**)&(*lplpMDB)->lpszSegSep);
    FreeIfNotNull((void**)&(*lplpMDB)->lpszMsgSep);
    FreeIfNotNull((void**)&(*lplpMDB)->lpszCCSID);
    FreeIfNotNull((void**)&(*lplpMDB)->lpszEncoding);

    /* Free any associated segment and element blocks */
    lpMSLL = (*lplpMDB)->lpMSLLHead;
    while(NULL != lpMSLL)
    {
        lpMSLLFree = lpMSLL;
        lpMSLL = (MsgSetLinkedListBlk*)lpMSLL->lpMSLLNext;
        FreeMSLL(&lpMSLLFree);
    }

    /* Free the message definition block itself */
    FreeIfNotNull((void**)lplpMDB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeSDB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an segment definition block and its contents                    */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpSDB Pointer to pointer to the segment definition block           */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeSDB(SegmentDefBlk** lplpSDB)
{
    MsgSetLinkedListBlk* lpMSLL;
    MsgSetLinkedListBlk* lpMSLLFree;

    /* Free the folder data strings */
    FreeIfNotNull((void**)&(*lplpSDB)->lpszType);
    FreeIfNotNull((void**)&(*lplpSDB)->lpszOutputFormat);
    FreeIfNotNull((void**)&(*lplpSDB)->lpszElementSep);
    FreeIfNotNull((void**)&(*lplpSDB)->lpszSegSep);
    FreeIfNotNull((void**)&(*lplpSDB)->lpszCCSID);
    FreeIfNotNull((void**)&(*lplpSDB)->lpszEncoding);

    /* Free any associated segment and element blocks */
    lpMSLL = (*lplpSDB)->lpMSLLHead;
    while(NULL != lpMSLL)
    {
        lpMSLLFree = lpMSLL;
        lpMSLL = (MsgSetLinkedListBlk*)lpMSLL->lpMSLLNext;
        FreeMSLL(&lpMSLLFree);
    }

    /* Free the segment definition block itself */
    FreeIfNotNull((void**)lplpSDB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeMSLL                                                             */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an segment or element block and its contents                    */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpMSLL Pointer to pointer to the segment or element block          */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeMSLL(MsgSetLinkedListBlk** lplpMSLL)
{
    switch((*lplpMSLL)->eMSLLBlockType)
    {
    case P_SEGMENT:
        FreeSGB((SegmentBlk**)lplpMSLL);
        break;
    case P_ELEMENT:
        FreeELB((ElementBlk**)lplpMSLL);
        break;
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeSGB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an segment block and its contents                               */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpSGB Pointer to pointer to the segment block                      */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeSGB(SegmentBlk** lplpSGB)
{
    /* Free the folder data strings */
    FreeIfNotNull((void**)&(*lplpSGB)->lpszType);
    FreeIfNotNull((void**)&(*lplpSGB)->lpszOutputFormat);
    FreeIfNotNull((void**)&(*lplpSGB)->lpszElementSep);
    FreeIfNotNull((void**)&(*lplpSGB)->lpszSegSep);
    FreeIfNotNull((void**)&(*lplpSGB)->lpszMinOccurs);
    FreeIfNotNull((void**)&(*lplpSGB)->lpszMaxOccurs);
    FreeIfNotNull((void**)&(*lplpSGB)->lpszCCSID);
    FreeIfNotNull((void**)&(*lplpSGB)->lpszEncoding);

    /* Free the segment definition block itself */
    FreeIfNotNull((void**)lplpSGB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeELB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an element block and its contents                               */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpELB Pointer to pointer to the element block                      */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeELB(ElementBlk** lplpELB)
{
    /* Free the folder data strings */
    FreeIfNotNull((void**)&(*lplpELB)->lpszName);
    FreeIfNotNull((void**)&(*lplpELB)->lpszElementType);
    FreeIfNotNull((void**)&(*lplpELB)->lpszElementSep);
    FreeIfNotNull((void**)&(*lplpELB)->lpszLen);
    FreeIfNotNull((void**)&(*lplpELB)->lpszPos);
    FreeIfNotNull((void**)&(*lplpELB)->lpszOffset);
    FreeIfNotNull((void**)&(*lplpELB)->lpszJustify);
    FreeIfNotNull((void**)&(*lplpELB)->lpszPadChar);
    FreeIfNotNull((void**)&(*lplpELB)->lpszEndAlign);
    FreeIfNotNull((void**)&(*lplpELB)->lpszMinOccurs);
    FreeIfNotNull((void**)&(*lplpELB)->lpszMaxOccurs);
    FreeIfNotNull((void**)&(*lplpELB)->lpszDefaultValue);

    /* Free the element block itself */
    FreeIfNotNull((void**)lplpELB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeQIB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an queue info block and its contents                            */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpQIB Pointer to pointer to the queue info block                   */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeQIB(QueueInfoBlk** lplpQIB)
{
    FreeIfNotNull((void**)&(*lplpQIB)->lpszTargQName);
    FreeIfNotNull((void**)&(*lplpQIB)->lpszTargDynQName);
    FreeIfNotNull((void**)&(*lplpQIB)->lpszTargQMgrName);
    FreeIfNotNull((void**)&(*lplpQIB)->lpszConnQMgrName);
    FreeIfNotNull((void**)&(*lplpQIB)->lpszResolvedQName);

    /* Free the queue info block itself */
    FreeIfNotNull((void**)lplpQIB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeIIB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an interval info block and its contents                         */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpIIB Pointer to pointer to the interval info block                */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeIIB(IntervalInfoBlk** lplpIIB)
{
    IntervalInfoRec* lpIIR;
    IntervalInfoRec* lpIIRFree;

    /* If we have an open interval file then close it */
    if(TRUE == (*lplpIIB)->bHaveIntervalFile)
    {
        fclose((*lplpIIB)->lpIntervalFile);
    }

    /* Free the interval name string */
    FreeIfNotNull((void**)&(*lplpIIB)->lpszIntervalName);

    /* Free any Interval Info records */
    lpIIR = (*lplpIIB)->lpIIRHead;
    while(NULL != lpIIR)
    {
        lpIIRFree = lpIIR;
        lpIIR = (IntervalInfoRec*)lpIIR->lpIIRNext;
        FreeIfNotNull((void**)&lpIIRFree);
    }
    
    /* Free the interval info block itself */
    FreeIfNotNull((void**)lplpIIB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeMRB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free a message rate block and its contents                           */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpMRB Pointer to pointer to the message rate block                 */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeMRB(MessageRateBlk** lplpMRB)
{
    /* Free up From and To strings */
    FreeIfNotNull((void**)&(*lplpMRB)->lpszFrom);
    FreeIfNotNull((void**)&(*lplpMRB)->lpszTo);
    FreeIfNotNull((void**)&(*lplpMRB)->lpszResFrom);
    FreeIfNotNull((void**)&(*lplpMRB)->lpszResTo);

    /* Free the message rate block itself */
    FreeIfNotNull((void**)lplpMRB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeMSFL                                                             */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free a message set file load block and its contents                  */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpMSFL Pointer to pointer to the message set file load block       */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeMSFL(MsgSetFileLoadBlk** lplpMSFL)
{
    /* Free up From and To strings */
    FreeIfNotNull((void**)&(*lplpMSFL)->lpszFileName);
    FreeIfNotNull((void**)&(*lplpMSFL)->lpszDir);
    FreeIfNotNull((void**)&(*lplpMSFL)->lpszParserName);

    /* Free the message set file load block itself */
    FreeIfNotNull((void**)lplpMSFL);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeRIB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an RFH2 info block and its contents                             */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpRIB Pointer to pointer to the RFH2 info block                    */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeRIB(RFH2InfoBlk** lplpRIB)
{
    /* Call the RIB clear function */
    ClearRIB(*lplpRIB);
    
    /* Free the RFH2 info block itself */
    FreeIfNotNull((void**)lplpRIB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeVIB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free an var info block and its contents                              */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpVIB Pointer to pointer to the var info block                     */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeVIB(VarInfoBlk** lplpVIB)
{
    /* Free variable name string */
    FreeIfNotNull((void**)&(*lplpVIB)->lpszVarName);
    FreeIfNotNull((void**)&(*lplpVIB)->lpszVarFormat);
    FreeIfNotNull((void**)&(*lplpVIB)->lpszCurrStrValue);

    /* Free attached control blocks */
    if((*lplpVIB)->lpCtrlBlk)
    {
        switch((*lplpVIB)->eVarType)
        {
        case V_LIST:
            FreeLIB((ListInfoBlk**)&(*lplpVIB)->lpCtrlBlk);
            break;
        case V_MQMD:
            FreeMIB((MQMDInfoBlk**)&(*lplpVIB)->lpCtrlBlk);
            break;
        case V_RFH2:
            FreeRIB((RFH2InfoBlk**)&(*lplpVIB)->lpCtrlBlk);
            break;
        }
    }

    /* Free the VIB itself */
    FreeIfNotNull((void**)lplpVIB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeCCB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free a command control block and its contents                        */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpCCB    Pointer to the command control block                        */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeCCB(CommandCtrlBlk** lplpCCB)
{
    GetFileBlk*       lpGFB;
    PutFileBlk*       lpPFB;
    InlineDataBlk*    lpIDB;
    OverlayBlk*       lpOVB;
    ExtractBlk*       lpEXB;
    SubstringBlk*     lpSSB;
    GetMsgBlk*        lpGMB;
    PutMsgBlk*        lpPMB;
    SelectBlk*        lpSLB;
    WaitDepthBlk*     lpWDB;
    WaitTimeBlk*      lpWTB;
    CompareBlk*       lpCMB;
    CommitBlk*        lpCOB;
    BackoutBlk*       lpBOB;
    SystemBlk*        lpSYB;
    MqscCommandBlk*   lpSCB;
    ForStartBlk*      lpFSB;
    ForEndBlk*        lpFEB;
    IfStartBlk*       lpIFS;
    ElseIfBlk*        lpELI;
    VarBlk*           lpVRB;
    MathFunctionBlk*  lpMFB;
    LengthBlk*        lpLNB;
    PositionBlk*      lpPSB;
    StdInBlk*         lpSIB;
    StdOutBlk*        lpSOB;
    UserLogBlk*       lpULB;

    /* Free command specific control blocks */
    /* Dont do the BIB's as they are chained off the TIB */
    /* Dont do the QIB's as they are chained off the MCB */
    /* Dont do the IIB's as they are chained off the MCB */
    switch((*lplpCCB)->eCommandType)
    {
    case C_GETFILE:
        lpGFB = (GetFileBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpGFB->lpszMQMDVarName);
        FreeIfNotNull((void**)&lpGFB->lpszRFH2VarName);
        FreeIfNotNull((void**)&lpGFB->lpszBuffName);
        FreeIfNotNull((void**)&lpGFB->lpszFileName);
        FreeIfNotNull((void**)&lpGFB->lpszMQMDFileName);
        FreeIfNotNull((void**)&lpGFB->lpszRFH2FileName);
        FreeIfNotNull((void**)&lpGFB->lpszDir);
        FreeIfNotNull((void**)&lpGFB->lpszCmdMaxWait);
        FreeIfNotNull((void**)&lpGFB->lpszResMQMDVarName);
        FreeIfNotNull((void**)&lpGFB->lpszResRFH2VarName);
        FreeIfNotNull((void**)&lpGFB->lpszResBuffName);
        FreeIfNotNull((void**)&lpGFB->lpszResFileName);
        FreeIfNotNull((void**)&lpGFB->lpszResMQMDFileName);
        FreeIfNotNull((void**)&lpGFB->lpszResRFH2FileName);
        FreeIfNotNull((void**)&lpGFB->lpszResDir);
        FreeIfNotNull((void**)&lpGFB->lpszResCmdMaxWait);
        break;
    case C_PUTFILE:
        lpPFB = (PutFileBlk*)(*lplpCCB);
        /* As all pointers in the BIB reference data that is under the */
        /* primary control of other control blocks or data that is     */
        /* static we only need to free the actual control block not    */
        /* its contents                                                */
        FreeIfNotNull((void**)&lpPFB->lpTempDataVarBIB);
        FreeIfNotNull((void**)&lpPFB->lpszDataVarName);
        FreeIfNotNull((void**)&lpPFB->lpszMQMDVarName);
        FreeIfNotNull((void**)&lpPFB->lpszRFH2VarName);
        FreeIfNotNull((void**)&lpPFB->lpszBuffName);
        FreeIfNotNull((void**)&lpPFB->lpszFileName);
        FreeIfNotNull((void**)&lpPFB->lpszMQMDFileName);
        FreeIfNotNull((void**)&lpPFB->lpszRFH2FileName);
        FreeIfNotNull((void**)&lpPFB->lpszDir);
        FreeIfNotNull((void**)&lpPFB->lpszResDataVarName);
        FreeIfNotNull((void**)&lpPFB->lpszResMQMDVarName);
        FreeIfNotNull((void**)&lpPFB->lpszResRFH2VarName);
        FreeIfNotNull((void**)&lpPFB->lpszResBuffName);
        FreeIfNotNull((void**)&lpPFB->lpszResFileName);
        FreeIfNotNull((void**)&lpPFB->lpszResMQMDFileName);
        FreeIfNotNull((void**)&lpPFB->lpszResRFH2FileName);
        FreeIfNotNull((void**)&lpPFB->lpszResDir);
        break;
    case C_INLINEDATA:
        lpIDB = (InlineDataBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpIDB->lpszBuffName);
        FreeIfNotNull((void**)&lpIDB->lpszData);
        FreeIfNotNull((void**)&lpIDB->lpszResBuffName);
        break;
    case C_OVERLAY:
        lpOVB = (OverlayBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpOVB->lpszPos);
        FreeIfNotNull((void**)&lpOVB->lpszLen);
        FreeIfNotNull((void**)&lpOVB->lpszBuffName);
        FreeIfNotNull((void**)&lpOVB->lpszData);
        FreeIfNotNull((void**)&lpOVB->lpszResBuffName);
        break;
    case C_EXTRACT:
        lpEXB = (ExtractBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpEXB->lpszVarName);
        FreeIfNotNull((void**)&lpEXB->lpszPos);
        FreeIfNotNull((void**)&lpEXB->lpszLen);
        FreeIfNotNull((void**)&lpEXB->lpszBuffName);
        FreeIfNotNull((void**)&lpEXB->lpszResVarName);
        FreeIfNotNull((void**)&lpEXB->lpszResBuffName);
        break;
    case C_SUBSTRING:
        lpSSB = (SubstringBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpSSB->lpszVarName);
        FreeIfNotNull((void**)&lpSSB->lpszFromVarName);
        FreeIfNotNull((void**)&lpSSB->lpszPos);
        FreeIfNotNull((void**)&lpSSB->lpszLen);
        FreeIfNotNull((void**)&lpSSB->lpszResVarName);
        FreeIfNotNull((void**)&lpSSB->lpszResFromVarName);
        break;
    case C_GETMSG:
        lpGMB = (GetMsgBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpGMB->lpszConnectionName);
        FreeIfNotNull((void**)&lpGMB->lpszMQMDVarName);
        FreeIfNotNull((void**)&lpGMB->lpszRFH2VarName);
        FreeIfNotNull((void**)&lpGMB->lpszBuffName);
        FreeIfNotNull((void**)&lpGMB->lpszQName);
        FreeIfNotNull((void**)&lpGMB->lpszCmdMaxWait);
        FreeIfNotNull((void**)&lpGMB->lpszGmoOptions);
        FreeIfNotNull((void**)&lpGMB->lpszGmoMatchOptions);
        FreeIfNotNull((void**)&lpGMB->lpszGmoWait);
        FreeIfNotNull((void**)&lpGMB->lpszMsgId);
        FreeIfNotNull((void**)&lpGMB->lpszCorrelId);
        FreeIfNotNull((void**)&lpGMB->lpszGroupId);
        FreeIfNotNull((void**)&lpGMB->lpszMsgSeqNumber);
        FreeIfNotNull((void**)&lpGMB->lpszResConnectionName);
        FreeIfNotNull((void**)&lpGMB->lpszResMQMDVarName);
        FreeIfNotNull((void**)&lpGMB->lpszResRFH2VarName);
        FreeIfNotNull((void**)&lpGMB->lpszResBuffName);
        FreeIfNotNull((void**)&lpGMB->lpszResQName);
        FreeIfNotNull((void**)&lpGMB->lpszResCmdMaxWait);
        FreeIfNotNull((void**)&lpGMB->lpszResGmoWait);
        FreeIfNotNull((void**)&lpGMB->lpszResMsgId);
        FreeIfNotNull((void**)&lpGMB->lpszResCorrelId);
        FreeIfNotNull((void**)&lpGMB->lpszResGroupId);
        FreeIfNotNull((void**)&lpGMB->lpszResMsgSeqNumber);
        break;
    case C_PUTMSG:
        lpPMB = (PutMsgBlk*)(*lplpCCB);
        /* As all pointers in the BIB reference data that is under the */
        /* primary control of other control blocks or data that is     */
        /* static we only need to free the actual control block not    */
        /* its contents                                                */
        if(NULL != lpPMB->lpMRB)
        {
            FreeMRB(&lpPMB->lpMRB);
        }
        FreeIfNotNull((void**)&lpPMB->lpTempDataVarBIB);
        FreeIfNotNull((void**)&lpPMB->lpszConnectionName);
        FreeIfNotNull((void**)&lpPMB->lpszDataVarName);
        FreeIfNotNull((void**)&lpPMB->lpszMQMDVarName);
        FreeIfNotNull((void**)&lpPMB->lpszRFH2VarName);
        FreeIfNotNull((void**)&lpPMB->lpszBuffName);
        FreeIfNotNull((void**)&lpPMB->lpszQName);
        FreeIfNotNull((void**)&lpPMB->lpszQMgrName);
        FreeIfNotNull((void**)&lpPMB->lpszPmoOptions);
        FreeIfNotNull((void**)&lpPMB->lpszMsgId);
        FreeIfNotNull((void**)&lpPMB->lpszCorrelId);
        FreeIfNotNull((void**)&lpPMB->lpszGroupId);
        FreeIfNotNull((void**)&lpPMB->lpszMsgSeqNumber);
        FreeIfNotNull((void**)&lpPMB->lpszFormat);
        FreeIfNotNull((void**)&lpPMB->lpszResConnectionName);
        FreeIfNotNull((void**)&lpPMB->lpszResDataVarName);
        FreeIfNotNull((void**)&lpPMB->lpszResMQMDVarName);
        FreeIfNotNull((void**)&lpPMB->lpszResRFH2VarName);
        FreeIfNotNull((void**)&lpPMB->lpszResBuffName);
        FreeIfNotNull((void**)&lpPMB->lpszResQName);
        FreeIfNotNull((void**)&lpPMB->lpszResQMgrName);
        FreeIfNotNull((void**)&lpPMB->lpszResMsgId);
        FreeIfNotNull((void**)&lpPMB->lpszResCorrelId);
        FreeIfNotNull((void**)&lpPMB->lpszResGroupId);
        FreeIfNotNull((void**)&lpPMB->lpszResMsgSeqNumber);
        FreeIfNotNull((void**)&lpPMB->lpszResFormat);
        break;
    case C_SELECT:
        lpSLB = (SelectBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpSLB->lpszMQMDVarName);
        FreeIfNotNull((void**)&lpSLB->lpszBuffName);
        FreeIfNotNull((void**)&lpSLB->lpszMsgSetName);
        FreeIfNotNull((void**)&lpSLB->lpszMsgNum);
        FreeIfNotNull((void**)&lpSLB->lpszResBuffName);
        FreeIfNotNull((void**)&lpSLB->lpszResMsgSetName);
        FreeIfNotNull((void**)&lpSLB->lpszResMsgNum);
        break;
    case C_WAITONMSGDEPTH:
        lpWDB = (WaitDepthBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpWDB->lpszConnectionName);
        FreeIfNotNull((void**)&lpWDB->lpszQName);
        FreeIfNotNull((void**)&lpWDB->lpszDepth);
        FreeIfNotNull((void**)&lpWDB->lpszCmdMaxWait);
        FreeIfNotNull((void**)&lpWDB->lpszResConnectionName);
        FreeIfNotNull((void**)&lpWDB->lpszResQName);
        FreeIfNotNull((void**)&lpWDB->lpszResDepth);
        FreeIfNotNull((void**)&lpWDB->lpszResCmdMaxWait);
        break;
    case C_WAITONTIME:
        lpWTB = (WaitTimeBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpWTB->lpszInterval);
        FreeIfNotNull((void**)&lpWTB->lpszIntervalMillis);
        FreeIfNotNull((void**)&lpWTB->lpszTime);
        FreeIfNotNull((void**)&lpWTB->lpszResInterval);
        FreeIfNotNull((void**)&lpWTB->lpszResIntervalMillis);
        FreeIfNotNull((void**)&lpWTB->lpszResTime);
        break;
    case C_COMPARE:
        lpCMB = (CompareBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpCMB->lpszBuffName);
        FreeIfNotNull((void**)&lpCMB->lpszCompFileName);
        FreeIfNotNull((void**)&lpCMB->lpszCompMaskFileName);
        FreeIfNotNull((void**)&lpCMB->lpszCompRegexFileName);
        FreeIfNotNull((void**)&lpCMB->lpszCompRFH2VarName1);
        FreeIfNotNull((void**)&lpCMB->lpszCompRFH2VarName2);
        FreeIfNotNull((void**)&lpCMB->lpszDir);
        FreeIfNotNull((void**)&lpCMB->lpszResBuffName);
        FreeIfNotNull((void**)&lpCMB->lpszResCompFileName);
        FreeIfNotNull((void**)&lpCMB->lpszResCompMaskFileName);
        FreeIfNotNull((void**)&lpCMB->lpszResCompRegexFileName);
        FreeIfNotNull((void**)&lpCMB->lpszResDir);
        FreeIfNotNull((void**)&lpCMB->lpszCompFileBuff);
        FreeIfNotNull((void**)&lpCMB->lpszCompMaskBuff);
        FreeIfNotNull((void**)&lpCMB->lpszCompRegexBuff);
        break;
    case C_COMMIT:
        lpCOB = (CommitBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpCOB->lpszConnectionName);
        FreeIfNotNull((void**)&lpCOB->lpszResConnectionName);
        break;
    case C_BACKOUT:
        lpBOB = (BackoutBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpBOB->lpszConnectionName);
        FreeIfNotNull((void**)&lpBOB->lpszResConnectionName);
        break;
    case C_SYSTEM:
        lpSYB = (SystemBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpSYB->lpszCommand);
        FreeIfNotNull((void**)&lpSYB->lpszResCommand);
        break;
    case C_MQSCCOMMAND:
        lpSCB = (MqscCommandBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpSCB->lpszConnectionName);
        FreeIfNotNull((void**)&lpSCB->lpszBuffName);
        FreeIfNotNull((void**)&lpSCB->lpszQName);
        FreeIfNotNull((void**)&lpSCB->lpszMqscReplyDynamicQName);
        FreeIfNotNull((void**)&lpSCB->lpszCommand);
        FreeIfNotNull((void**)&lpSCB->lpszCmdMaxWait);
        FreeIfNotNull((void**)&lpSCB->lpszResConnectionName);
        FreeIfNotNull((void**)&lpSCB->lpszResBuffName);
        FreeIfNotNull((void**)&lpSCB->lpszResQName);
        FreeIfNotNull((void**)&lpSCB->lpszResMqscReplyDynamicQName);
        FreeIfNotNull((void**)&lpSCB->lpszResCommand);
        FreeIfNotNull((void**)&lpSCB->lpszResCmdMaxWait);
        break;
    case C_FORSTART:
        lpFSB = (ForStartBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpFSB->lpszForName);
        FreeIfNotNull((void**)&lpFSB->lpszFrom);
        FreeIfNotNull((void**)&lpFSB->lpszTo);
        FreeIfNotNull((void**)&lpFSB->lpszIncr);
        FreeIfNotNull((void**)&lpFSB->lpszResFrom);
        FreeIfNotNull((void**)&lpFSB->lpszResTo);
        FreeIfNotNull((void**)&lpFSB->lpszResIncr);
        break;
    case C_FOREND:
        lpFEB = (ForEndBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpFEB->lpszForName);
        break;
    case C_IFSTART:
        lpIFS = (IfStartBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpIFS->lpszCond);
        FreeIfNotNull((void**)&lpIFS->lpszResCond);
        break;
    case C_IFEND:
        /* Nothing to free other than the CCB itself as IfEnd is a target only */
        break;
    case C_ELSEIF:
        lpELI = (ElseIfBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpELI->lpszCond);
        FreeIfNotNull((void**)&lpELI->lpszResCond);
        break;
    case C_ELSE:
        /* Nothing to free other than the CCB itself as Else is a target only */
        break;
    case C_CONTINUE:
        /* Nothing to free other than the CCB itself */
        break;
    case C_BREAK:
        /* Nothing to free other than the CCB itself */
        break;
    case C_EXITTEST:
        /* Nothing to free other than the CCB itself */
        break;
    case C_FAILTEST:
        /* Nothing to free other than the CCB itself */
        break;
    case C_FINALLYSTART:
        /* Nothing to free other than the CCB itself as FinallyStart is a place marker only */
        break;
    case C_FINALLYEND:
        /* Nothing to free other than the CCB itself as FinallyEnd is a place marker only */
        break;
    case C_INTERVALSTART:
        /* Nothing to free other than the CCB itself as IIB's are chained of MCB */
        break;
    case C_INTERVALEND:
        /* Nothing to free other than the CCB itself as IIB's are chained of MCB */
        break;
    case C_VAR:
        lpVRB = (VarBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpVRB->lpszVarName);
        FreeIfNotNull((void**)&lpVRB->lpszVarFormat);
        FreeIfNotNull((void**)&lpVRB->lpszNewStrValue);
        if(lpVRB->lpNewCtrlBlk)
        {
            switch(lpVRB->eVarType)
            {
            case V_LIST:
                FreeLIB((ListInfoBlk**)&lpVRB->lpNewCtrlBlk);
                break;
            case V_MQMD:
                FreeMIB((MQMDInfoBlk**)&lpVRB->lpNewCtrlBlk);
                break;
            case V_RFH2:
                FreeRIB((RFH2InfoBlk**)&lpVRB->lpNewCtrlBlk);
                break;
            }
        }
        FreeIfNotNull((void**)&lpVRB->lpszResVarName);
        break;
    case C_MATH:
        lpMFB = (MathFunctionBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpMFB->lpszVarName);
        FreeIfNotNull((void**)&lpMFB->lpszRemName);
        FreeIfNotNull((void**)&lpMFB->lpszValue);
        FreeIfNotNull((void**)&lpMFB->lpszResVarName);
        FreeIfNotNull((void**)&lpMFB->lpszResRemName);
        FreeIfNotNull((void**)&lpMFB->lpszResValue);
        break;
    case C_LENGTH:
        lpLNB = (LengthBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpLNB->lpszVarName);
        FreeIfNotNull((void**)&lpLNB->lpszOf);
        FreeIfNotNull((void**)&lpLNB->lpszResVarName);
        FreeIfNotNull((void**)&lpLNB->lpszResOf);
        break;
    case C_POSITION:
        lpPSB = (PositionBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpPSB->lpszVarName);
        FreeIfNotNull((void**)&lpPSB->lpszOf);
        FreeIfNotNull((void**)&lpPSB->lpszIn);
        FreeIfNotNull((void**)&lpPSB->lpszFrom);
        FreeIfNotNull((void**)&lpPSB->lpszResVarName);
        FreeIfNotNull((void**)&lpPSB->lpszResOf);
        FreeIfNotNull((void**)&lpPSB->lpszResIn);
        FreeIfNotNull((void**)&lpPSB->lpszResFrom);
        break;
    case C_STDIN:
        lpSIB = (StdInBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpSIB->lpszVarName);
        FreeIfNotNull((void**)&lpSIB->lpszResVarName);
        break;
    case C_STDOUT:
        lpSOB = (StdOutBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpSOB->lpszTextOut);
        FreeIfNotNull((void**)&lpSOB->lpszResTextOut);
        break;
    case C_USERLOG:
        lpULB = (UserLogBlk*)(*lplpCCB);
        FreeIfNotNull((void**)&lpULB->lpszTextOut);
        FreeIfNotNull((void**)&lpULB->lpszResTextOut);
        break;
    }

    /* Free the command block itself */
    FreeIfNotNull((void**)lplpCCB);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeTIB                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free a test info block and its contents                              */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTIB    Pointer to the test info block                              */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeTIB(TestInfoBlk** lplpTIB)
{
    CommandCtrlBlk* lpCCB;
    CommandCtrlBlk* lpCCBFree;
    VarInfoBlk*     lpVIB;
    VarInfoBlk*     lpVIBFree;
    BuffInfoBlk*    lpBIB;
    BuffInfoBlk*    lpBIBFree;
    QueueInfoBlk*   lpQIB;
    QueueInfoBlk*   lpQIBFree;
    MQLONG          lCloseOpts;
    MQLONG          lCC;
    MQLONG          lRC;

    /* Initialise local variables */
    lCloseOpts = MQCO_NONE;
    lCC = MQCC_OK;
    lRC = MQRC_NONE;

    /* Free commands control blocks if any */
    lpCCB = (*lplpTIB)->lpCCBHead;
    while(NULL != lpCCB)
    {
        lpCCBFree = lpCCB;
        lpCCB = (CommandCtrlBlk*)lpCCB->lpCCBNext;
        FreeCCB(&lpCCBFree);
    }

    /* Free var info blocks if any */
    lpVIB = (*lplpTIB)->lpVIBHead;
    while(NULL != lpVIB)
    {
        lpVIBFree = lpVIB;
        lpVIB = (VarInfoBlk*)lpVIB->lpVIBNext;
        FreeVIB(&lpVIBFree);
    }

    /* Free buff info blocks if any */
    lpBIB = (*lplpTIB)->lpBIBHead;
    while(NULL != lpBIB)
    {
        lpBIBFree = lpBIB;
        lpBIB = (BuffInfoBlk*)lpBIB->lpBIBNext;
        FreeBIB(&lpBIBFree);
    }

    /* Close any MQ queues that are open and then free the QIB */
    lpQIB = (*lplpTIB)->lpQIBHead;
    while(NULL != lpQIB)
    {
        MQCLOSE(lpQIB->hConn, &lpQIB->hQ, lCloseOpts, &lCC, &lRC);
        if(MQRC_NONE == lRC)
        {
            if(0 == StrlenNull(lpQIB->lpszTargDynQName))
            {
                LogMsg(56, (*lplpTIB)->lTestNum, lpQIB->lpszTargQName, lpQIB->lpszTargQMgrName, lpQIB->lpszConnQMgrName, lCC, lRC);
            }
            else
            {
                LogMsg(251, (*lplpTIB)->lTestNum, lpQIB->lpszTargQName, lpQIB->lpszTargQMgrName, lpQIB->lpszTargDynQName, lpQIB->lpszConnQMgrName, lCC, lRC);
            }
        }
        else
        {
            if(0 == StrlenNull(lpQIB->lpszTargDynQName))
            {
                LogMsg(57, (*lplpTIB)->lTestNum, lpQIB->lpszTargQName, lpQIB->lpszTargQMgrName, lpQIB->lpszConnQMgrName, lCC, lRC);
            }
            else
            {
                LogMsg(252, (*lplpTIB)->lTestNum, lpQIB->lpszTargQName, lpQIB->lpszTargQMgrName, lpQIB->lpszTargDynQName, lpQIB->lpszConnQMgrName, lCC, lRC);
            }
        }
        lpQIBFree = lpQIB;
        lpQIB = (QueueInfoBlk*)lpQIB->lpQIBNext;
        FreeQIB(&lpQIBFree);
    }

    /* Free the test name string */
    FreeIfNotNull((void**)&(*lplpTIB)->lpszTestName);
    FreeIfNotNull((void**)&(*lplpTIB)->lpszLastFile);
    FreeIfNotNull((void**)&(*lplpTIB)->lpszLastDir);
    FreeIfNotNull((void**)&(*lplpTIB)->lpszLastQ);
    FreeIfNotNull((void**)&(*lplpTIB)->lpszLastQMgr);

    /* Finally free the test info block itself */
    FreeIfNotNull((void**)lplpTIB);
}
