/*==========================================================================*/
/* MsgTestUtil.c contains the various utility routines required to          */
/* manipulate linked lists, locate control blocks in linked lists           */
/* and special string manipulation routines                                 */
/*==========================================================================*/
/* (C) Copyright Tim Armstrong 2003, 2004, 2005, 2006, 2007, 2008           */
/*==========================================================================*/
/* Includes                                                                 */
/*==========================================================================*/
#ifndef __TJAINC_MSGTEST_H
#include "MsgTest.h"
#endif /* __TJAINC_MSGTEST_H */

#ifndef __TJAINC_MSGTESTSTATICAV_H
#include "MsgTestStaticAV.h"
#endif /* __TJAINC_MSGTESTSTATICAV_H */

#ifndef __TJAINC_MSGTESTSTATICMD_H
#include "MsgTestStaticMD.h"
#endif /* __TJAINC_MSGTESTSTATICMD_H */

/* Handle HP Unix having a different name for strtoll     */
/* Requires a -D__HP_MA0T compiler option in the makefile */
#ifdef __HP_MA0T
#include <sys/_inttypes.h>
#define strtoll __strtoll
#endif

/*==========================================================================*/
/* Static variables                                                         */
/*==========================================================================*/
static char* kpszEmptyString =              "\0";
static char* kpszUNKNOWN =                  "Unknown\0";
static char* kpszNUMFMT =                   "%d\0";
static char* kpszDOUBLEFMT =                "%lf\0";
static char* kpszSTRFMT =                   "%s\0";
static char* kpszNUM =                      "Num\0";
static char* kpszDOUBLE =                   "Double\0";
static char* kpszSTR =                      "Str\0";
static char* kpszRANDOMNUM =                "RandomNum\0";
static char* kpszLIST =                     "List\0";
static char* kpszRFH2 =                     "RFH2\0";
static char* kpszTRUE =                     "TRUE\0";
static char* kpszFALSE =                    "FALSE\0";
static char* kpszMQMI_NONE =                "MQMI_NONE\0";
static char* kpszMQCI_NONE =                "MQCI_NONE\0";
static char* kpszMQGI_NONE =                "MQGI_NONE\0";
static char* kpszNULLS =                    "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
static char  kcDECIMALPOINT =               '.';
static char* kpszSTRINGEVAL =               "String\0";
static char* kpszINTEGEREVAL =              "Integer\0";
static char* kpszDOUBLEEVAL =               "Double\0";
static char* kpszEQUALTO =                  ".EQ.\0";
static char* kpszNOTEQUALTO =               ".NE.\0";
static char* kpszGREATER =                  ".GT.\0";
static char* kpszLESS =                     ".LT.\0";
static char* kpszGREATEREQUAL =             ".GE.\0";
static char* kpszLESSEQUAL =                ".LE.\0";
static char* kpszADD =                      ".ADD.\0";
static char* kpszSUBTRACT =                 ".SUB.\0";
static char* kpszMULTIPLY =                 ".MUL.\0";
static char* kpszDIVIDE =                   ".DIV.\0";
static char* kpszREMAINDER =                ".REM.\0";

static unsigned long  klFFFFFFFF =          0xffffffff;

/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FindBuffInfoBlk                                                      */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Find a BuffInfoBlock in the linked list                              */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpBIB              Pointer to first BuffInfoBlock in the chain       */
/*     lpszFindBuffName   Name of the buffer to find                        */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to the BuffInfoBlk, NULL if not found                        */
/*==========================================================================*/
BuffInfoBlk*
FindBuffInfoBlk(BuffInfoBlk* lpBIB, char* lpszFindBuffName)
{
    long lFindBuffNameLen;
    long lBIBBuffNameLen;

    /* If no name is supplied then we can't find it  */
    if(NULL == lpszFindBuffName)
    {
        return NULL;
    }

    /* If its an empty string then we don't allow it */
    lFindBuffNameLen = StrlenNull(lpszFindBuffName);
    if(0 == lFindBuffNameLen)
    {
        return NULL;
    }

    while(NULL != lpBIB)
    {
        lBIBBuffNameLen = StrlenNull(lpBIB->lpszBuffName);
        if(lFindBuffNameLen == lBIBBuffNameLen)
        {
            if(0 == memcmp(lpszFindBuffName, lpBIB->lpszBuffName,
                           lFindBuffNameLen))
            {
                return lpBIB;
            }
        }
        lpBIB = (BuffInfoBlk*)lpBIB->lpBIBNext;
    }
    return NULL;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     CheckAndAddBIB                                                       */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Check to see if a BuffInfoBlock exists if so just return it. If not  */
/*     create one, initialise it and add it to the tests linked list.       */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTIB              Pointer to a Test Info Block                      */
/*     lpszFindBuffName   Name of the buffer to find                        */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to a BuffInfoBlk                                             */
/*==========================================================================*/
BuffInfoBlk*
CheckAndAddBIB(TestInfoBlk* lpTIB, char* lpszFindBuffName)
{
    BuffInfoBlk* lpBIB;

    /* If no name is supplied then fail the test and return NULL */
    if(NULL == lpszFindBuffName)
    {
        lpTIB->bCmdResult = FAILURE;
        return NULL;
    }

    /* If its an empty string then fail the test and return NULL */
    if(0 == StrlenNull(lpszFindBuffName))
    {
        lpTIB->bCmdResult = FAILURE;
        return NULL;
    }

    /* If we can find it just return the pointer */
    lpBIB = FindBuffInfoBlk(lpTIB->lpBIBHead, lpszFindBuffName);
    if(lpBIB)
    {
        return lpBIB;
    }

    /* Ok can't find a buffer with this name lets make one */
    lpBIB = malloc(sizeof(BuffInfoBlk));
    if(lpBIB)
    {
        memset(lpBIB, 0x00, sizeof(BuffInfoBlk));
        lpBIB->lpBIBNext = NULL;
        lpBIB->lpBIBPrev = NULL;
        lpBIB->lpszBuffName = CloneString(lpszFindBuffName);
        lpBIB->lpszBuff = NULL;
        lpBIB->lBuffLen = 0;
        lpBIB->lDataLen = 0;
        /* Add it to the tests BIB linked list */
        AddToTailOfList((LinkedListBlk**)&lpTIB->lpBIBHead,
                        (LinkedListBlk**)&lpTIB->lpBIBTail,
                        (LinkedListBlk*)lpBIB);
    }
    else
    {
        LogMsg(3, sizeof(BuffInfoBlk), "BuffInfoBlk", __LINE__, __FILE__);
        lpTIB->bCmdResult = FAILURE;
        kpMCB->lProcessing = FAILURE;
        return NULL;
    }

    return lpBIB;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FindConnectionDefBlk                                                 */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Find a ConnectionDefBlock in the linked list                         */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpCDB              Pointer to first ConnectionDefBlock in the chain  */
/*     lpszFindConnectionName   Name of the buffer to find                  */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to the ConnectionDefBlk, NULL if not found                   */
/*==========================================================================*/
ConnectionDefBlk*
FindConnectionDefBlk(ConnectionDefBlk* lpCDB, const char* const kpszFindConnectionName)
{
    long lFindConnectionNameLen;
    long lCDBConnectionNameLen;

    /* If no name is supplied then we can't find it  */
    if(NULL == kpszFindConnectionName)
    {
        return NULL;
    }

    /* If its an empty string then we don't allow it */
    lFindConnectionNameLen = StrlenNull(kpszFindConnectionName);
    if(0 == lFindConnectionNameLen)
    {
        return NULL;
    }

    while(NULL != lpCDB)
    {
        lCDBConnectionNameLen = StrlenNull(lpCDB->lpszConnectionName);
        if(lFindConnectionNameLen == lCDBConnectionNameLen)
        {
            if(0 == memcmp(kpszFindConnectionName, lpCDB->lpszConnectionName,
                           lFindConnectionNameLen))
            {
                return lpCDB;
            }
        }
        lpCDB = (ConnectionDefBlk*)lpCDB->lpCDBNext;
    }
    return NULL;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FindFolderInfoBlk                                                    */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Find a FolderInfoBlock in the linked list                            */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpFIB              Pointer to first FolderInfoBlock in the chain     */
/*     lpszFindFolderName Name of the buffer to find                        */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to the FolderInfoBlk, NULL if not found                      */
/*==========================================================================*/
FolderInfoBlk*
FindFolderInfoBlk(FolderInfoBlk* lpFIB, char* lpszFindFolderName)
{
    long lFindFolderNameLen;
    long lFIBFolderNameLen;

    /* If no name is supplied then we can't find it  */
    if(NULL == lpszFindFolderName)
    {
        return NULL;
    }

    /* If its an empty string then we don't allow it */
    lFindFolderNameLen = StrlenNull(lpszFindFolderName);
    if(0 == lFindFolderNameLen)
    {
        return NULL;
    }

    while(NULL != lpFIB)
    {
        lFIBFolderNameLen = StrlenNull(lpFIB->lpszFolderName);
        if(lFindFolderNameLen == lFIBFolderNameLen)
        {
            if(0 == memcmp(lpszFindFolderName, lpFIB->lpszFolderName,
                           lFindFolderNameLen))
            {
                return lpFIB;
            }
        }
        lpFIB = (FolderInfoBlk*)lpFIB->lpFIBNext;
    }
    return NULL;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     CheckAndAddFIB                                                       */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Check to see if a FolderInfoBlock exists if so replace it. If not    */
/*     create one, initialise it and add it to the RIB's linked list.       */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTIB              Pointer to a Test Info Block                      */
/*     lpRIB              Pointer to a RFH2 Info Block                      */
/*     bPad               Pad folder data with blanks flag                  */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to a FolderInfoBlk                                           */
/*==========================================================================*/
FolderInfoBlk*
CheckAndAddFIB(TestInfoBlk* lpTIB, RFH2InfoBlk* lpRIB, long bPad)
{
    FolderInfoBlk* lpFIBNew;
    FolderInfoBlk* lpFIBFind;
    FolderInfoBlk* lpFIBCurr;

    /* Create the new Folder info block */
    lpFIBNew = (FolderInfoBlk*)CreateFolderInfoBlk(lpTIB, bPad);

    /* Check to see if a folder with this name exists on the RIB's */
    /* FIB chain if so remove it before adding the new FIB         */
    lpFIBFind = FindFolderInfoBlk(lpRIB->lpFIBHead, lpFIBNew->lpszFolderName);
    if(lpFIBFind)
    {
        RemoveFromList((LinkedListBlk**)&lpRIB->lpFIBHead,
                       (LinkedListBlk**)&lpRIB->lpFIBTail,
                       (LinkedListBlk*)lpFIBFind);
        FreeFIB(&lpFIBFind);
    }

    /* Add the new FIB */
    AddToTailOfList((LinkedListBlk**)&lpRIB->lpFIBHead,
                    (LinkedListBlk**)&lpRIB->lpFIBTail,
                    (LinkedListBlk*)lpFIBNew);
    
    /* Recalculate the RIB folder data count and length variables */
    lpRIB->lRFH2FolderCount = 0;
    lpRIB->lRFH2FolderDataLen = 0;
    lpFIBCurr = lpRIB->lpFIBHead;
    while(NULL != lpFIBCurr)
    {
        lpRIB->lRFH2FolderCount++;
        lpRIB->lRFH2FolderDataLen += lpFIBCurr->lFolderDataLen + 4;
        lpFIBCurr = (FolderInfoBlk*)lpFIBCurr->lpFIBNext;
    }

    /* Return a pointer to our new FIB */
    return lpFIBNew;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     AddLEB                                                               */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Create and add a ListElementInfoBlock                                */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpLIB              Pointer to a List Info Block                      */
/*     kpszListElement    Pointer to the text of the list element           */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to a ListElementInfoBlk                                      */
/*==========================================================================*/
ListElementInfoBlk*
AddLEB(ListInfoBlk* lpLIB, const char* const kpszListElement)
{
    ListElementInfoBlk* lpLEBNew;

    /* Create the new ListElement info block */
    lpLEBNew = (ListElementInfoBlk*)CreateListElementInfoBlk(kpszListElement);

    /* Add the new LEB */
    AddToTailOfList((LinkedListBlk**)&lpLIB->lpLEBHead,
                    (LinkedListBlk**)&lpLIB->lpLEBTail,
                    (LinkedListBlk*)lpLEBNew);
    
    /* Return a pointer to our new LEB */
    return lpLEBNew;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FindIntervalInfoBlk                                                  */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Find a IntervalInfoBlock in the linked list                          */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpszFindIntervalName   Name of the Interval to find                  */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to the IntervalInfoBlk, NULL if not found                    */
/*==========================================================================*/
IntervalInfoBlk*
FindIntervalInfoBlk(char* lpszFindIntervalName)
{
    IntervalInfoBlk* lpIIB;
    long             lFindIntervalNameLen;
    long             lIIBIntervalNameLen;

    /* If no name is supplied then we can't find it  */
    if(NULL == lpszFindIntervalName)
    {
        return NULL;
    }

    /* If its an empty string then we don't allow it */
    lFindIntervalNameLen = StrlenNull(lpszFindIntervalName);
    if(0 == lFindIntervalNameLen)
    {
        return NULL;
    }

    lpIIB = kpMCB->lpIIBHead;
    while(NULL != lpIIB)
    {
        lIIBIntervalNameLen = StrlenNull(lpIIB->lpszIntervalName);
        if(lFindIntervalNameLen == lIIBIntervalNameLen)
        {
            if(0 == memcmp(lpszFindIntervalName, lpIIB->lpszIntervalName,
                           lFindIntervalNameLen))
            {
                return lpIIB;
            }
        }
        lpIIB = (IntervalInfoBlk*)lpIIB->lpIIBNext;
    }
    return NULL;
}

/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FindMsgSetLinkedListBlk                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Find a NamedLinkedListBlock in the linked list                       */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpszFindName   Name of the Element to find                           */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to the MsgSetLinkedListBlk, NULL if not found                */
/*==========================================================================*/
MsgSetLinkedListBlk*
FindMsgSetLinkedListBlk(MsgSetLinkedListBlk* lpMSLL,
                        char* lpszFindName,
                        long lFindNameLen)
{
    /* If no name is supplied then we can't find it  */
    if(NULL == lpszFindName)
    {
        return NULL;
    }

    /* If its an empty string then we don't allow it */
    if(0 == lFindNameLen)
    {
        return NULL;
    }

    while(NULL != lpMSLL)
    {
        if(lFindNameLen == lpMSLL->lNameOrTypeLen)
        {
            if(0 == memcmp(lpszFindName, lpMSLL->lpszNameOrType,
                           lFindNameLen))
            {
                return lpMSLL;
            }
        }
        lpMSLL = (MsgSetLinkedListBlk*)lpMSLL->lpMSLLNext;
    }

    return NULL;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FindNextRequiredMSLL                                                 */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Find the next MSLL with a MinOccurs value greater than 0             */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpszFindElementName   Name of the Element to find                    */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to the MsgSetLinkedListBlk, NULL if not found                */
/*==========================================================================*/
MsgSetLinkedListBlk*
FindNextRequiredMSLL(MsgSetLinkedListBlk* lpMSLL)
{
    ElementBlk* lpELB;
    SegmentBlk* lpSGB;

    /* Start with the next node after this one */
    lpMSLL = (MsgSetLinkedListBlk*)lpMSLL->lpMSLLNext;
    while(NULL != lpMSLL)
    {
        switch(lpMSLL->eMSLLBlockType)
        {
        case P_ELEMENT:
            lpELB = (ElementBlk*)lpMSLL;
            if(0 < lpELB->lMinOccurs)
            {
                return lpMSLL;
            }
            break;
        case P_SEGMENT:
            lpSGB = (SegmentBlk*)lpMSLL;
            if(0 < lpSGB->lMinOccurs)
            {
                return lpMSLL;
            }
            break;
        }
        lpMSLL = (MsgSetLinkedListBlk*)lpMSLL->lpMSLLNext;
    }
    return NULL;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FindMsgTreeNode                                                      */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Find a tree node from ther current pointer if available or by        */
/*     scanning all the children of the parent if necessary                 */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTNParent    Pointer to parent element who's children are to be     */
/*                   scanned for a matching element                         */
/*     lpTNCurr      Pointer to current element who's next siblings are to  */
/*                   be scanned for a matching element                      */
/*     lpszFindName  Name of the element to find                            */
/*     lFindNameLen  Length of the element name                             */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to the MsgSetLinkedListBlk, NULL if not found                */
/*==========================================================================*/
MsgSetTreeNode*
FindMsgTreeNode(MsgSetTreeNode* lpTNParent, MsgSetTreeNode* lpTNCurr,
                char* lpszFindName, long lFindNameLen)
{
    MsgSetTreeNode* lpTNWork;
    /* If we have no children then there is nothing to scan */
    if(NULL == lpTNParent->lpFirstChild)
    {
        return NULL;
    }

    /* Try to locate the node from the current node pointer */
    if(NULL != lpTNCurr)
    {
        lpTNWork = (MsgSetTreeNode*)lpTNCurr->lpNextSibling;
        while(NULL != lpTNWork)
        {
            if(lFindNameLen == lpTNWork->lNodeNameLen)
            {
                if(0 == memcmp(lpszFindName, lpTNWork->lpszNodeName, lFindNameLen))
                {
                    return lpTNWork;
                }
            }
            lpTNWork = (MsgSetTreeNode*)lpTNWork->lpNextSibling;
        }
    }

    /* Try to locate the node from the parent node pointer */
    lpTNWork = (MsgSetTreeNode*)lpTNParent->lpFirstChild;
    while(NULL != lpTNWork)
    {
        if(lFindNameLen == lpTNWork->lNodeNameLen)
        {
            if(0 == memcmp(lpszFindName, lpTNWork->lpszNodeName, lFindNameLen))
            {
                return lpTNWork;
            }
        }
        lpTNWork = (MsgSetTreeNode*)lpTNWork->lpNextSibling;
    }

    return NULL;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     AddOrUpdateIIR                                                       */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Add or update an IntervalInfoRec on the IIB's IIR chain              */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpIIB             Pointer to Interval Info block                     */
/*     eIntervalStartEnd IntervalStart or IntervalEnd indicator             */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to the IntervalInfoRec                                       */
/*==========================================================================*/
void
AddOrUpdateIIR(IntervalInfoBlk* lpIIB, enum eIntervalEnum eIntervalStartEnd)
{
    IntervalInfoRec* lpIIR;

    lpIIR = lpIIB->lpIIRTail;
    if(lpIIR)
    {
        switch(eIntervalStartEnd)
        {
        case I_START:
            /* If the interval start has already been set */
            /* then create a new IIR record               */
            if(lpIIR->bHaveStart)
            {
                lpIIR = CreateAndAddIntervalInfoRec(lpIIB);
                if(! lpIIR)
                {
                    return;
                }
            }
            if(lpIIR->bHaveEnd)
            {
                lpIIR->bEndBeforeStart = TRUE;
            }
            SetIntervalStart(lpIIR);
            /* If we have both a start, an end and are writing */
            /* to a file then save the current interval value  */
            if((TRUE == lpIIR->bHaveStart) &&
               (TRUE == lpIIR->bHaveEnd) &&
               (TRUE == lpIIB->bHaveIntervalFile))
            {
                WriteIntervalRecord(lpIIB, lpIIR);
            }
            break;
        case I_END:
            /* If the interval end has already been set */
            /* then create a new IIR record             */
            if(lpIIR->bHaveEnd)
            {
                lpIIR = CreateAndAddIntervalInfoRec(lpIIB);
                if(! lpIIR)
                {
                    return;
                }
            }
            SetIntervalEnd(lpIIR);
            /* If we have both a start, an end and are writing */
            /* to a file then save the current interval value  */
            if((TRUE == lpIIR->bHaveStart) &&
               (TRUE == lpIIR->bHaveEnd) &&
               (TRUE == lpIIB->bHaveIntervalFile))
            {
                WriteIntervalRecord(lpIIB, lpIIR);
            }
            break;
        }
    }
    else
    {
        /* Create first IIR record on the chain */
        lpIIR = CreateAndAddIntervalInfoRec(lpIIB);
        if(lpIIR)
        {
            switch(eIntervalStartEnd)
            {
            case I_START:
                SetIntervalStart(lpIIR);
                break;
            case I_END:
                SetIntervalEnd(lpIIR);
                break;
            }
        }
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     CloneFIB                                                             */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Clone a FIB                                                          */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpFIBTo    Pointer to pointer of new FIB                           */
/*     lpFIBFrom    Pointer to FIB being cloned                             */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
CloneFIB(FolderInfoBlk** lplpFIBTo, FolderInfoBlk* lpFIBFrom)
{
    /* Allocate the FIB we are copying to */
    (*lplpFIBTo) = malloc(sizeof(FolderInfoBlk));
    if((*lplpFIBTo))
    {
        /* Initialise the Folder info block fields */
        memset((*lplpFIBTo), 0x00, sizeof(FolderInfoBlk));
        /* If we have an existing FIB use it otherwise its a fatal error */
        if(lpFIBFrom)
        {
            (*lplpFIBTo)->lpszFolderName = CloneString(lpFIBFrom->lpszFolderName);
            (*lplpFIBTo)->lpszFolderData = CloneString(lpFIBFrom->lpszFolderData);
            (*lplpFIBTo)->lFolderDataLen = lpFIBFrom->lFolderDataLen;
            (*lplpFIBTo)->bAlreadyMatched = lpFIBFrom->bAlreadyMatched;
        }
        else
        {
            ConsoleMsg(92);
            kpMCB->lProcessing = FAILURE;
        }
        (*lplpFIBTo)->lpFIBNext = NULL;
        (*lplpFIBTo)->lpFIBPrev = NULL;
    }
    else
    {
        ConsoleMsg(3, sizeof(FolderInfoBlk), "FolderInfoBlk", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     CloneLEB                                                             */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Clone a LEB                                                          */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpLEBTo    Pointer to pointer of new LEB                           */
/*     lpLEBFrom    Pointer to LEB being cloned                             */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
CloneLEB(ListElementInfoBlk** lplpLEBTo, ListElementInfoBlk* lpLEBFrom)
{
    /* Allocate the LEB we are copying to */
    (*lplpLEBTo) = malloc(sizeof(ListElementInfoBlk));
    if((*lplpLEBTo))
    {
        /* Initialise the ListElement info block fields */
        memset((*lplpLEBTo), 0x00, sizeof(ListElementInfoBlk));
        /* If we have an existing LEB use it otherwise its a fatal error */
        if(lpLEBFrom)
        {
            (*lplpLEBTo)->lpszListElementValue = CloneString(lpLEBFrom->lpszListElementValue);
        }
        else
        {
            ConsoleMsg(177);
            kpMCB->lProcessing = FAILURE;
        }
        (*lplpLEBTo)->lpLEBNext = NULL;
        (*lplpLEBTo)->lpLEBPrev = NULL;
    }
    else
    {
        ConsoleMsg(3, sizeof(ListElementInfoBlk), "ListElementInfoBlk", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     CloneLIB                                                             */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Clone a LIB free the target if it exists and was dynamically created */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpLIBTo    Pointer to pointer of new LIB                           */
/*     lpLIBFrom    Pointer to LIB being cloned                             */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
CloneLIB(ListInfoBlk** lplpLIBTo, ListInfoBlk* lpLIBFrom)
{
    ListElementInfoBlk* lpLEBFrom;
    ListElementInfoBlk* lpLEBClone;

    /* If the pointer for the target LIB points to an existing LIB */
    /* then free it if it is dynamic before doing the cloning      */
    if((*lplpLIBTo))
    {
        if((*lplpLIBTo)->bDynamicLIB)
        {
            FreeLIB(&(*lplpLIBTo));
            *lplpLIBTo = NULL;
        }
    }

    /* Allocate the LIB we are copying to */
    (*lplpLIBTo) = malloc(sizeof(ListInfoBlk));
    if((*lplpLIBTo))
    {
        /* Initialise the List info block fields */
        memset((*lplpLIBTo), 0x00, sizeof(ListInfoBlk));
        /* If we have an existing LIB use it otherwise generate an empty one */
        if(lpLIBFrom)
        {
            (*lplpLIBTo)->lpLEBHead = NULL;
            (*lplpLIBTo)->lpLEBTail = NULL;
            (*lplpLIBTo)->lpLEBPointerArray = NULL;
            (*lplpLIBTo)->lpszListFileName = CloneString(lpLIBFrom->lpszListFileName);
            (*lplpLIBTo)->lpszDir = CloneString(lpLIBFrom->lpszDir);
            (*lplpLIBTo)->lpszResListFileName = NULL;
            (*lplpLIBTo)->lpszResDir = NULL;
            (*lplpLIBTo)->lListElementCount = 0;
            lpLEBFrom = lpLIBFrom->lpLEBHead;
            while(NULL != lpLEBFrom)
            {
                CloneLEB(&lpLEBClone, lpLEBFrom);
                AddToTailOfList((LinkedListBlk**)&(*lplpLIBTo)->lpLEBHead,
                                (LinkedListBlk**)&(*lplpLIBTo)->lpLEBTail,
                                (LinkedListBlk*)lpLEBClone);
                lpLEBFrom = (ListElementInfoBlk*)lpLEBFrom->lpLEBNext;
                (*lplpLIBTo)->lListElementCount++;
            }
        }
        else
        {
            (*lplpLIBTo)->lpLEBHead = NULL;
            (*lplpLIBTo)->lpLEBTail = NULL;
            (*lplpLIBTo)->lpLEBPointerArray = NULL;
            (*lplpLIBTo)->lpszListFileName = CloneString(kpszEmptyString);
            (*lplpLIBTo)->lpszDir = CloneString(kpszEmptyString);
            (*lplpLIBTo)->lListElementCount = 0;
        }
        (*lplpLIBTo)->bDynamicLIB = TRUE;
    }
    else
    {
        ConsoleMsg(3, sizeof(ListInfoBlk), "ListInfoBlk", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     CloneMIB                                                             */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Clone a MIB free the target if it exists and was dynamically created */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpMIBTo    Pointer to pointer of new MIB                           */
/*     lpMIBFrom    Pointer to MIB being cloned                             */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
CloneMIB(MQMDInfoBlk** lplpMIBTo, MQMDInfoBlk* lpMIBFrom)
{
    /* If the pointer for the target MIB points to an existing MIB */
    /* then free it if it is dynamic before doing the cloning      */
    if((*lplpMIBTo))
    {
        if((*lplpMIBTo)->bDynamicMIB)
        {
            FreeMIB(&(*lplpMIBTo));
            *lplpMIBTo = NULL;
        }
    }

    /* Allocate the MIB we are copying to */
    (*lplpMIBTo) = malloc(sizeof(MQMDInfoBlk));
    if((*lplpMIBTo))
    {
        /* Initialise the MQMD info block fields */
        memset((*lplpMIBTo), 0x00, sizeof(MQMDInfoBlk));
        /* If we have an existing MIB use it otherwise generate an empty one */
        if(lpMIBFrom)
        {
            (*lplpMIBTo)->lpszVersion = CloneStringNull(lpMIBFrom->lpszVersion);
            (*lplpMIBTo)->lpszReport = CloneStringNull(lpMIBFrom->lpszReport);
            (*lplpMIBTo)->lpszMsgType = CloneStringNull(lpMIBFrom->lpszMsgType);
            (*lplpMIBTo)->lpszExpiry = CloneStringNull(lpMIBFrom->lpszExpiry);
            (*lplpMIBTo)->lpszFeedback = CloneStringNull(lpMIBFrom->lpszFeedback);
            (*lplpMIBTo)->lpszEncoding = CloneStringNull(lpMIBFrom->lpszEncoding);
            (*lplpMIBTo)->lpszCodedCharSetId = CloneStringNull(lpMIBFrom->lpszCodedCharSetId);
            (*lplpMIBTo)->lpszFormat = CloneStringNull(lpMIBFrom->lpszFormat);
            (*lplpMIBTo)->lpszPriority = CloneStringNull(lpMIBFrom->lpszPriority);
            (*lplpMIBTo)->lpszPersistence = CloneStringNull(lpMIBFrom->lpszPersistence);
            (*lplpMIBTo)->lpszMsgId = CloneStringNull(lpMIBFrom->lpszMsgId);
            (*lplpMIBTo)->lpszCorrelId = CloneStringNull(lpMIBFrom->lpszCorrelId);
            (*lplpMIBTo)->lpszBackoutCount = CloneStringNull(lpMIBFrom->lpszBackoutCount);
            (*lplpMIBTo)->lpszReplyToQ = CloneStringNull(lpMIBFrom->lpszReplyToQ);
            (*lplpMIBTo)->lpszReplyToQMgr = CloneStringNull(lpMIBFrom->lpszReplyToQMgr);
            (*lplpMIBTo)->lpszUserIdentifier = CloneStringNull(lpMIBFrom->lpszUserIdentifier);
            (*lplpMIBTo)->lpszAccountingToken = CloneStringNull(lpMIBFrom->lpszAccountingToken);
            (*lplpMIBTo)->lpszApplIdentityData = CloneStringNull(lpMIBFrom->lpszApplIdentityData);
            (*lplpMIBTo)->lpszPutApplType = CloneStringNull(lpMIBFrom->lpszPutApplType);
            (*lplpMIBTo)->lpszPutApplName = CloneStringNull(lpMIBFrom->lpszPutApplName);
            (*lplpMIBTo)->lpszPutDate = CloneStringNull(lpMIBFrom->lpszPutDate);
            (*lplpMIBTo)->lpszPutTime = CloneStringNull(lpMIBFrom->lpszPutTime);
            (*lplpMIBTo)->lpszApplOriginData = CloneStringNull(lpMIBFrom->lpszApplOriginData);
            (*lplpMIBTo)->lpszGroupId = CloneStringNull(lpMIBFrom->lpszGroupId);
            (*lplpMIBTo)->lpszMsgSeqNumber = CloneStringNull(lpMIBFrom->lpszMsgSeqNumber);
            (*lplpMIBTo)->lpszOffset = CloneStringNull(lpMIBFrom->lpszOffset);
            (*lplpMIBTo)->lpszMsgFlags = CloneStringNull(lpMIBFrom->lpszMsgFlags);
            (*lplpMIBTo)->lpszOriginalLength = CloneStringNull(lpMIBFrom->lpszOriginalLength);
            (*lplpMIBTo)->lpszMQMDFileName = CloneString(lpMIBFrom->lpszMQMDFileName);
            (*lplpMIBTo)->lpszDir = CloneString(lpMIBFrom->lpszDir);
            (*lplpMIBTo)->lpszResMQMDFileName = NULL;
            (*lplpMIBTo)->lpszResDir = NULL;
        }
        else
        {
            (*lplpMIBTo)->lpszVersion = NULL;
            (*lplpMIBTo)->lpszReport = NULL;
            (*lplpMIBTo)->lpszMsgType = NULL;
            (*lplpMIBTo)->lpszExpiry = NULL;
            (*lplpMIBTo)->lpszFeedback = NULL;
            (*lplpMIBTo)->lpszEncoding = NULL;
            (*lplpMIBTo)->lpszCodedCharSetId = NULL;
            (*lplpMIBTo)->lpszFormat = NULL;
            (*lplpMIBTo)->lpszPriority = NULL;
            (*lplpMIBTo)->lpszPersistence = NULL;
            (*lplpMIBTo)->lpszMsgId = NULL;
            (*lplpMIBTo)->lpszCorrelId = NULL;
            (*lplpMIBTo)->lpszBackoutCount = NULL;
            (*lplpMIBTo)->lpszReplyToQ = NULL;
            (*lplpMIBTo)->lpszReplyToQMgr = NULL;
            (*lplpMIBTo)->lpszUserIdentifier = NULL;
            (*lplpMIBTo)->lpszAccountingToken = NULL;
            (*lplpMIBTo)->lpszApplIdentityData = NULL;
            (*lplpMIBTo)->lpszPutApplType = NULL;
            (*lplpMIBTo)->lpszPutApplName = NULL;
            (*lplpMIBTo)->lpszPutDate = NULL;
            (*lplpMIBTo)->lpszPutTime = NULL;
            (*lplpMIBTo)->lpszApplOriginData = NULL;
            (*lplpMIBTo)->lpszGroupId = NULL;
            (*lplpMIBTo)->lpszMsgSeqNumber = NULL;
            (*lplpMIBTo)->lpszOffset = NULL;
            (*lplpMIBTo)->lpszMsgFlags = NULL;
            (*lplpMIBTo)->lpszOriginalLength = NULL;
            (*lplpMIBTo)->lpszMQMDFileName = CloneString(kpszEmptyString);
            (*lplpMIBTo)->lpszDir = CloneString(kpszEmptyString);
            (*lplpMIBTo)->lpszResMQMDFileName = NULL;
            (*lplpMIBTo)->lpszResDir = NULL;
        }
        (*lplpMIBTo)->bDynamicMIB = TRUE;
    }
    else
    {
        ConsoleMsg(3, sizeof(MQMDInfoBlk), "MQMDInfoBlk", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     CloneRIB                                                             */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Clone a RIB free the target if it exists and was dynamically created */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpRIBTo    Pointer to pointer of new RIB                           */
/*     lpRIBFrom    Pointer to RIB being cloned                             */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
CloneRIB(RFH2InfoBlk** lplpRIBTo, RFH2InfoBlk* lpRIBFrom)
{
    FolderInfoBlk* lpFIBFrom;
    FolderInfoBlk* lpFIBClone;
    FolderInfoBlk* lpFIBCurr;

    /* If the pointer for the target MIB points to an existing MIB */
    /* then free it if it is dynamic before doing the cloning      */
    if((*lplpRIBTo))
    {
        if((*lplpRIBTo)->bDynamicRIB)
        {
            FreeRIB(&(*lplpRIBTo));
            *lplpRIBTo = NULL;
        }
    }

    /* Allocate the RIB we are copying to */
    (*lplpRIBTo) = malloc(sizeof(RFH2InfoBlk));
    if((*lplpRIBTo))
    {
        /* Initialise the RFH2 info block fields */
        memset((*lplpRIBTo), 0x00, sizeof(RFH2InfoBlk));
        /* If we have an existing RIB use it otherwise generate an empty one */
        if(lpRIBFrom)
        {
            (*lplpRIBTo)->lpFIBHead = NULL;
            (*lplpRIBTo)->lpFIBTail = NULL;
            (*lplpRIBTo)->lpszFormat = CloneStringNull(lpRIBFrom->lpszFormat);
            (*lplpRIBTo)->lpszRFH2FileName = CloneString(lpRIBFrom->lpszRFH2FileName);
            (*lplpRIBTo)->lpszDir = CloneString(lpRIBFrom->lpszDir);
            (*lplpRIBTo)->lpszResRFH2FileName = NULL;
            (*lplpRIBTo)->lpszResDir = NULL;
            (*lplpRIBTo)->lCodedCharSetId = lpRIBFrom->lCodedCharSetId;
            (*lplpRIBTo)->lEncoding = lpRIBFrom->lEncoding;
            (*lplpRIBTo)->bUseRIBEncodingAndCCSID = lpRIBFrom->bUseRIBEncodingAndCCSID;
            lpFIBFrom = lpRIBFrom->lpFIBHead;
            while(NULL != lpFIBFrom)
            {
                CloneFIB(&lpFIBClone, lpFIBFrom);
                AddToTailOfList((LinkedListBlk**)&(*lplpRIBTo)->lpFIBHead,
                                (LinkedListBlk**)&(*lplpRIBTo)->lpFIBTail,
                                (LinkedListBlk*)lpFIBClone);
                lpFIBFrom = (FolderInfoBlk*)lpFIBFrom->lpFIBNext;
            }
        }
        else
        {
            (*lplpRIBTo)->lpFIBHead = NULL;
            (*lplpRIBTo)->lpFIBTail = NULL;
            (*lplpRIBTo)->lpszFormat = NULL;
            (*lplpRIBTo)->lpszRFH2FileName = CloneString(kpszEmptyString);
            (*lplpRIBTo)->lpszDir = CloneString(kpszEmptyString);
            (*lplpRIBTo)->lpszResRFH2FileName = NULL;
            (*lplpRIBTo)->lpszResDir = NULL;
            (*lplpRIBTo)->lCodedCharSetId = 0;
            (*lplpRIBTo)->lEncoding = 0;
            (*lplpRIBTo)->bUseRIBEncodingAndCCSID = FALSE;
            (*lplpRIBTo)->lRFH2FolderDataLen = MQENC_NATIVE;
            (*lplpRIBTo)->lRFH2FolderCount = 0;
        }
        (*lplpRIBTo)->bDynamicRIB = TRUE;
    }
    else
    {
        ConsoleMsg(3, sizeof(RFH2InfoBlk), "RFH2InfoBlk", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
    }

    /* Recalculate the RIB folder data count and length variables */
    (*lplpRIBTo)->lRFH2FolderCount = 0;
    (*lplpRIBTo)->lRFH2FolderDataLen = 0;
    lpFIBCurr = (*lplpRIBTo)->lpFIBHead;
    while(NULL != lpFIBCurr)
    {
        (*lplpRIBTo)->lRFH2FolderCount++;
        (*lplpRIBTo)->lRFH2FolderDataLen += lpFIBCurr->lFolderDataLen + 4;
        lpFIBCurr = (FolderInfoBlk*)lpFIBCurr->lpFIBNext;
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     OverrideLIB                                                          */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Override fields in LIBTo if the field exists in LIBFrom              */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpLIBTo      Pointer to LIB to be overridden                         */
/*     lpLIBFrom    Pointer to LIB containing overrides if any              */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
OverrideLIB(ListInfoBlk* lpLIBTo, ListInfoBlk* lpLIBFrom)
{
    ListElementInfoBlk* lpLEBCurr;
    ListElementInfoBlk* lpLEBClone;

    /* ???TA Implement V130 free up lebpointerarray */

    /* Add LEB's from lpLIBFrom to lpLIBTo */
    lpLEBCurr = lpLIBFrom->lpLEBHead;
    while(NULL != lpLEBCurr)
    {
        CloneLEB(&lpLEBClone, lpLEBCurr);
        AddToTailOfList((LinkedListBlk**)&lpLIBTo->lpLEBHead,
                        (LinkedListBlk**)&lpLIBTo->lpLEBTail,
                        (LinkedListBlk*)lpLEBClone);
        lpLEBCurr = (ListElementInfoBlk*)lpLEBCurr->lpLEBNext;
    }

    /* ???TA Implement V130 rebuild the lebpointerarray */
    lpLIBTo->lListElementCount = 0;
    lpLEBCurr = lpLIBTo->lpLEBHead;
    while(NULL != lpLEBCurr)
    {
        lpLEBCurr = (ListElementInfoBlk*)lpLEBCurr->lpLEBNext;
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     OverrideMIB                                                          */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Override fields in MIBTo if the field exists in MIBFrom              */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpMIBTo      Pointer to MIB to be overridden                         */
/*     lpMIBFrom    Pointer to MIB containing overrides if any              */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
OverrideMIB(MQMDInfoBlk* lpMIBTo, MQMDInfoBlk* lpMIBFrom)
{
    ChangeIfStr2Exists(&lpMIBTo->lpszVersion, &lpMIBFrom->lpszVersion);
    ChangeIfStr2Exists(&lpMIBTo->lpszReport, &lpMIBFrom->lpszReport);
    ChangeIfStr2Exists(&lpMIBTo->lpszMsgType, &lpMIBFrom->lpszMsgType);
    ChangeIfStr2Exists(&lpMIBTo->lpszExpiry, &lpMIBFrom->lpszExpiry);
    ChangeIfStr2Exists(&lpMIBTo->lpszFeedback, &lpMIBFrom->lpszFeedback);
    ChangeIfStr2Exists(&lpMIBTo->lpszEncoding, &lpMIBFrom->lpszEncoding);
    ChangeIfStr2Exists(&lpMIBTo->lpszCodedCharSetId, &lpMIBFrom->lpszCodedCharSetId);
    ChangeIfStr2Exists(&lpMIBTo->lpszFormat, &lpMIBFrom->lpszFormat);
    ChangeIfStr2Exists(&lpMIBTo->lpszPriority, &lpMIBFrom->lpszPriority);
    ChangeIfStr2Exists(&lpMIBTo->lpszPersistence, &lpMIBFrom->lpszPersistence);
    ChangeIfStr2Exists(&lpMIBTo->lpszMsgId, &lpMIBFrom->lpszMsgId);
    ChangeIfStr2Exists(&lpMIBTo->lpszCorrelId, &lpMIBFrom->lpszCorrelId);
    ChangeIfStr2Exists(&lpMIBTo->lpszBackoutCount, &lpMIBFrom->lpszBackoutCount);
    ChangeIfStr2Exists(&lpMIBTo->lpszReplyToQ, &lpMIBFrom->lpszReplyToQ);
    ChangeIfStr2Exists(&lpMIBTo->lpszReplyToQMgr, &lpMIBFrom->lpszReplyToQMgr);
    ChangeIfStr2Exists(&lpMIBTo->lpszUserIdentifier, &lpMIBFrom->lpszUserIdentifier);
    ChangeIfStr2Exists(&lpMIBTo->lpszAccountingToken, &lpMIBFrom->lpszAccountingToken);
    ChangeIfStr2Exists(&lpMIBTo->lpszApplIdentityData, &lpMIBFrom->lpszApplIdentityData);
    ChangeIfStr2Exists(&lpMIBTo->lpszPutApplType, &lpMIBFrom->lpszPutApplType);
    ChangeIfStr2Exists(&lpMIBTo->lpszPutApplName, &lpMIBFrom->lpszPutApplName);
    ChangeIfStr2Exists(&lpMIBTo->lpszPutDate, &lpMIBFrom->lpszPutDate);
    ChangeIfStr2Exists(&lpMIBTo->lpszPutTime, &lpMIBFrom->lpszPutTime);
    ChangeIfStr2Exists(&lpMIBTo->lpszApplOriginData, &lpMIBFrom->lpszApplOriginData);
    ChangeIfStr2Exists(&lpMIBTo->lpszGroupId, &lpMIBFrom->lpszGroupId);
    ChangeIfStr2Exists(&lpMIBTo->lpszMsgSeqNumber, &lpMIBFrom->lpszMsgSeqNumber);
    ChangeIfStr2Exists(&lpMIBTo->lpszOffset, &lpMIBFrom->lpszOffset);
    ChangeIfStr2Exists(&lpMIBTo->lpszMsgFlags, &lpMIBFrom->lpszMsgFlags);
    ChangeIfStr2Exists(&lpMIBTo->lpszOriginalLength, &lpMIBFrom->lpszOriginalLength);}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     OverrideRIB                                                          */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Override fields in RIBTo if the field exists in RIBFrom              */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpRIBTo      Pointer to RIB to be overridden                         */
/*     lpRIBFrom    Pointer to RIB containing overrides if any              */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
OverrideRIB(RFH2InfoBlk* lpRIBTo, RFH2InfoBlk* lpRIBFrom)
{
    FolderInfoBlk* lpFIBCurr;
    FolderInfoBlk* lpFIBFind;
    FolderInfoBlk* lpFIBClone;

    ChangeIfStr2Exists(&lpRIBTo->lpszFormat, &lpRIBFrom->lpszFormat);

    /* Add FIB's from lpRIBFrom to lpRIBTo replacing any that already exist */
    lpFIBCurr = lpRIBFrom->lpFIBHead;
    while(NULL != lpFIBCurr)
    {
        CloneFIB(&lpFIBClone, lpFIBCurr);
        lpFIBFind = FindFolderInfoBlk(lpRIBTo->lpFIBHead, lpFIBClone->lpszFolderName);
        if(lpFIBFind)
        {
            RemoveFromList((LinkedListBlk**)&lpRIBTo->lpFIBHead,
                           (LinkedListBlk**)&lpRIBTo->lpFIBTail,
                           (LinkedListBlk*)lpFIBFind);
            FreeFIB(&lpFIBFind);
        }
        AddToTailOfList((LinkedListBlk**)&lpRIBTo->lpFIBHead,
                        (LinkedListBlk**)&lpRIBTo->lpFIBTail,
                        (LinkedListBlk*)lpFIBClone);
        lpFIBCurr = (FolderInfoBlk*)lpFIBCurr->lpFIBNext;
    }

    /* Recalculate the RIB folder data count and length variables */
    lpRIBTo->lRFH2FolderCount = 0;
    lpRIBTo->lRFH2FolderDataLen = 0;
    lpFIBCurr = lpRIBTo->lpFIBHead;
    while(NULL != lpFIBCurr)
    {
        lpRIBTo->lRFH2FolderCount++;
        lpRIBTo->lRFH2FolderDataLen += lpFIBCurr->lFolderDataLen + 4;
        lpFIBCurr = (FolderInfoBlk*)lpFIBCurr->lpFIBNext;
    }

    /* Copy the values of the Encoding and CCSID fields just in case */
    lpRIBTo->lCodedCharSetId = lpRIBFrom->lCodedCharSetId;
    lpRIBTo->lEncoding = lpRIBFrom->lEncoding;
    lpRIBTo->bUseRIBEncodingAndCCSID = lpRIBFrom->bUseRIBEncodingAndCCSID;

}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     CompareRIBs                                                          */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Compare two RFH2 info blocks                                         */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTIB        TIB Pointer                                             */
/*     lpCompRIB    RIB Pointer                                             */
/*     lpBuffRIB    RIB Pointer                                             */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     SUCCESS or FAILURE                                                   */
/*==========================================================================*/
long
CompareRIBs(TestInfoBlk* lpTIB,
            RFH2InfoBlk* lpCompRIB, RFH2InfoBlk* lpBuffRIB)
{
    FolderInfoBlk* lpCompFIB;
    FolderInfoBlk* lpBuffFIB;
    long           lMatchedCount;
    char           lpszExtraInfo1[EXTRA_MSG_INFO_LEN + 4];

    /* Initialise common use local variables */
    memset(lpszExtraInfo1, 0x00, EXTRA_MSG_INFO_LEN + 4);

    /* Reset the already matched flags in the lpCompRIB's FIB's */
    lpCompFIB = lpCompRIB->lpFIBHead;
    while(NULL != lpCompFIB)
    {
        lpCompFIB->bAlreadyMatched = FALSE;
        lpCompFIB = (FolderInfoBlk*)lpCompFIB->lpFIBNext;
    }

    /* Compare the RIB's */
    lMatchedCount = 0;
    lpBuffFIB = lpBuffRIB->lpFIBHead;
    while(NULL != lpBuffFIB)
    {
        lpCompFIB = lpCompRIB->lpFIBHead;
        while(NULL != lpCompFIB)
        {
            /* If this CompRIB FIB has not been matched to a */
            /* BuffRIB FIB yet check it for a match          */
            if(! lpCompFIB->bAlreadyMatched)
            {
                /* If lengths match then do a data comparison */
                if(lpCompFIB->lFolderDataLen == lpBuffFIB->lFolderDataLen)
                {
                    if(0 == memcmp(lpCompFIB->lpszFolderData,
                                   lpBuffFIB->lpszFolderData,
                                   lpCompFIB->lFolderDataLen))
                    {
                        lpCompFIB->bAlreadyMatched = TRUE;
                        lMatchedCount++;
                    }
                }
            }
            lpCompFIB = (FolderInfoBlk*)lpCompFIB->lpFIBNext;
        }
        lpBuffFIB = (FolderInfoBlk*)lpBuffFIB->lpFIBNext;
    }

    /* If the number of folders to be matched has been then thats it */
    if(lpCompRIB->lRFH2FolderCount == lMatchedCount)
    {
        return SUCCESS;
    }

    /* Otherwise tell the user which ones didn't get a match */
    lpCompFIB = lpCompRIB->lpFIBHead;
    while(NULL != lpCompFIB)
    {
        /* If this CompRIB FIB has not been matched to a */
        /* BuffRIB FIB yet check it for a match          */
        if(! lpCompFIB->bAlreadyMatched)
        {
            LogMsg(74, lpTIB->lTestNum, lpCompFIB->lpszFolderData);
        }
        lpCompFIB = (FolderInfoBlk*)lpCompFIB->lpFIBNext;
    }
    return FAILURE;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     GetFolderName                                                        */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Get the name of the containing folder from the folder data           */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     TestInfoBlk    TIB Pointer                                           */
/*     lpszFolderData Pointer to a string containing the folders data       */
/*     lpszFolderName Pointer to a string containing the folders name       */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     SUCCESS or FAILURE                                                   */
/*==========================================================================*/
long
GetFolderName(TestInfoBlk* lpTIB,
              char** lplpszFolderData, char** lplpszFolderName)
{
    char*       lpszFolderDataCurr;
    char*       lpszWorkCurr;
    long        lFolderDataLen;
    long        lFolderNameLen;
    long        lIdx;
    char        lpszWorkBuff[WORK_BUFF_LEN + 4];

    /* Initialise local variables */
    lpszFolderDataCurr = *lplpszFolderData;
    lpszWorkCurr = lpszWorkBuff;
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);

    /* If the name string already exists free it */
    FreeIfNotNull((void**)&(*lplpszFolderName));

    /* Now extract the folder name from the folder data if we can */
    lFolderDataLen = StrlenNull(*lplpszFolderData);
    if(lFolderDataLen > WORK_BUFF_LEN)
    {
        lFolderDataLen = WORK_BUFF_LEN;
    }

    /* Now extract the folder name from the folder data if we can */
    if('<' == *lpszFolderDataCurr)
    {
        lpszFolderDataCurr++;
        for(lIdx = 0; lIdx < lFolderDataLen; lIdx++)
        {
            if(('>' == *lpszFolderDataCurr) ||
               (' ' == *lpszFolderDataCurr) ||
               ('/' == *lpszFolderDataCurr))
            {
                break; /* for(lIdx = 0; ... */
            }
            *lpszWorkCurr = *lpszFolderDataCurr;
            lpszFolderDataCurr++;
            lpszWorkCurr++;
        }
    }

    /* Check for success */
    lFolderNameLen = StrlenNull(lpszWorkBuff);
    if(0 == lFolderNameLen)
    {
        if(lpTIB)
        {
            LogMsg(90, lpTIB->lTestNum, *lplpszFolderData);
            lpTIB->bCmdResult = FAILURE;
        }
        else
        {
            LogMsg(91, *lplpszFolderData);
        }
        return FAILURE;
    }

    *lplpszFolderName = CloneString(lpszWorkBuff);
    return SUCCESS;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ClearAttrs                                                           */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Clear the attribute buffers in the Main Control Block                */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     N/A                                                                  */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
ClearAttrs()
{
    memset(kpMCB->lpszCurrNameAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrFormatAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrOutputFormatAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrMQMDAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrRFH2Attr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrR1Attr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrR2Attr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrFromAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrToAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrIncrAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrCondAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrModeAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrIntervalAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrPosAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrLenAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrTypeAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrMsgSepAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrSegSepAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrElementSepAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrMinOccursAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrMaxOccursAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrCCSIDAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrEncodingAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrMsgSetNameAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrMsgNumAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrOffsetAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrJustifyAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrPadCharAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrEndAlignAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrCmqValuesAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrTruncateAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrListAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrDataVarAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrConnectionAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrOfAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrInAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrDelimiterAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrFileExtensionAttr, 0x00, ATTR_BUFF_LEN + 4);
    memset(kpMCB->lpszCurrRemAttr, 0x00, ATTR_BUFF_LEN + 4);
    kpMCB->eForType = F_UNKNOWN;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ClearRC                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Clear the Reason Code counters and arrays                            */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     N/A                                                                  */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
ClearRC()
{
    long lIdx;

    kpMCB->lContinueOnRCCount = 0;
    kpMCB->lBreakOnRCCount = 0;
    kpMCB->lSuppressRCCount = 0;

    for(lIdx = 0; lIdx < MAX_REASON_CODES; lIdx++)
    {
        kpMCB->lContinueOnRC[lIdx] = 0;
        kpMCB->lBreakOnRC[lIdx] = 0;
        kpMCB->lSuppressRC[lIdx] = 0;
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ClearCIB                                                             */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Release specific CIB strings that are re-used                        */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     N/A                                                                  */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
ClearCIB()
{
    /* Release any assigned strings */
    FreeIfNotNull((void**)&kpMCB->lpCIB->lpszFilePrefix);
    FreeIfNotNull((void**)&kpMCB->lpCIB->lpszParserName);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ClearCSE                                                             */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Release any strings assigned to the CSE control block                */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     N/A                                                                  */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
ClearCSE()
{
    /* Release any assigned strings */
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszBuffName);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszFileName);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszListFileName);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszRegexFileName);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszMQMDFileName);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszRFH2FileName);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszDir);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszData);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszQName);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszMqscReplyDynamicQName);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszQMgrName);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszDepth);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszInterval);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszIntervalMillis);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszTime);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszCommand);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszMaskFileName);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszCmdMaxWait);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszGmoOptions);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszPmoOptions);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszGmoMatchOptions);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszGmoWait);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszMsgId);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszCorrelId);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszGroupId);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszMsgSeqNumber);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszFormat);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszCompRFH2VarName1);
    FreeIfNotNull((void**)&kpMCB->lpCSE->lpszCompRFH2VarName2);

    /* Also clear the flags */
    kpMCB->lpCSE->bIgnoreRC = FALSE;
    kpMCB->lpCSE->bConvert = FALSE;
    kpMCB->lpCSE->bClear = FALSE;
    kpMCB->lpCSE->bLogAndContinue = FALSE;
    kpMCB->lpCSE->bIntervalInMsgId = FALSE;
    kpMCB->lpCSE->bIntervalInCorrelId = FALSE;
    kpMCB->lpCSE->bSyncpoint = FALSE;
    kpMCB->lpCSE->bSetIdentityContext = FALSE;
    kpMCB->lpCSE->bSetAllContext = FALSE;
    kpMCB->lpCSE->bRemoveDLH = FALSE;
    kpMCB->lpCSE->bRemoveXQH = FALSE;
    kpMCB->lpCSE->bAppend = FALSE;

    kpMCB->lpCSE->eBrowseType = B_NONE;
    kpMCB->lpCSE->eGroupType = G_NONE;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ClearPCDB                                                            */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Release any strings assigned to the parse char data buffer           */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     N/A                                                                  */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
ClearPCDB()
{
    FreeIfNotNull((void**)&kpMCB->lpszParseCharDataBuff);
    kpMCB->lParseCharDataLen = 0;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ClearLIB                                                             */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free RFH2 info block contents                                        */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpLIB    Pointer to the RFH2 info block                              */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
ClearLIB(ListInfoBlk* lpLIB)
{
    ListElementInfoBlk* lpLEB;
    ListElementInfoBlk* lpLEBFree;

    FreeIfNotNull((void**)&lpLIB->lpLEBPointerArray);
    FreeIfNotNull((void**)&lpLIB->lpszListFileName);
    FreeIfNotNull((void**)&lpLIB->lpszDir);
    FreeIfNotNull((void**)&lpLIB->lpszResListFileName);
    FreeIfNotNull((void**)&lpLIB->lpszResDir);

    /* Free ListElements if any */
    lpLEB = lpLIB->lpLEBHead;
    while(NULL != lpLEB)
    {
        lpLEBFree = lpLEB;
        lpLEB = (ListElementInfoBlk*)lpLEB->lpLEBNext;
        FreeLEB(&lpLEBFree);
    }

    /* Reset head and tail pointers */
    lpLIB->lpLEBHead = NULL;
    lpLIB->lpLEBTail = NULL;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ClearMIB                                                             */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Clear an MQMD info blocks contents                                   */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpMIB    Pointer to the MQMD info block                              */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
ClearMIB(MQMDInfoBlk* lpMIB)
{
    /* Free the MQMD info strings if assigned */
    FreeIfNotNull((void**)&lpMIB->lpszVersion);
    FreeIfNotNull((void**)&lpMIB->lpszReport);
    FreeIfNotNull((void**)&lpMIB->lpszMsgType);
    FreeIfNotNull((void**)&lpMIB->lpszExpiry);
    FreeIfNotNull((void**)&lpMIB->lpszFeedback);
    FreeIfNotNull((void**)&lpMIB->lpszEncoding);
    FreeIfNotNull((void**)&lpMIB->lpszCodedCharSetId);
    FreeIfNotNull((void**)&lpMIB->lpszFormat);
    FreeIfNotNull((void**)&lpMIB->lpszPriority);
    FreeIfNotNull((void**)&lpMIB->lpszPersistence);
    FreeIfNotNull((void**)&lpMIB->lpszMsgId);
    FreeIfNotNull((void**)&lpMIB->lpszCorrelId);
    FreeIfNotNull((void**)&lpMIB->lpszBackoutCount);
    FreeIfNotNull((void**)&lpMIB->lpszReplyToQ);
    FreeIfNotNull((void**)&lpMIB->lpszReplyToQMgr);
    FreeIfNotNull((void**)&lpMIB->lpszUserIdentifier);
    FreeIfNotNull((void**)&lpMIB->lpszAccountingToken);
    FreeIfNotNull((void**)&lpMIB->lpszApplIdentityData);
    FreeIfNotNull((void**)&lpMIB->lpszPutApplType);
    FreeIfNotNull((void**)&lpMIB->lpszPutApplName);
    FreeIfNotNull((void**)&lpMIB->lpszPutDate);
    FreeIfNotNull((void**)&lpMIB->lpszPutTime);
    FreeIfNotNull((void**)&lpMIB->lpszApplOriginData);
    FreeIfNotNull((void**)&lpMIB->lpszGroupId);
    FreeIfNotNull((void**)&lpMIB->lpszMsgSeqNumber);
    FreeIfNotNull((void**)&lpMIB->lpszOffset);
    FreeIfNotNull((void**)&lpMIB->lpszMsgFlags);
    FreeIfNotNull((void**)&lpMIB->lpszOriginalLength);
    FreeIfNotNull((void**)&lpMIB->lpszMQMDFileName);
    FreeIfNotNull((void**)&lpMIB->lpszDir);
    FreeIfNotNull((void**)&lpMIB->lpszResMQMDFileName);
    FreeIfNotNull((void**)&lpMIB->lpszResDir);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ClearRIB                                                             */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free RFH2 info block contents                                        */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpRIB    Pointer to the RFH2 info block                              */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
ClearRIB(RFH2InfoBlk* lpRIB)
{
    FolderInfoBlk* lpFIB;
    FolderInfoBlk* lpFIBFree;

    /* Free format name string */
    FreeIfNotNull((void**)&lpRIB->lpszFormat);
    FreeIfNotNull((void**)&lpRIB->lpszRFH2FileName);
    FreeIfNotNull((void**)&lpRIB->lpszDir);
    FreeIfNotNull((void**)&lpRIB->lpszResRFH2FileName);
    FreeIfNotNull((void**)&lpRIB->lpszResDir);

    lpRIB->lCodedCharSetId = 0;
    lpRIB->lEncoding = MQENC_NATIVE;
    lpRIB->bUseRIBEncodingAndCCSID = FALSE;

    /* Free folders if any */
    lpFIB = lpRIB->lpFIBHead;
    while(NULL != lpFIB)
    {
        lpFIBFree = lpFIB;
        lpFIB = (FolderInfoBlk*)lpFIB->lpFIBNext;
        FreeFIB(&lpFIBFree);
    }

    /* Reset head and tail pointers */
    lpRIB->lpFIBHead = NULL;
    lpRIB->lpFIBTail = NULL;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     AddAutoNumVar                                                        */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Add an automatic numeric variable to the var info block chain        */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpVIBHead     Pointer to head of variable info block chain pointer */
/*     lplpVIBTail     Pointer to tail of variable info block chain pointer */
/*     lpszAutoVarName Name of the automatic variable being added           */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
VarInfoBlk*
AddAutoNumVar(VarInfoBlk** lplpVIBHead,
              VarInfoBlk** lplpVIBTail, char* lpszAutoVarName)
{
    VarInfoBlk* lpVIB;

    lpVIB = malloc(sizeof(VarInfoBlk));
    if(! lpVIB)
    {
        ConsoleMsg(3, sizeof(VarInfoBlk), "VarInfoBlk", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
        return NULL;
    }

    memset(lpVIB, 0x00, sizeof(VarInfoBlk));
    lpVIB->lpVIBNext = NULL;
    lpVIB->lpVIBPrev = NULL;
    lpVIB->eVarType = V_NUM;
    lpVIB->lpszVarName = CloneString(lpszAutoVarName);
    lpVIB->lpszVarFormat = CloneString(kpszNUMFMT);
    lpVIB->lpszCurrStrValue = NULL;
    lpVIB->lpCtrlBlk = NULL;

    AddToTailOfList((LinkedListBlk**)lplpVIBHead,
                    (LinkedListBlk**)lplpVIBTail,
                    (LinkedListBlk*)lpVIB);

    return lpVIB;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     AddAutoStrVar                                                        */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Add an automatic string variable to the var info block chain         */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpVIBHead     Pointer to head of variable info block chain pointer */
/*     lplpVIBTail     Pointer to tail of variable info block chain pointer */
/*     lpszAutoVarName Name of the automatic variable being added           */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
VarInfoBlk*
AddAutoStrVar(VarInfoBlk** lplpVIBHead,
              VarInfoBlk** lplpVIBTail, char* lpszAutoVarName)
{
    VarInfoBlk* lpVIB;

    lpVIB = malloc(sizeof(VarInfoBlk));
    if(! lpVIB)
    {
        ConsoleMsg(3, sizeof(VarInfoBlk), "VarInfoBlk", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
        return NULL;
    }

    memset(lpVIB, 0x00, sizeof(VarInfoBlk));
    lpVIB->lpVIBNext = NULL;
    lpVIB->lpVIBPrev = NULL;
    lpVIB->eVarType = V_STR;
    lpVIB->lpszVarName = CloneString(lpszAutoVarName);
    lpVIB->lpszVarFormat = CloneString(kpszSTRFMT);
    lpVIB->lpszCurrStrValue = NULL;
    lpVIB->lpCtrlBlk = NULL;

    AddToTailOfList((LinkedListBlk**)lplpVIBHead,
                    (LinkedListBlk**)lplpVIBTail,
                    (LinkedListBlk*)lpVIB);

    return lpVIB;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     SetAutoVar                                                           */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Set the value of an automatic variable                               */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTIB           Pointer to the test info block                       */
/*     lpVIBHead       Head of variable info block chain                    */
/*     lpszAutoVarName Name of the automatic variable being set             */
/*     lpVoid          Void pointer to the object of interest               */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
SetAutoVar(TestInfoBlk* lpTIB, VarInfoBlk* lpVIBHead,
           char* lpszAutoVarName, void* lpVoid)
{
    VarInfoBlk* lpVIB;
    struct tm*  poTimeTm ;
    time_t*     lpTimeT;
    MQMD*       lpMsgDesc;
    long        bIsAutoVar;
    long        bTrimIt;
    char        lpszWorkBuff[WORK_BUFF_LEN + 4];

    /* Initialise local variables */
    bIsAutoVar = TRUE;
    bTrimIt = FALSE;
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);

    /* Find the variable by name */
    lpVIB = FindVarInfoBlk(lpVIBHead, lpszAutoVarName);
    if(! lpVIB)
    {
        LogMsg(77, lpszAutoVarName);
        if(NULL != lpTIB)
        {
            lpTIB->bCmdResult = FAILURE;
        }
        kpMCB->lProcessing = FAILURE;
        return;
    }

    /* Process the variable according to its name */
    /* NOTE localtime() is not thread safe. This is not a concern  */
    /* for this program in its current pseudo-threaded form but... */
    if(0 == memcmp(kpszYEAR4, lpszAutoVarName, strlen(kpszYEAR4) + 1))
    {
        lpTimeT = lpVoid;
        poTimeTm = localtime(lpTimeT);
        strftime(lpszWorkBuff, WORK_BUFF_LEN, "%Y\0", poTimeTm);
    }
    else if(0 == memcmp(kpszYEAR2, lpszAutoVarName, strlen(kpszYEAR2) + 1))
    {
        lpTimeT = lpVoid;
        poTimeTm = localtime(lpTimeT);
        strftime(lpszWorkBuff, WORK_BUFF_LEN, "%y\0", poTimeTm);
    }
    else if(0 == memcmp(kpszMONTH3, lpszAutoVarName, strlen(kpszMONTH3) + 1))
    {
        lpTimeT = lpVoid;
        poTimeTm = localtime(lpTimeT);
        strftime(lpszWorkBuff, WORK_BUFF_LEN, "%b\0", poTimeTm);
    }
    else if(0 == memcmp(kpszMONTH2, lpszAutoVarName, strlen(kpszMONTH2) + 1))
    {
        lpTimeT = lpVoid;
        poTimeTm = localtime(lpTimeT);
        strftime(lpszWorkBuff, WORK_BUFF_LEN, "%m\0", poTimeTm);
    }
    else if(0 == memcmp(kpszDAY3, lpszAutoVarName, strlen(kpszDAY3) + 1))
    {
        lpTimeT = lpVoid;
        poTimeTm = localtime(lpTimeT);
        strftime(lpszWorkBuff, WORK_BUFF_LEN, "%a\0", poTimeTm);
    }
    else if(0 == memcmp(kpszDAY2, lpszAutoVarName, strlen(kpszDAY2) + 1))
    {
        lpTimeT = lpVoid;
        poTimeTm = localtime(lpTimeT);
        strftime(lpszWorkBuff, WORK_BUFF_LEN, "%d\0", poTimeTm);
    }
    else if(0 == memcmp(kpszHOURS, lpszAutoVarName, strlen(kpszHOURS) + 1))
    {
        lpTimeT = lpVoid;
        poTimeTm = localtime(lpTimeT);
        strftime(lpszWorkBuff, WORK_BUFF_LEN, "%H\0", poTimeTm);
    }
    else if(0 == memcmp(kpszMINUTES, lpszAutoVarName, strlen(kpszMINUTES) + 1))
    {
        lpTimeT = lpVoid;
        poTimeTm = localtime(lpTimeT);
        strftime(lpszWorkBuff, WORK_BUFF_LEN, "%M\0", poTimeTm);
    }
    else if(0 == memcmp(kpszSECONDS, lpszAutoVarName, strlen(kpszSECONDS) + 1))
    {
        lpTimeT = lpVoid;
        poTimeTm = localtime(lpTimeT);
        strftime(lpszWorkBuff, WORK_BUFF_LEN, "%S\0", poTimeTm);
    }
    else if(0 == memcmp(kpszPID, lpszAutoVarName, strlen(kpszPID) + 1))
    {
        sprintf(lpszWorkBuff, "%d\0", getpid());
    }
    else if(0 == memcmp(kpszSCRIPTNAME, lpszAutoVarName, strlen(kpszSCRIPTNAME) + 1))
    {
        sprintf(lpszWorkBuff, "%s\0", kpMCB->lpszScriptName);
    }
    else if(0 == memcmp(kpszMSGID, lpszAutoVarName, strlen(kpszMSGID) + 1))
    {
        lpMsgDesc = lpVoid;
        if(0 == memcmp(kpszNULLS, lpMsgDesc->MsgId, MQ_MSG_ID_LENGTH))
        {
            strncpy(lpszWorkBuff, kpszMQMI_NONE, MQ_MSG_ID_LENGTH);
        }
        else
        {
            GenByte24Str(lpszWorkBuff, (char*)lpMsgDesc->MsgId, TRUE);
        }
    }
    else if(0 == memcmp(kpszCORRELID, lpszAutoVarName, strlen(kpszCORRELID) + 1))
    {
        lpMsgDesc = lpVoid;
        if(0 == memcmp(kpszNULLS, lpMsgDesc->CorrelId, MQ_CORREL_ID_LENGTH))
        {
            strncpy(lpszWorkBuff, kpszMQCI_NONE, MQ_CORREL_ID_LENGTH);
        }
        else
        {
            GenByte24Str(lpszWorkBuff, (char*)lpMsgDesc->CorrelId, TRUE);
        }
    }
    else if(0 == memcmp(kpszGROUPID, lpszAutoVarName, strlen(kpszGROUPID) + 1))
    {
        lpMsgDesc = lpVoid;
        if(0 == memcmp(kpszNULLS, lpMsgDesc->GroupId, MQ_GROUP_ID_LENGTH))
        {
            strncpy(lpszWorkBuff, kpszMQGI_NONE, MQ_GROUP_ID_LENGTH);
        }
        else
        {
            GenByte24Str(lpszWorkBuff, (char*)lpMsgDesc->GroupId, TRUE);
        }
    }
    else if(0 == memcmp(kpszMSGSEQNUMBER, lpszAutoVarName, strlen(kpszMSGSEQNUMBER) + 1))
    {
        lpMsgDesc = lpVoid;
        sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->MsgSeqNumber);
    }
    else if(0 == memcmp(kpszREPLYTOQ, lpszAutoVarName, strlen(kpszREPLYTOQ) + 1))
    {
        lpMsgDesc = lpVoid;
        memcpy(lpszWorkBuff, lpMsgDesc->ReplyToQ, MQ_Q_NAME_LENGTH);
        bTrimIt = TRUE;
    }
    else if(0 == memcmp(kpszREPLYTOQMGR, lpszAutoVarName, strlen(kpszREPLYTOQMGR) + 1))
    {
        lpMsgDesc = lpVoid;
        memcpy(lpszWorkBuff, lpMsgDesc->ReplyToQMgr, MQ_Q_MGR_NAME_LENGTH);
        bTrimIt = TRUE;
    }
    else if(0 == memcmp(kpszMSGTYPE, lpszAutoVarName, strlen(kpszMSGTYPE) + 1))
    {
        lpMsgDesc = lpVoid;
        sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->MsgType);
    }
    else if(0 == memcmp(kpszPRIORITY, lpszAutoVarName, strlen(kpszPRIORITY) + 1))
    {
        lpMsgDesc = lpVoid;
        sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->Priority);
    }
    else if(0 == memcmp(kpszPERSISTENCE, lpszAutoVarName, strlen(kpszPERSISTENCE) + 1))
    {
        lpMsgDesc = lpVoid;
        sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->Persistence);
    }
    else if(NULL != lpTIB)
    {
        if(0 == memcmp(kpszLASTCC, lpszAutoVarName, strlen(kpszLASTCC) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lLastCC);
        }
        else if(0 == memcmp(kpszLASTRC, lpszAutoVarName, strlen(kpszLASTRC) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lLastRC);
        }
        else if(0 == memcmp(kpszLASTDATALEN, lpszAutoVarName, strlen(kpszLASTDATALEN) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lLastDataLen);
        }
        else if(0 == memcmp(kpszLASTFILE, lpszAutoVarName, strlen(kpszLASTFILE) + 1))
        {
            sprintf(lpszWorkBuff, "%s\0", lpTIB->lpszLastFile);
        }
        else if(0 == memcmp(kpszLASTDIR, lpszAutoVarName, strlen(kpszLASTDIR) + 1))
        {
            sprintf(lpszWorkBuff, "%s\0", lpTIB->lpszLastDir);
        }
        else if(0 == memcmp(kpszLASTQ, lpszAutoVarName, strlen(kpszLASTQ) + 1))
        {
            sprintf(lpszWorkBuff, "%s\0", lpTIB->lpszLastQ);
        }
        else if(0 == memcmp(kpszLASTQMGR, lpszAutoVarName, strlen(kpszLASTQMGR) + 1))
        {
            sprintf(lpszWorkBuff, "%s\0", lpTIB->lpszLastQMgr);
        }
        else if(0 == memcmp(kpszTESTCURRENTMSGRATE, lpszAutoVarName, strlen(kpszTESTCURRENTMSGRATE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestCurrentMsgRate);
        }
        else if(0 == memcmp(kpszTESTTOTALGETFILE, lpszAutoVarName, strlen(kpszTESTTOTALGETFILE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestTotalGetFile);
        }
        else if(0 == memcmp(kpszTESTSUCCESSFULGETFILE, lpszAutoVarName, strlen(kpszTESTSUCCESSFULGETFILE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestSuccessfulGetFile);
        }
        else if(0 == memcmp(kpszTESTFAILEDGETFILE, lpszAutoVarName, strlen(kpszTESTFAILEDGETFILE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestFailedGetFile);
        }
        else if(0 == memcmp(kpszTESTTOTALPUTFILE, lpszAutoVarName, strlen(kpszTESTTOTALPUTFILE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestTotalPutFile);
        }
        else if(0 == memcmp(kpszTESTSUCCESSFULPUTFILE, lpszAutoVarName, strlen(kpszTESTSUCCESSFULPUTFILE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestSuccessfulPutFile);
        }
        else if(0 == memcmp(kpszTESTFAILEDPUTFILE, lpszAutoVarName, strlen(kpszTESTFAILEDPUTFILE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestFailedPutFile);
        }
        else if(0 == memcmp(kpszTESTTOTALGETMSG, lpszAutoVarName, strlen(kpszTESTTOTALGETMSG) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestTotalGetMsg);
        }
        else if(0 == memcmp(kpszTESTSUCCESSFULGETMSG, lpszAutoVarName, strlen(kpszTESTSUCCESSFULGETMSG) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestSuccessfulGetMsg);
        }
        else if(0 == memcmp(kpszTESTFAILEDGETMSG, lpszAutoVarName, strlen(kpszTESTFAILEDGETMSG) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestFailedGetMsg);
        }
        else if(0 == memcmp(kpszTESTTOTALPUTMSG, lpszAutoVarName, strlen(kpszTESTTOTALPUTMSG) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestTotalPutMsg);
        }
        else if(0 == memcmp(kpszTESTSUCCESSFULPUTMSG, lpszAutoVarName, strlen(kpszTESTSUCCESSFULPUTMSG) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestSuccessfulPutMsg);
        }
        else if(0 == memcmp(kpszTESTFAILEDPUTMSG, lpszAutoVarName, strlen(kpszTESTFAILEDPUTMSG) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestFailedPutMsg);
        }
        else if(0 == memcmp(kpszTESTTOTALCOMPARE, lpszAutoVarName, strlen(kpszTESTTOTALCOMPARE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestTotalCompare);
        }
        else if(0 == memcmp(kpszTESTSUCCESSFULCOMPARE, lpszAutoVarName, strlen(kpszTESTSUCCESSFULCOMPARE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestSuccessfulCompare);
        }
        else if(0 == memcmp(kpszTESTFAILEDCOMPARE, lpszAutoVarName, strlen(kpszTESTFAILEDCOMPARE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", lpTIB->lTestFailedCompare);
        }
        else if(0 == memcmp(kpszSCRIPTTOTALGETFILE, lpszAutoVarName, strlen(kpszSCRIPTTOTALGETFILE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", kpMCB->lScriptTotalGetFile);
        }
        else if(0 == memcmp(kpszSCRIPTSUCCESSFULGETFILE, lpszAutoVarName, strlen(kpszSCRIPTSUCCESSFULGETFILE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", kpMCB->lScriptSuccessfulGetFile);
        }
        else if(0 == memcmp(kpszSCRIPTFAILEDGETFILE, lpszAutoVarName, strlen(kpszSCRIPTFAILEDGETFILE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", kpMCB->lScriptFailedGetFile);
        }
        else if(0 == memcmp(kpszSCRIPTTOTALPUTFILE, lpszAutoVarName, strlen(kpszSCRIPTTOTALPUTFILE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", kpMCB->lScriptTotalPutFile);
        }
        else if(0 == memcmp(kpszSCRIPTSUCCESSFULPUTFILE, lpszAutoVarName, strlen(kpszSCRIPTSUCCESSFULPUTFILE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", kpMCB->lScriptSuccessfulPutFile);
        }
        else if(0 == memcmp(kpszSCRIPTFAILEDPUTFILE, lpszAutoVarName, strlen(kpszSCRIPTFAILEDPUTFILE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", kpMCB->lScriptFailedPutFile);
        }
        else if(0 == memcmp(kpszSCRIPTTOTALGETMSG, lpszAutoVarName, strlen(kpszSCRIPTTOTALGETMSG) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", kpMCB->lScriptTotalGetMsg);
        }
        else if(0 == memcmp(kpszSCRIPTSUCCESSFULGETMSG, lpszAutoVarName, strlen(kpszSCRIPTSUCCESSFULGETMSG) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", kpMCB->lScriptSuccessfulGetMsg);
        }
        else if(0 == memcmp(kpszSCRIPTFAILEDGETMSG, lpszAutoVarName, strlen(kpszSCRIPTFAILEDGETMSG) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", kpMCB->lScriptFailedGetMsg);
        }
        else if(0 == memcmp(kpszSCRIPTTOTALPUTMSG, lpszAutoVarName, strlen(kpszSCRIPTTOTALPUTMSG) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", kpMCB->lScriptTotalPutMsg);
        }
        else if(0 == memcmp(kpszSCRIPTSUCCESSFULPUTMSG, lpszAutoVarName, strlen(kpszSCRIPTSUCCESSFULPUTMSG) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", kpMCB->lScriptSuccessfulPutMsg);
        }
        else if(0 == memcmp(kpszSCRIPTFAILEDPUTMSG, lpszAutoVarName, strlen(kpszSCRIPTFAILEDPUTMSG) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", kpMCB->lScriptFailedPutMsg);
        }
        else if(0 == memcmp(kpszSCRIPTTOTALCOMPARE, lpszAutoVarName, strlen(kpszSCRIPTTOTALCOMPARE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", kpMCB->lScriptTotalCompare);
        }
        else if(0 == memcmp(kpszSCRIPTSUCCESSFULCOMPARE, lpszAutoVarName, strlen(kpszSCRIPTSUCCESSFULCOMPARE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", kpMCB->lScriptSuccessfulCompare);
        }
        else if(0 == memcmp(kpszSCRIPTFAILEDCOMPARE, lpszAutoVarName, strlen(kpszSCRIPTFAILEDCOMPARE) + 1))
        {
            sprintf(lpszWorkBuff, "%d\0", kpMCB->lScriptFailedCompare);
        }
        else
        {
            bIsAutoVar = FALSE;
        }
    }
    else
    {
        bIsAutoVar = FALSE;
    }

    /* If its a recognised automatic variable copy the new value */
    if(bIsAutoVar)
    {
        /* Replacing the current value with a new one so free it */
        FreeIfNotNull((void**)&lpVIB->lpszCurrStrValue);
        
        /* Clone and assign the new value */
        if(bTrimIt)
        {
            lpVIB->lpszCurrStrValue = TrimString(lpszWorkBuff);
        }
        else
        {
            lpVIB->lpszCurrStrValue = CloneString(lpszWorkBuff);
        }
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     InitTimePidAutoVars                                                  */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Set the values of the time and pid based automatic variables         */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTIB           Pointer to the test info block                       */
/*     lpVIBHead       Head of variable info block chain                    */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
InitTimePidAutoVars(TestInfoBlk* lpTIB, VarInfoBlk* lpVIBHead)
{
    time_t          oCurrTime;

    /* Get the current time value */
    time(&oCurrTime);

    /* Now set all the time related automatic variables */
    SetAutoVar(lpTIB, lpVIBHead, kpszYEAR4, &oCurrTime);
    SetAutoVar(lpTIB, lpVIBHead, kpszYEAR2, &oCurrTime);
    SetAutoVar(lpTIB, lpVIBHead, kpszMONTH3, &oCurrTime);
    SetAutoVar(lpTIB, lpVIBHead, kpszMONTH2, &oCurrTime);
    SetAutoVar(lpTIB, lpVIBHead, kpszDAY3, &oCurrTime);
    SetAutoVar(lpTIB, lpVIBHead, kpszDAY2, &oCurrTime);
    SetAutoVar(lpTIB, lpVIBHead, kpszHOURS, &oCurrTime);
    SetAutoVar(lpTIB, lpVIBHead, kpszMINUTES, &oCurrTime);
    SetAutoVar(lpTIB, lpVIBHead, kpszSECONDS, &oCurrTime);
    SetAutoVar(lpTIB, lpVIBHead, kpszPID, NULL);
}

/*==========================================================================*/
/* FUNCTION:                                                                */
/*     OpenQueue                                                            */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Open an MQ queue. First check to see if we have already opened it.   */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpCDB            Pointer to the connection definition block          */
/*     lpQIB            Pointer to first Queue Info Block in the chain      */
/*     lpszTargQName    Name of the target queue to be opened               */
/*     lpszTargQMgrName Name of the target queue manager                    */ 
/*     lpszTargDynQName Name of the target dynamic queue if dynamic         */
/*     bOutputOnly      Only open queue for output                          */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to the QueueInfoBlk NULL if not able to create one           */
/*==========================================================================*/
QueueInfoBlk*
OpenQueue(TestInfoBlk* lpTIB, ConnectionDefBlk* lpCDB, QueueInfoBlk* lpQIB,
          char* lpszTargQName, char* lpszTargQMgrName, char* lpszTargDynQName,
          long bOutputOnly)
{
    enum eSetContextTypeEnum eSetContextType;
    CommandCtrlBlk*          lpCCBCheck;
    PutMsgBlk*               lpPMBCheck;
    char*                    lpszResQNameCheck;
    char*                    lpszResQMgrNameCheck;
    char*                    lpszResConnectionNameCheck;
    char                     lpszResolvedQNameBuff[MQ_Q_NAME_LENGTH + 4];
    long                     lTargQNameLen;
    long                     lQIBTargQNameLen;
    long                     lResQNameCheckLen;
    long                     lTargQMgrNameLen;
    long                     lQIBTargQMgrNameLen;
    long                     lResQMgrNameCheckLen;
    long                     lTargDynQNameLen;
    long                     lQIBTargDynQNameLen;
    long                     lConnQMgrNameLen;
    long                     lQIBConnQMgrNameLen;
    long                     lCDBConnectionNameLen;
    long                     lResConnectionNameCheckLen;
    MQOD                     oODQueue = {MQOD_DEFAULT};
    MQHOBJ                   hQ;
    MQLONG                   lOpenOpts;
    MQLONG                   lCC;
    MQLONG                   lRC;

    /* Check we got the required parameters */
    if((NULL == lpszTargQName) || (NULL == lpszTargQMgrName))
    {
        SetTIBCodes(lpTIB, MQCC_FAILED, MQRC_UNKNOWN_OBJECT_NAME);
        return NULL;
    }

    /* Length of the queue name cannot be an empty string */
    lTargQNameLen = StrlenNull(lpszTargQName);
    if(0 == lTargQNameLen)
    {
        SetTIBCodes(lpTIB, MQCC_FAILED, MQRC_UNKNOWN_OBJECT_NAME);
        return NULL;
    }

    /* Length of the queue manager name */
    lTargQMgrNameLen = StrlenNull(lpszTargQMgrName);

    /* Length of the dynamic queue name. */
    /* An empty string if the queue is not dynamic */
    lTargDynQNameLen = StrlenNull(lpszTargDynQName);

    /* Length of the connection definition queue manager name */
    lConnQMgrNameLen = StrlenNull(lpCDB->lpszConnQMgrName);

    /* See if we already have the queue open */
    while(NULL != lpQIB)
    {
        lQIBTargQNameLen = StrlenNull(lpQIB->lpszTargQName);
        lQIBTargQMgrNameLen = StrlenNull(lpQIB->lpszTargQMgrName);
        lQIBTargDynQNameLen = StrlenNull(lpQIB->lpszTargDynQName);
        lQIBConnQMgrNameLen = StrlenNull(lpQIB->lpszConnQMgrName);
        if((lTargQNameLen == lQIBTargQNameLen) &&
           (lTargQMgrNameLen == lQIBTargQMgrNameLen) &&
           (lTargDynQNameLen == lQIBTargDynQNameLen) &&
           (lConnQMgrNameLen == lQIBConnQMgrNameLen))
        {
            if((0 == memcmp(lpszTargQName, lpQIB->lpszTargQName, lTargQNameLen)) &&
               (0 == memcmp(lpszTargQMgrName, lpQIB->lpszTargQMgrName, lTargQMgrNameLen)) &&
               (0 == memcmp(lpszTargDynQName, lpQIB->lpszTargDynQName, lTargDynQNameLen)) &&
               (0 == memcmp(lpCDB->lpszConnQMgrName, lpQIB->lpszConnQMgrName, lConnQMgrNameLen)))
            {
                return lpQIB;
            }
        }
        lpQIB = (QueueInfoBlk*)lpQIB->lpQIBNext;
    }

    /* Get the SetContextType value for this queue by working through  */
    /* the list of commands for this test looking for PutMsg commands  */
    /* which specify a set context option and have the same queue name */
    lpszResQNameCheck = NULL;
    lpszResQMgrNameCheck = NULL;
    lpszResConnectionNameCheck = NULL;
    eSetContextType = X_NONE;
    lpCCBCheck = lpTIB->lpCCBHead;
    while(NULL != lpCCBCheck)
    {
        /* Is it a PutMsg command */
        if(C_PUTMSG == lpCCBCheck->eCommandType)
        {
            lpPMBCheck = (PutMsgBlk*)lpCCBCheck;
            /* Do we need some kind of set context authority */
            if((TRUE == lpPMBCheck->bSetIdentityContext) || (TRUE == lpPMBCheck->bSetAllContext))
            {
                /* After resolving values see if its the same queue */
                /* with the same connection name                    */
                ResolveValue(lpTIB, &lpszResQNameCheck, &lpPMBCheck->lpszQName, TRUE);
                ResolveValue(lpTIB, &lpszResQMgrNameCheck, &lpPMBCheck->lpszQMgrName, TRUE);
                ResolveValue(lpTIB, &lpszResConnectionNameCheck, &lpPMBCheck->lpszConnectionName, TRUE);
                lResQNameCheckLen = StrlenNull(lpszResQNameCheck);
                lResQMgrNameCheckLen = StrlenNull(lpszResQMgrNameCheck);
                lCDBConnectionNameLen = StrlenNull(lpCDB->lpszConnectionName);
                lResConnectionNameCheckLen = StrlenNull(lpszResConnectionNameCheck);
                if((lTargQNameLen == lResQNameCheckLen) &&
                   (lTargQMgrNameLen == lResQMgrNameCheckLen) &&
                   (lCDBConnectionNameLen == lResConnectionNameCheckLen))
                {
                    if((0 == memcmp(lpszTargQName, lpszResQNameCheck, lTargQNameLen)) &&
                       (0 == memcmp(lpszTargQMgrName, lpszResQMgrNameCheck, lTargQMgrNameLen)) &&
                       (0 == memcmp(lpCDB->lpszConnectionName, lpszResConnectionNameCheck, lCDBConnectionNameLen)))
                    {
                        if(TRUE == lpPMBCheck->bSetIdentityContext)
                        {
                            /* If already at SetAllContext leave things alone */
                            if(X_SETALL != eSetContextType)
                            {
                                eSetContextType = X_SETIDENTITY;
                            }
                        }
                        if(TRUE == lpPMBCheck->bSetAllContext)
                        {
                            eSetContextType = X_SETALL;
                        }
                    }
                }
            }
        }
        lpCCBCheck = (CommandCtrlBlk*)lpCCBCheck->lpCCBNext;
    }
    /* Free up value resolution check work areas */
    FreeIfNotNull((void**)&lpszResQNameCheck);
    FreeIfNotNull((void**)&lpszResQMgrNameCheck);
    FreeIfNotNull((void**)&lpszResConnectionNameCheck);

    /* Couldn't find a matching QIB so lets open the queue    */
    /* Set the fields in the object descriptor for the MQOPEN */
    strncpy(oODQueue.ObjectName, lpszTargQName, MQ_Q_NAME_LENGTH);
    strncpy(oODQueue.ObjectQMgrName, lpszTargQMgrName, MQ_Q_MGR_NAME_LENGTH);
    if(0 < StrlenNull(lpszTargDynQName))
    {
        strncpy(oODQueue.DynamicQName, lpszTargDynQName, MQ_Q_NAME_LENGTH);
    }

    /* Set the object descriptor to the current version */
    oODQueue.Version = MQOD_CURRENT_VERSION;

    /* Open the queue for output, shared input, browsing and inquiry */
    /* and the appropriate level of context setting capability       */
    lOpenOpts = 0;
    if(TRUE == bOutputOnly)
    {
        lOpenOpts = MQOO_FAIL_IF_QUIESCING + MQOO_OUTPUT;
    }
    else
    {
        switch(eSetContextType)
        {
        case X_NONE:
            lOpenOpts = MQOO_FAIL_IF_QUIESCING + MQOO_OUTPUT +
                        MQOO_INPUT_SHARED + MQOO_BROWSE + MQOO_INQUIRE;
            break;
        case X_SETIDENTITY:
            lOpenOpts = MQOO_FAIL_IF_QUIESCING + MQOO_OUTPUT +
                        MQOO_INPUT_SHARED + MQOO_BROWSE + MQOO_INQUIRE +
                        MQOO_SET_IDENTITY_CONTEXT;
            break;
        case X_SETALL:
            lOpenOpts = MQOO_FAIL_IF_QUIESCING + MQOO_OUTPUT +
                        MQOO_INPUT_SHARED + MQOO_BROWSE + MQOO_INQUIRE +
                        MQOO_SET_ALL_CONTEXT;
            break;
        }
    }

    /* Open the queue and log the result */
    MQOPEN(lpCDB->hConn, &oODQueue, lOpenOpts, &hQ, &lCC, &lRC);
    if(MQRC_NONE == lRC)
    {
        if(0 == StrlenNull(lpszTargDynQName))
        {
            LogMsg(54, lpTIB->lTestNum, lpszTargQName, lpszTargQMgrName, lpCDB->lpszConnQMgrName, lCC, lRC);
        }
        else
        {
            LogMsg(249, lpTIB->lTestNum, lpszTargQName, lpszTargQMgrName, lpszTargDynQName, lpCDB->lpszConnQMgrName, lCC, lRC);
        }
    }
    else
    {
        /* Can't open a remote queue for input, browsing or inquiry retry */
        /* the MQOPEN for just output. If its a cluster queue that is not */
        /* defined locally trying to open it for input and browsing will  */
        /* result in a reason code of MQRC_UNKNOWN_OBJECT_NAME            */
        if((MQRC_OPTION_NOT_VALID_FOR_TYPE == lRC) ||
           (MQRC_UNKNOWN_OBJECT_NAME == lRC))
        {
            switch(eSetContextType)
            {
            case X_NONE:
                lOpenOpts = MQOO_FAIL_IF_QUIESCING + MQOO_OUTPUT;
                break;
            case X_SETIDENTITY:
                lOpenOpts = MQOO_FAIL_IF_QUIESCING + MQOO_OUTPUT +
                            MQOO_SET_IDENTITY_CONTEXT;
                break;
            case X_SETALL:
                lOpenOpts = MQOO_FAIL_IF_QUIESCING + MQOO_OUTPUT +
                            MQOO_SET_ALL_CONTEXT;
                break;
            }
            MQOPEN(lpCDB->hConn, &oODQueue, lOpenOpts, &hQ, &lCC, &lRC);
            if(MQRC_NONE == lRC)
            {
                LogMsg(123, lpTIB->lTestNum, lpszTargQName, lpszTargQMgrName, lpCDB->lpszConnQMgrName, lCC, lRC);
            }
            else
            {
                LogMsg(55, lpTIB->lTestNum, lpszTargQName, lpszTargQMgrName, lpCDB->lpszConnQMgrName, lCC, lRC);
                SetTIBCodes(lpTIB, lCC, lRC);
                return NULL;
            }
        }
        else
        {
            if(0 == StrlenNull(lpszTargDynQName))
            {
                LogMsg(55, lpTIB->lTestNum, lpszTargQName, lpszTargQMgrName, lpCDB->lpszConnQMgrName, lCC, lRC);
            }
            else
            {
                LogMsg(250, lpTIB->lTestNum, lpszTargQName, lpszTargQMgrName, lpszTargDynQName, lpCDB->lpszConnQMgrName, lCC, lRC);
            }
            SetTIBCodes(lpTIB, lCC, lRC);
            return NULL;
        }
    }

    /* Opened the queue successfully need a QIB for it */
    lpQIB = malloc(sizeof(QueueInfoBlk));
    if(lpQIB)
    {
        memset(lpQIB, 0x00, sizeof(QueueInfoBlk));
        lpQIB->lpQIBNext = NULL;
        lpQIB->lpQIBPrev = NULL;
        lpQIB->lpszTargQName = CloneString(lpszTargQName);
        lpQIB->lpszTargQMgrName = CloneString(lpszTargQMgrName);
        lpQIB->lpszTargDynQName = CloneString(lpszTargDynQName);
        lpQIB->lpszConnQMgrName = CloneString(lpCDB->lpszConnQMgrName);
        lpQIB->hConn = lpCDB->hConn;
        lpQIB->hQ = hQ;

        /* Clear the buffer for the resolved queue name */
        memset(lpszResolvedQNameBuff, 0x00, MQ_Q_NAME_LENGTH + 4);
        strncpy(lpszResolvedQNameBuff, oODQueue.ObjectName, MQ_Q_NAME_LENGTH);
        lpQIB->lpszResolvedQName = TrimString(lpszResolvedQNameBuff);

        AddToTailOfList((LinkedListBlk**)&lpTIB->lpQIBHead,
                        (LinkedListBlk**)&lpTIB->lpQIBTail,
                        (LinkedListBlk*)lpQIB);
    }
    else
    {
        LogMsg(3, sizeof(QueueInfoBlk), "QueueInfoBlk", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
        SetTIBCodes(lpTIB, MTCC_FAIL, MTRC_MALLOC_FAIL);
        return NULL;
    }
    return lpQIB;
}

/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FindVarInfoBlk                                                       */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Find a Variable Info Block in the linked list                        */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpVIB           Pointer to first Var Info Block in the chain         */
/*     lpszFindVarName Name of the variable to find                         */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to the VarInfoBlk NULL if not found                          */
/*==========================================================================*/
VarInfoBlk*
FindVarInfoBlk(VarInfoBlk* lpVIB, char* lpszFindVarName)
{
    long lFindVarNameLen;
    long lVIBVarNameLen;

    /* If no name is supplied then fail the test and return NULL */
    if(NULL == lpszFindVarName)
    {
        return NULL;
    }

    /* Empty strings not allowed */
    lFindVarNameLen = StrlenNull(lpszFindVarName);
    if(0 == lFindVarNameLen)
    {
        return NULL;
    }

    /* See if we can find a VIB for a variable with this name */
    while(NULL != lpVIB)
    {
        lVIBVarNameLen = StrlenNull(lpVIB->lpszVarName);
        if(lFindVarNameLen == lVIBVarNameLen)
        {
            if(0 == memcmp(lpszFindVarName, lpVIB->lpszVarName,
                           lFindVarNameLen))
            {
                return lpVIB;
            }
        }
        lpVIB = (VarInfoBlk*)lpVIB->lpVIBNext;
    }

    return NULL;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     CheckAndAddVIB                                                       */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Check to see if a VarInfoBlock exists if so just return it. If not   */
/*     create one, initialise it and add it to the tests linked list.       */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTIB           Pointer to a Test Info Block                         */
/*     eVarType        Variable type                                        */
/*     lpszVarName     Pointer to the unresolved variable name              */
/*     lpszResVarName  Pointer to the resolved varaible name                */
/*     lpszFormat      Format specifier for numbers and strings             */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to a BuffInfoBlk                                             */
/*==========================================================================*/
VarInfoBlk*
CheckAndAddVIB(TestInfoBlk* lpTIB, enum eVariableEnum eVarType,
               char* lpszVarName, char* lpszResVarName,
               char* lpszVarFormat)
{
    VarInfoBlk*  lpVIB;
    VarInfoBlk*  lpVIBDIB;
    ListInfoBlk* lpLIBDIB;
    ListInfoBlk* lpLIBParsed;
    MQMDInfoBlk* lpMIBDIB;
    MQMDInfoBlk* lpMIBParsed;
    RFH2InfoBlk* lpRIBDIB;
    RFH2InfoBlk* lpRIBParsed;

    /* If no names are supplied then fail the test and return NULL */
    if(NULL == lpszVarName)
    {
        lpTIB->bCmdResult = FAILURE;
        return NULL;
    }
    if(NULL == lpszResVarName)
    {
        lpTIB->bCmdResult = FAILURE;
        return NULL;
    }

    /* If its an empty string then fail the test and return NULL */
    if(0 == StrlenNull(lpszVarName))
    {
        lpTIB->bCmdResult = FAILURE;
        return NULL;
    }
    if(0 == StrlenNull(lpszResVarName))
    {
        lpTIB->bCmdResult = FAILURE;
        return NULL;
    }

    /* If we can find it just return the pointer */
    lpVIB = FindVarInfoBlk(lpTIB->lpVIBHead, lpszResVarName);
    if(lpVIB)
    {
        /* If the VRB has a format replace the VIB's format with it */
        if(lpszVarFormat)
        {
            if(0 < StrlenNull(lpszVarFormat))
            {
                 FreeIfNotNull((void**)&lpVIB->lpszVarFormat);
                 lpVIB->lpszVarFormat = CloneString(lpszVarFormat);
            }
        }
        return lpVIB;
    }

    /* Create a new VIB */
    lpVIB = CreateVarInfoBlk(lpszResVarName);
    /* If not enough memory report it and fail */
    if(! lpVIB)
    {
        switch(eVarType)
        {
        case V_NUM:
            ConsoleMsg(26, kpszNUM, lpszResVarName);
            break;
        case V_DOUBLE:
            ConsoleMsg(26, kpszDOUBLE, lpszResVarName);
            break;
        case V_STR:
            ConsoleMsg(26, kpszSTR, lpszResVarName);
            break;
        case V_RANDOMNUM:
            ConsoleMsg(26, kpszRANDOMNUM, lpszResVarName);
            break;
        case V_LIST:
            ConsoleMsg(26, kpszLIST, lpszResVarName);
            break;
        case V_MQMD:
            ConsoleMsg(26, kpszMQMD, lpszResVarName);
            break;
        case V_RFH2:
            ConsoleMsg(26, kpszRFH2, lpszResVarName);
            break;
        }
        kpMCB->lProcessing = FAILURE;
        lpTIB->bCmdResult = FAILURE;
        return NULL;
    }

    /* See if we can find a default VIB for initialisation */
    lpVIBDIB = FindVarInfoBlk(kpMCB->lpDIB->lpVIBHead, lpszResVarName);
    if(! lpVIBDIB)
    {
        /* Try using the unresolved name. This is done so a */
        /* subscripted variable can have a default value    */
        lpVIBDIB = FindVarInfoBlk(kpMCB->lpDIB->lpVIBHead, lpszVarName);
    }

    /* If we have a default VIB use it for initialisation */
    /* otherwise generate an empty VIB                    */
    if(lpVIBDIB)
    {
        switch(eVarType)
        {
        case V_NUM:
            lpVIB->eVarType = V_NUM;
            /* Free any existing format string */
            FreeIfNotNull((void**)&lpVIB->lpszVarFormat);
            /* Clone values from the default VIB */
            lpVIB->lpszVarFormat = CloneString(lpVIBDIB->lpszVarFormat);
            lpVIB->lpszCurrStrValue = CloneString(lpVIBDIB->lpszCurrStrValue);
            break;
        case V_DOUBLE:
            lpVIB->eVarType = V_DOUBLE;
            /* Free any existing format string */
            FreeIfNotNull((void**)&lpVIB->lpszVarFormat);
            /* Clone values from the default VIB */
            lpVIB->lpszVarFormat = CloneString(lpVIBDIB->lpszVarFormat);
            lpVIB->lpszCurrStrValue = CloneString(lpVIBDIB->lpszCurrStrValue);
            break;
        case V_STR:
            lpVIB->eVarType = V_STR;
            /* Free any existing format string */
            FreeIfNotNull((void**)&lpVIB->lpszVarFormat);
            /* Clone values from the default VIB */
            lpVIB->lpszVarFormat = CloneString(lpVIBDIB->lpszVarFormat);
            lpVIB->lpszCurrStrValue = CloneString(lpVIBDIB->lpszCurrStrValue);
            break;
        case V_RANDOMNUM:
            lpVIB->eVarType = V_RANDOMNUM;
            /* Free any existing format string */
            FreeIfNotNull((void**)&lpVIB->lpszVarFormat);
            /* Clone values from the default VIB */
            lpVIB->lpszVarFormat = CloneString(lpVIBDIB->lpszVarFormat);
            lpVIB->lpszCurrStrValue = CloneString(lpVIBDIB->lpszCurrStrValue);
            break;
        case V_LIST:
            lpVIB->eVarType = V_LIST;
            /* Get a pointer to the defaults LIB info if any */
            lpLIBDIB = lpVIBDIB->lpCtrlBlk;
            if(lpLIBDIB)
            {
                /* Work out the ListFile name if any */
                ResolveValue(lpTIB, &lpLIBDIB->lpszResListFileName,
                             &lpLIBDIB->lpszListFileName, TRUE);
                ResolveValue(lpTIB, &lpLIBDIB->lpszResDir, &lpLIBDIB->lpszDir, TRUE);
                if(FAILURE == lpTIB->bCmdResult)
                {
                    return NULL;
                }
                if(0 == StrlenNull(lpLIBDIB->lpszResListFileName))
                {
                    /* If the defaults LIB doesnt have a ListFile to load */
                    /* just use the defaults LIB values for the new LIB   */
                    CloneLIB((ListInfoBlk**)&lpVIB->lpCtrlBlk, lpLIBDIB);
                }
                else
                {
                    /* Load a LIB with the values in the ListFile */
                    lpLIBParsed = NULL;
                    LoadLIBFromFile(lpTIB, &lpLIBParsed,
                                    lpLIBDIB->lpszResListFileName,
                                    lpLIBDIB->lpszResDir);
                    if(FAILURE == lpTIB->bCmdResult)
                    {
                        return NULL;
                    }
                    /* Override any ListFile values with those */
                    /* in the defaults LIB                     */
                    OverrideLIB(lpLIBParsed, lpLIBDIB);
                    /* Point to the LIB we just created and modified */
                    lpVIB->lpCtrlBlk = lpLIBParsed;
                }
            }
            else
            {
                /* Shouldnt happen but just use the NullLIB */
                CloneLIB((ListInfoBlk**)&lpVIB->lpCtrlBlk, kpMCB->lpNullLIB);
            }
            break;
        case V_MQMD:
            lpVIB->eVarType = V_MQMD;
            /* Get a pointer to the defaults MIB info if any */
            lpMIBDIB = lpVIBDIB->lpCtrlBlk;
            if(lpMIBDIB)
            {
                /* Work out the MQMDFile name if any */
                ResolveValue(lpTIB, &lpMIBDIB->lpszResMQMDFileName,
                             &lpMIBDIB->lpszMQMDFileName, TRUE);
                ResolveValue(lpTIB, &lpMIBDIB->lpszResDir, &lpMIBDIB->lpszDir, TRUE);
                if(FAILURE == lpTIB->bCmdResult)
                {
                    return NULL;
                }
                if(0 == StrlenNull(lpMIBDIB->lpszResMQMDFileName))
                {
                    /* If the defaults MIB doesnt have a MQMDFile to load */
                    /* just use the DIB MIB values for the new MIB        */
                    CloneMIB((MQMDInfoBlk**)&lpVIB->lpCtrlBlk, lpMIBDIB);
                }
                else
                {
                    /* Load a MIB with the values in the MQMDFile */
                    lpMIBParsed = NULL;
                    LoadMIBFromFile(lpTIB, &lpMIBParsed,
                                    lpMIBDIB->lpszResMQMDFileName,
                                    lpMIBDIB->lpszResDir);
                    if(FAILURE == lpTIB->bCmdResult)
                    {
                        return NULL;
                    }
                    /* Override any MQMDFile values with those */
                    /* in the defaults MIB                     */
                    OverrideMIB(lpMIBParsed, lpMIBDIB);
                    /* Point to the MIB we just created and modified */
                    lpVIB->lpCtrlBlk = lpMIBParsed;
                }
            }
            else
            {
                /* Shouldnt happen but just use the NullMIB */
                CloneMIB((MQMDInfoBlk**)&lpVIB->lpCtrlBlk, kpMCB->lpNullMIB);
            }
            break;
        case V_RFH2:
            lpVIB->eVarType = V_RFH2;
            /* Get a pointer to the defaults RIB info if any */
            lpRIBDIB = lpVIBDIB->lpCtrlBlk;
            if(lpRIBDIB)
            {
                /* Work out the RFH2File name if any */
                ResolveValue(lpTIB, &lpRIBDIB->lpszResRFH2FileName,
                             &lpRIBDIB->lpszRFH2FileName, TRUE);
                ResolveValue(lpTIB, &lpRIBDIB->lpszResDir, &lpRIBDIB->lpszDir, TRUE);
                if(FAILURE == lpTIB->bCmdResult)
                {
                    return NULL;
                }
                if(0 == StrlenNull(lpRIBDIB->lpszResRFH2FileName))
                {
                    /* If the defaults RIB doesnt have a RFH2File to load */
                    /* just use the defaults RIB values for the new RIB   */
                    CloneRIB((RFH2InfoBlk**)&lpVIB->lpCtrlBlk, lpRIBDIB);
                }
                else
                {
                    /* Load a RIB with the values in the RFH2File */
                    lpRIBParsed = NULL;
                    LoadRIBFromFile(lpTIB, &lpRIBParsed,
                                    lpRIBDIB->lpszResRFH2FileName,
                                    lpRIBDIB->lpszResDir);
                    if(FAILURE == lpTIB->bCmdResult)
                    {
                        return NULL;
                    }
                    /* Override any RFH2File values with those */
                    /* in the defaults RIB                     */
                    OverrideRIB(lpRIBParsed, lpRIBDIB);
                    /* Point to the RIB we just created and modified */
                    lpVIB->lpCtrlBlk = lpRIBParsed;
                }
            }
            else
            {
                /* Shouldnt happen but just use the NullRIB */
                CloneRIB((RFH2InfoBlk**)&lpVIB->lpCtrlBlk, kpMCB->lpNullRIB);
            }
            break;
        }
    }
    else
    {
        switch(eVarType)
        {
        case V_NUM:
            lpVIB->eVarType = V_NUM;
            FreeIfNotNull((void**)&lpVIB->lpszVarFormat);
            if(lpszVarFormat)
            {
                if(0 < StrlenNull(lpszVarFormat))
                {
                    lpVIB->lpszVarFormat = CloneString(lpszVarFormat);
                }
                else
                {
                    lpVIB->lpszVarFormat = CloneString(kpszNUMFMT);
                }
            }
            else
            {
                lpVIB->lpszVarFormat = CloneString(kpszNUMFMT);
            }
            lpVIB->lpszCurrStrValue = CloneString(kpszEmptyString);
            break;
        case V_DOUBLE:
            lpVIB->eVarType = V_DOUBLE;
            FreeIfNotNull((void**)&lpVIB->lpszVarFormat);
            if(lpszVarFormat)
            {
                if(0 < StrlenNull(lpszVarFormat))
                {
                    lpVIB->lpszVarFormat = CloneString(lpszVarFormat);
                }
                else
                {
                    lpVIB->lpszVarFormat = CloneString(kpszDOUBLEFMT);
                }
            }
            else
            {
                lpVIB->lpszVarFormat = CloneString(kpszDOUBLEFMT);
            }
            lpVIB->lpszCurrStrValue = CloneString(kpszEmptyString);
            break;
        case V_STR:
            lpVIB->eVarType = V_STR;
            FreeIfNotNull((void**)&lpVIB->lpszVarFormat);
            if(lpszVarFormat)
            {
                if(0 < StrlenNull(lpszVarFormat))
                {
                    lpVIB->lpszVarFormat = CloneString(lpszVarFormat);
                }
                else
                {
                    lpVIB->lpszVarFormat = CloneString(kpszSTRFMT);
                }
            }
            else
            {
                lpVIB->lpszVarFormat = CloneString(kpszSTRFMT);
            }
            lpVIB->lpszCurrStrValue = CloneString(kpszEmptyString);
            break;
        case V_RANDOMNUM:
            lpVIB->eVarType = V_RANDOMNUM;
            FreeIfNotNull((void**)&lpVIB->lpszVarFormat);
            if(lpszVarFormat)
            {
                if(0 < StrlenNull(lpszVarFormat))
                {
                    lpVIB->lpszVarFormat = CloneString(lpszVarFormat);
                }
                else
                {
                    lpVIB->lpszVarFormat = CloneString(kpszNUMFMT);
                }
            }
            else
            {
                lpVIB->lpszVarFormat = CloneString(kpszNUMFMT);
            }
            lpVIB->lpszCurrStrValue = CloneString(kpszEmptyString);
            break;
        case V_LIST:
            lpVIB->eVarType = V_LIST;
            CloneLIB((ListInfoBlk**)&lpVIB->lpCtrlBlk, kpMCB->lpNullLIB);
            break;
        case V_MQMD:
            lpVIB->eVarType = V_MQMD;
            CloneMIB((MQMDInfoBlk**)&lpVIB->lpCtrlBlk, kpMCB->lpNullMIB);
            break;
        case V_RFH2:
            lpVIB->eVarType = V_RFH2;
            CloneRIB((RFH2InfoBlk**)&lpVIB->lpCtrlBlk, kpMCB->lpNullRIB);
            break;
        }
    }

    /* Add the VIB to the tests variable list */
    AddToTailOfList((LinkedListBlk**)&lpTIB->lpVIBHead,
                    (LinkedListBlk**)&lpTIB->lpVIBTail,
                    (LinkedListBlk*)lpVIB);

    return lpVIB;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     AddToTailOfList                                                      */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Add a control block to the tail of a linked list                     */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpHead  Pointer to head of linked list pointer                     */
/*     lplpTail  Pointer to tail of linked list pointer                     */
/*     lpBlk     Pointer to control block to be added to the linked list    */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
AddToTailOfList(LinkedListBlk** lplpHead, LinkedListBlk** lplpTail,
                LinkedListBlk* lpBlk)
{
    LinkedListBlk* lpOldTailBlk;

    if(NULL == *lplpHead)
    {
        *lplpHead = lpBlk;
        *lplpTail = lpBlk;
    }
    else
    {
        lpOldTailBlk = *lplpTail;
        lpOldTailBlk->lpLLBNext = (struct LinkedListBlk*)lpBlk;
        *lplpTail = lpBlk;
        (*lplpTail)->lpLLBPrev = (struct LinkedListBlk*)lpOldTailBlk;
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     RemoveFromList                                                       */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Remove a control block from the linked list                          */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpHead  Pointer to head of linked list pointer                     */
/*     lplpTail  Pointer to tail of linked list pointer                     */
/*     lpBlk     Pointer to the control block to be removed from the list   */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to the linked list element just removed                      */
/*==========================================================================*/
void
RemoveFromList(LinkedListBlk** lplpHead, LinkedListBlk** lplpTail,
               LinkedListBlk* lpBlk)
{
    /* Get local copies of the next and previous pointers */
    LinkedListBlk* const kpNextBlk = (LinkedListBlk*)lpBlk->lpLLBNext;
    LinkedListBlk* const kpPrevBlk = (LinkedListBlk*)lpBlk->lpLLBPrev;

    /* Sanity check 1. Pointer to a non-existent block */
    if(NULL == lpBlk)
    {
        return;
    }

    /* Sanity check 2. Its not on this list which is empty */
    if((NULL == *lplpHead) &&
       (NULL == *lplpTail))
    {
        return;
    }

    /* As we are detaching this block we need to clear its pointers */
    lpBlk->lpLLBNext = NULL;
    lpBlk->lpLLBPrev = NULL;

    /* Are we at the head of the list */
    if(NULL == kpPrevBlk)
    {
        /* Modify the head of the list pointer to this blocks next pointer */
        *lplpHead = kpNextBlk;
        /* If there is a next block we need to fix its prev pointer */
        if(NULL != kpNextBlk)
        {
            /* Now its at the head of the list */
            kpNextBlk->lpLLBPrev = NULL;
        }
    }
    else
    {
        /* Take this block out of the next chain */
        kpPrevBlk->lpLLBNext = (struct LinkedListBlk*)kpNextBlk;
        /* If there is a next block we need to fix the prev chain */
        if(NULL != kpNextBlk)
        {
            kpNextBlk->lpLLBPrev = (struct LinkedListBlk*)kpPrevBlk;
        }
    }
    /* Are we at the tail of the list */
    if(NULL == kpNextBlk)
    {
        /* Modify the tail of the list pointer to this blocks prev pointer */
        *lplpTail = kpPrevBlk;
        /* If there is a prev block we need to fix its next pointer */
        if(NULL != kpPrevBlk)
        {
            /* Now its at the tail of the list */
            kpPrevBlk->lpLLBNext = NULL;
        }
    }
    else
    {
        /* Take this block out of the prev chain */
        kpNextBlk->lpLLBPrev = (struct LinkedListBlk*)kpPrevBlk;
        /* If there is a prev block we need to fix the next chain */
        if(NULL != kpPrevBlk)
        {
            kpPrevBlk->lpLLBNext = (struct LinkedListBlk*)kpNextBlk;
        }
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     CloneString                                                          */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Clone a string.  If asked to clone a non-existent string then return */
/*     an empty string in a buffer that is 8 bytes long                     */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     kpszStringToClone Pointer to string to be cloned                     */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to the copied string                                         */
/*==========================================================================*/
char*
CloneString(const char* const kpszStringToClone)
{
    char* lpszNewString;
    long  lNewStringLen;

    /* Buffer length of 8 if pointer is null */
    if(NULL == kpszStringToClone)
    {
        lNewStringLen = 8;
    }
    else
    {
        lNewStringLen = (((StrlenNull(kpszStringToClone) + 8) / 8) + 1) * 8;
    }

    lpszNewString = malloc(lNewStringLen);
    if(! lpszNewString)
    {
        LogMsg(3, sizeof(lNewStringLen), "CloneString", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
        return kpszEmptyString;
    }

    /* Clear the string buffer */
    memset(lpszNewString, 0x00, lNewStringLen);

    /* Only copy if the string is present */
    if(NULL != kpszStringToClone)
    {
        strncpy(lpszNewString, kpszStringToClone, lNewStringLen);
    }

    return lpszNewString;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     CloneStringNull                                                      */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Clone a string.  If given a NULL to clone return NULL                */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     kpszStringToClone Pointer to string to be cloned                     */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to the copied string                                         */
/*==========================================================================*/
char*
CloneStringNull(const char* const kpszStringToClone)
{
    char* lpszNewString;
    long  lNewStringLen;

    /* Return null if pointer is null */
    if(NULL == kpszStringToClone)
    {
        return NULL;
    }

    /* Work out how big our new string buffer needs to be */
    lNewStringLen = (((StrlenNull(kpszStringToClone) + 8) / 8) + 1) * 8;

    /* Allocate the string */
    lpszNewString = malloc(lNewStringLen);
    if(! lpszNewString)
    {
        LogMsg(3, sizeof(lNewStringLen), "CloneStringNull", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
        return NULL;
    }

    /* Clear the string buffer */
    memset(lpszNewString, 0x00, lNewStringLen);

    strncpy(lpszNewString, kpszStringToClone, lNewStringLen);

    return lpszNewString;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     TrimString                                                           */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Trim a string.   If asked to trim a non-existent string then return  */
/*     an empty string in a buffer that is 8 bytes long                     */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     kpszStringToTrim Pointer to string to be trimmed                     */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Pointer to the copied string                                         */
/*==========================================================================*/
char*
TrimString(const char* const kpszStringToTrim)
{
    char* lpszNewString;
    char* lpszFirstNonWS;
    char* lpszLastNonWS;
    long  lStringLen;
    long  lNewStringBuffLen;
    long  lIdx;

    /* Buffer length of 8 if pointer is null */
    if(NULL == kpszStringToTrim)
    {
        lStringLen = 0;
        lNewStringBuffLen = 8;
    }
    else
    {
        lStringLen = StrlenNull(kpszStringToTrim);
        lNewStringBuffLen = (((lStringLen + 8) / 8) + 1) * 8;
    }

    lpszNewString = malloc(lNewStringBuffLen);
    if(! lpszNewString)
    {
        LogMsg(3, sizeof(lNewStringBuffLen), "TrimString", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
        return kpszEmptyString;
    }

    /* Clear the string buffer */
    memset(lpszNewString, 0x00, lNewStringBuffLen);

    /* Only copy if the string is present */
    if(NULL != kpszStringToTrim)
    {
        /* If its an empty string return straight away */
        if(0 == lStringLen)
        {
            return lpszNewString;
        }

        /* Initialise first and last pointers */
        lpszFirstNonWS = (char*)kpszStringToTrim;
        lpszLastNonWS = (char*)kpszStringToTrim + lStringLen - 1;
        
        /* Process until we find the first non whitespace character */
        for(lIdx = 0; lIdx < lStringLen; lIdx++)
        {
            /* Stop at the first non blank character */
            if(0 == isspace(*lpszFirstNonWS))
            {
                lpszFirstNonWS++; /* Make pointer adjustment consistent */
                break; /* for(lIdx... */
            }
            lpszFirstNonWS++;
        }
        
        /* Adjust pointer */
        lpszFirstNonWS--;
        
        /* If the pointers are the same and the character being */
        /* pointed to is a space then everything was spaces     */
        if(lpszFirstNonWS == lpszLastNonWS)
        {
            if(0 != isspace(*lpszFirstNonWS))
            {
                return lpszNewString;
            }
        }
        
        /* Process until we find the last non whitespace character */
        for(lIdx = 0; lIdx < lStringLen; lIdx++)
        {
            /* Stop at the Last non blank character */
            if(0 == isspace(*lpszLastNonWS))
            {
                lpszLastNonWS--; /* Make pointer adjustment consistent */
                break; /* for(lIdx... */
            }
            lpszLastNonWS--;
        }
        
        /* Adjust pointer */
        lpszLastNonWS++;
        
        /* Now copy the string with leading and trailing whitespace removed */
        lStringLen = lpszLastNonWS - lpszFirstNonWS + 1;
        memcpy(lpszNewString, lpszFirstNonWS, lStringLen);
    }

    return lpszNewString;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ChangeIfStr1Empty                                                    */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     If string 1's pointer is NULL or points to an empty string then      */
/*     clone string 2. If string 2's pointer is NULL then provide a clone   */
/*     of kpszEmptyString                                                   */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpszStr1 Pointer to the first string pointer                       */
/*     lplpszStr2 Pointer to the second string pointer                      */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
ChangeIfStr1Empty(char** lplpszStr1, char** lplpszStr2)
{
    if(NULL == *lplpszStr1)
    {
        /* If the pointer to the pointer is empty return an empty string */
        if(NULL == lplpszStr2)
        {
            *lplpszStr1 = CloneString(kpszEmptyString);
            return;
        }
        /* If the string pointer is empty return an empty string */
        if(NULL == *lplpszStr2)
        {
            *lplpszStr1 = CloneString(kpszEmptyString);
            return;
        }
        /* Clone string 2 */
        *lplpszStr1 = CloneString(*lplpszStr2);
        return;
    }

    if(0 == StrlenNull(*lplpszStr1))
    {
        /* If the pointer to the pointer is empty return an empty string */
        if(NULL == lplpszStr2)
        {
            /* Already an empty string just return */
            return;
        }
        /* If the string pointer is empty return an empty string */
        if(NULL == *lplpszStr2)
        {
            /* Already an empty string just return */
            return;
        }
        /* Free the empty string and clone string 2 */
        free(*lplpszStr1);
        *lplpszStr1 = CloneString(*lplpszStr2);
        return;
    }
    /* Got something in string 1 so leave everything unchanged */
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ChangeIfStr2Exists                                                   */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     If string 2 exists then change string 1 to string 2's value.         */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpszStr1 Pointer to the first string pointer                       */
/*     lplpszStr2 Pointer to the second string pointer                      */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
ChangeIfStr2Exists(char** lplpszStr1, char** lplpszStr2)
{
    /* If string 2 does not exist just return */
    if(NULL == lplpszStr2)
    {
        return;
    }
    if(NULL == *lplpszStr2)
    {
        return;
    }

    /* If string 1 exists free it */
    if(*lplpszStr1)
    {
        free(*lplpszStr1);
    }

    /* Now clone string 2 as string 1 */
    *lplpszStr1 = CloneString(*lplpszStr2);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ResolvePgmValue                                                      */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Resolve a string that can contain variable defined content           */
/*     Program invocation variables only in a non-recursive manner          */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpszResStr Pointer to a string pointer to return resolved string   */
/*     lplpszVarStr Pointer to a string pointer containing variable content */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
ResolvePgmValue(char** lplpszResStr, char** lplpszVarStr)
{
    VarInfoBlk*      lpVIB;
    char             lpszWork[RESOLVE_BUFF_LEN + 4];
    char             lpszName[NAME_BUFF_LEN + 4];
    char*            lpszWorkCurr;
    char*            lpszNameCurr;
    char*            lpszVarStrCopy;
    char*            lpszVarStrCurr;
    long             lResStrLen;
    long             lVarStrLen;
    long             lValueLen;
    long             bFirstPercentSign;
    long             bGotPercentSign;
    long             lIdx;

    /* Copy the input string locally in case the two string */
    /* pointers reference the same variable                 */
    lpszVarStrCopy = CloneString(*lplpszVarStr);
    lpszVarStrCurr = lpszVarStrCopy;

    /* If we have a pointer to an old resolved variable string drop it */
    if(*lplpszResStr)
    {
        free(*lplpszResStr);
        *lplpszResStr = NULL;
    }

    /* Initialise common use local variables */
    lResStrLen = 0;
    lVarStrLen = StrlenNull(lpszVarStrCopy);
    bFirstPercentSign = FALSE;
    bGotPercentSign = FALSE;
    lpszWorkCurr = lpszWork;
    memset(lpszWork, 0x00, RESOLVE_BUFF_LEN + 4);
    lpszNameCurr = lpszName;

    /* Process the string that might contain a variable */
    for(lIdx = 0; lIdx < lVarStrLen; lIdx++)
    {
        if(bFirstPercentSign)
        {
            /* Got a second percent sign so reached the end of the var name */
            if('%' == *lpszVarStrCurr)
            {
                bFirstPercentSign = FALSE;
                /* See if we can find a program level VIB in the main block */
                lpVIB = FindVarInfoBlk(kpMCB->lpVIBHead, lpszName);
                /* Couldn't find it anywhere so issue an error and quit */
                if(! lpVIB)
                {
                    ConsoleMsg(58, lpszName);
                    kpMCB->lProcessing = FAILURE;
                    break; /* for(lIdx=0; ... */
                }
                /* Make sure we have enough buffer left to copy the name */
                lValueLen = StrlenNull(lpVIB->lpszCurrStrValue);
                if(lValueLen < (RESOLVE_BUFF_LEN - (lpszWorkCurr - lpszWork)))
                {
                    /* Substitute the variables value into the work buffer */
                    strcpy(lpszWorkCurr, lpVIB->lpszCurrStrValue);
                    lpszWorkCurr += lValueLen;
                }
                else
                {
                    ConsoleMsg(59, lpszName);
                    kpMCB->lProcessing = FAILURE;
                    break; /* for(lIdx=0; ... */
                }
            }
            else
            {
                /* Copy the current character to the variable name buffer */
                *lpszNameCurr = *lpszVarStrCurr;
                lpszNameCurr++;
            }
        }
        else
        {
            if('%' == *lpszVarStrCurr)
            {
                lpszNameCurr = lpszName;
                memset(lpszName, 0x00, NAME_BUFF_LEN + 4);
                bFirstPercentSign = TRUE;
                bGotPercentSign = TRUE;
            }
            else
            {
                /* Copy anything outside of paired percent signs to the */
                /* work buffer                                          */
                *lpszWorkCurr = *lpszVarStrCurr;
                lpszWorkCurr++;
            }
        }
        lpszVarStrCurr++;
    }

    if(bFirstPercentSign)
    {
        ConsoleMsg(60,  lpszVarStrCopy);
        kpMCB->lProcessing = FAILURE;
        *lplpszResStr = CloneString("BADBADBAD\0");
    }
    else
    {
        if(SUCCESS == kpMCB->lProcessing)
        {
            *lplpszResStr = CloneString(lpszWork);
        }
        else
        {
            *lplpszResStr = CloneString("BADBADBAD\0");
        }
    }
    /* Free the local copy of the string */
    free(lpszVarStrCopy);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ResolveValue                                                         */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Resolve a string that can contain variable defined content           */
/*     To handle multiple level substitutions execute recursively           */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTIB         Pointer to the test info block                         */
/*     lplpszResStr  Pointer to a pointer to return resolved string         */
/*     lplpszVarStr  Pointer to a pointer containing variable content       */
/*     bDisplayMsg61 Flag controlling display of message 61                 */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
ResolveValue(TestInfoBlk* lpTIB, char** lplpszResStr, char** lplpszVarStr,
             long bDisplayMsg61)
{
    DefaultsInfoBlk*  lpDIB;
    VarInfoBlk*       lpVIB;
    MQMDInfoBlk*      lpMIB;
    enum eElementEnum eElementIs;
    char*             lpszWork;
    char*             lpszWorkCurr;
    char              lpszName[NAME_BUFF_LEN + 4];
    char              lpszNameDotted[NAME_BUFF_LEN + 4];
    char*             lpszNameComponentCurr;
    char*             lpszVarStrCopy;
    char*             lpszVarStrCurr;
    char*             lpszValue;
    char*             lpszEnvValue;
    long              lVarStrLen;
    long              lValueLen;
    long              bFirstPercentSign;
    long              bGotPercentSign;
    long              bGotDot;
    long              lIdx;

    /* Allocate a work buffer for variable name resolution */
    lpszWork = malloc(RESOLVE_BUFF_LEN + 4);
    if(lpszWork)
    {
        memset(lpszWork, 0x00, RESOLVE_BUFF_LEN + 4);
    }
    else
    {
        LogMsg(3, RESOLVE_BUFF_LEN + 4, "ResolveValue", __LINE__, __FILE__);
        lpTIB->bCmdResult = FAILURE;
        kpMCB->lProcessing = FAILURE;
        return;
    }

    /* Copy the input string locally in case the two string */
    /* pointers reference the same variable                 */
    lpszVarStrCopy = CloneString(*lplpszVarStr);
    if(FAILURE == kpMCB->lProcessing)
    {
        free(lpszWork);
        return;
    }

    /* If we have a pointer to an old resolved variable string drop it */
    if(*lplpszResStr)
    {
        free(*lplpszResStr);
        *lplpszResStr = NULL;
    }

    /* Resolve any subscripts */
    ResolveSubscript(lpTIB, &lpszVarStrCopy, &lpszVarStrCopy, bDisplayMsg61);

    /* Initialise common use local variables */
    lpDIB = kpMCB->lpDIB;
    lVarStrLen = StrlenNull(lpszVarStrCopy);
    bGotPercentSign = FALSE;
    bFirstPercentSign = FALSE;
    bGotDot = FALSE;
    lpszWorkCurr = lpszWork;
    lpszNameComponentCurr = lpszName;
    lpszVarStrCurr = lpszVarStrCopy;

    for(lIdx = 0; lIdx < lVarStrLen; lIdx++)
    {
        /* Process the string that might contain a variable */
        if(bFirstPercentSign)
        {
            /* If we have a dotted name send the reset of the */
            /* variable name to the dotted component buffer   */
            if('.' == *lpszVarStrCurr)
            {
                lpszNameComponentCurr = lpszNameDotted;
                bGotDot = TRUE;
                lpszVarStrCurr++;
                continue; /* for(lIdx=0; ... */
            }
            /* Got a second percent sign so we have */ 
            /* reached the end of the variable name */
            if('%' == *lpszVarStrCurr)
            {
                bFirstPercentSign = FALSE;
                /* Try the tests VIB's */
                lpVIB = FindVarInfoBlk(lpTIB->lpVIBHead, lpszName);
                if(NULL == lpVIB)
                {
                    /* Couldn't find it in the test VIB's */
                    /* try the defaults block             */
                    lpVIB = FindVarInfoBlk(lpDIB->lpVIBHead, lpszName);
                }
                if(NULL == lpVIB)
                {
                    /* Couldn't find it in the default VIB's */
                    /* try the main block                    */
                    lpVIB = FindVarInfoBlk(kpMCB->lpVIBHead, lpszName);
                }
                if(NULL == lpVIB)
                {
                    /* Couldn't find any matching VIB's at all   */
                    /* See if we can find a matching environment */
                    /* variable name and if so generate a VIB    */
                    /* from it. Should be safe as environment    */
                    /* variables do not change after a process   */
                    /* has been initiated                        */
                    lpszEnvValue = getenv(lpszName);
                    if(NULL != lpszEnvValue)
                    {
                        lpVIB = CreateVarInfoBlk(lpszName);
                        if(lpVIB)
                        {
                            lpVIB->eVarType = V_STR;
                            lpVIB->lpszVarFormat = CloneString(kpszSTRFMT);
                            lpVIB->lpszCurrStrValue = CloneString(lpszEnvValue);
                            AddToTailOfList((LinkedListBlk**)&lpDIB->lpVIBHead,
                                            (LinkedListBlk**)&lpDIB->lpVIBTail,
                                            (LinkedListBlk*)lpVIB);
                        }
                        else
                        {
                            ConsoleMsg(26, kpszSTR, kpMCB->lpszCurrNameAttr);
                            lpTIB->bCmdResult = FAILURE;
                            kpMCB->lProcessing = FAILURE;
                            return;
                        }

                    }
                }
                /* Couldn't find it anywhere so issue an error and quit */
                if(NULL == lpVIB)
                {
                    *lplpszResStr = CloneString("BADBADBAD\0");
                    LogMsg(46, lpTIB->lTestNum, lpszName, lpszVarStrCopy);
                    lpTIB->bCmdResult = FAILURE;
                    return;
                }
                /* Check to see if the variable has been declared or initialised at this point in time */
                if((NULL == lpVIB->lpszCurrStrValue) && (NULL == lpVIB->lpCtrlBlk))
                {
                    *lplpszResStr = CloneString("BADBADBAD\0");
                    LogMsg(269, lpTIB->lTestNum, lpszName);
                    lpTIB->bCmdResult = FAILURE;
                    return;
                }
                /* If we are processing a dotted variable get its value */
                /* from the relevant MIB. Otherwise copy the current    */
                /* string value from the VIB                            */
                lpszValue = lpVIB->lpszCurrStrValue;
                if(bGotDot)
                {
                    switch(lpVIB->eVarType)
                    {
                    case V_MQMD:
                        lpMIB = lpVIB->lpCtrlBlk;
                        eElementIs = PCBGetElementType(lpszNameDotted);
                        switch(eElementIs)
                        {
                        case E_VERSION:
                            lpszValue = lpMIB->lpszVersion;
                            break;
                        case E_REPORT:
                            lpszValue = lpMIB->lpszReport;
                            break;
                        case E_MSGTYPE:
                            lpszValue = lpMIB->lpszMsgType;
                            break;
                        case E_EXPIRY:
                            lpszValue = lpMIB->lpszExpiry;
                            break;
                        case E_FEEDBACK:
                            lpszValue = lpMIB->lpszFeedback;
                            break;
                        case E_ENCODING:
                            lpszValue = lpMIB->lpszEncoding;
                            break;
                        case E_CODEDCHARSETID:
                            lpszValue = lpMIB->lpszCodedCharSetId;
                            break;
                        case E_FORMAT:
                            lpszValue = lpMIB->lpszFormat;
                            break;
                        case E_PRIORITY:
                            lpszValue = lpMIB->lpszPriority;
                            break;
                        case E_PERSISTENCE:
                            lpszValue = lpMIB->lpszPersistence;
                            break;
                        case E_MSGID:
                            lpszValue = lpMIB->lpszMsgId;
                            break;
                        case E_CORRELID:
                            lpszValue = lpMIB->lpszCorrelId;
                            break;
                        case E_BACKOUTCOUNT:
                            lpszValue = lpMIB->lpszBackoutCount;
                            break;
                        case E_REPLYTOQ:
                            lpszValue = lpMIB->lpszReplyToQ;
                            break;
                        case E_REPLYTOQMGR:
                            lpszValue = lpMIB->lpszReplyToQMgr;
                            break;
                        case E_USERIDENTIFIER:
                            lpszValue = lpMIB->lpszUserIdentifier;
                            break;
                        case E_ACCOUNTINGTOKEN:
                            lpszValue = lpMIB->lpszAccountingToken;
                            break;
                        case E_APPLIDENTITYDATA:
                            lpszValue = lpMIB->lpszApplIdentityData;
                            break;
                        case E_PUTAPPLTYPE:
                            lpszValue = lpMIB->lpszPutApplType;
                            break;
                        case E_PUTAPPLNAME:
                            lpszValue = lpMIB->lpszPutApplName;
                            break;
                        case E_PUTDATE:
                            lpszValue = lpMIB->lpszPutDate;
                            break;
                        case E_PUTTIME:
                            lpszValue = lpMIB->lpszPutTime;
                            break;
                        case E_APPLORIGINDATA:
                            lpszValue = lpMIB->lpszApplOriginData;
                            break;
                        case E_GROUPID:
                            lpszValue = lpMIB->lpszGroupId;
                            break;
                        case E_MSGSEQNUMBER:
                            lpszValue = lpMIB->lpszMsgSeqNumber;
                            break;
                        case E_OFFSET:
                            lpszValue = lpMIB->lpszOffset;
                            break;
                        case E_MSGFLAGS:
                            lpszValue = lpMIB->lpszMsgFlags;
                            break;
                        case E_ORIGINALLENGTH:
                            lpszValue = lpMIB->lpszOriginalLength;
                            break;
                        default:
                            *lplpszResStr = CloneString("BADBADBAD\0");
                            LogMsg(89, lpTIB->lTestNum, lpszNameDotted);
                            lpTIB->bCmdResult = FAILURE;
                            return;
                        }
                        break;
                    /* MQMD's are the only thing we allow dotted name elements for */
                    default:
                        *lplpszResStr = CloneString("BADBADBAD\0");
                        LogMsg(88, lpTIB->lTestNum, lpszName);
                        lpTIB->bCmdResult = FAILURE;
                        return;
                    }
                }
                /* Make sure we have enough buffer left to copy the name */
                lValueLen = StrlenNull(lpszValue);
                if(lValueLen < (RESOLVE_BUFF_LEN - (lpszWorkCurr - lpszWork)))
                {
                    strcpy(lpszWorkCurr, lpszValue);
                    lpszWorkCurr += lValueLen;
                }
                else
                {
                    LogMsg(47,  lpTIB->lTestNum, lpszName);
                    lpTIB->bCmdResult = FAILURE;
                    return;
                }
            }
            else
            {
                *lpszNameComponentCurr = *lpszVarStrCurr;
                lpszNameComponentCurr++;
            }
        }
        else
        {
            if('%' == *lpszVarStrCurr)
            {
                lpszNameComponentCurr = lpszName;
                memset(lpszName, 0x00, NAME_BUFF_LEN + 4);
                memset(lpszNameDotted, 0x00, NAME_BUFF_LEN + 4);
                bFirstPercentSign = TRUE;
                bGotPercentSign = TRUE;
                bGotDot = FALSE;
            }
            else
            {
                *lpszWorkCurr = *lpszVarStrCurr;
                lpszWorkCurr++;
            }
        }
        lpszVarStrCurr++;
    }

    if(bFirstPercentSign)
    {
        LogMsg(48,  lpTIB->lTestNum, lpszVarStrCopy);
        *lplpszResStr = CloneString("BADBADBAD\0");
        lpTIB->bCmdResult = FAILURE;
    }
    else
    {
        if(SUCCESS == lpTIB->bCmdResult)
        {
            if(bGotPercentSign)
            {
                /* Call ourselves recursively to handle */
                /* multiple levels of substitution      */
                if(bDisplayMsg61)
                {
                    LogMsg(61, lpTIB->lTestNum,
                           lpszVarStrCopy, lpszWork);
                }
                ResolveValue(lpTIB, &lpszWork, &lpszWork, bDisplayMsg61);
            }
            *lplpszResStr = CloneString(lpszWork);
        }
        else
        {
            *lplpszResStr = CloneString("BADBADBAD\0");
            lpTIB->bCmdResult = FAILURE;
        }
    }

    /* Free the local copy of the string */
    free(lpszVarStrCopy);
    free(lpszWork);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ResolveSubscript                                                     */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Resolve a variables subscripting.                                    */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTIB         Pointer to the test info block                         */
/*     lplpszResStr  Pointer to a string pointer to return resolved string  */
/*     lplpszVarStr  Pointer to a string pointer containing variable name   */
/*     bDisplayMsg87 Flag controlling display of message 87                 */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
ResolveSubscript(TestInfoBlk* lpTIB, char** lplpszResStr, char** lplpszVarStr,
                 long bDisplayMsg87)
{
    char*            lpszWork;
    char*            lpszSubscript;
    char*            lpszWorkCurr;
    char*            lpszSubscriptCurr;
    char*            lpszVarStrCopy;
    char*            lpszVarStrCurr;
    char*            lpszSubscriptCopy;
    long             lVarStrLen;
    long             lValueLen;
    long             bFirstBracket;
    long             bGotBracket;
    long             lIdx;

    /* Allocate a work buffer for variable subscript resolution */
    lpszWork = malloc(RESOLVE_BUFF_LEN + 4);
    if(lpszWork)
    {
        memset(lpszWork, 0x00, RESOLVE_BUFF_LEN + 4);
    }
    else
    {
        LogMsg(3, RESOLVE_BUFF_LEN + 4, "ResolveSubscript", __LINE__, __FILE__);
        lpTIB->bCmdResult = FAILURE;
        kpMCB->lProcessing = FAILURE;
        return;
    }

    /* Allocate a buffer for the subscript component */
    lpszSubscript = malloc(RESOLVE_BUFF_LEN + 4);
    if(lpszSubscript)
    {
        memset(lpszSubscript, 0x00, RESOLVE_BUFF_LEN + 4);
    }
    else
    {
        LogMsg(3, RESOLVE_BUFF_LEN + 4, "ResolveSubscript", __LINE__, __FILE__);
        lpTIB->bCmdResult = FAILURE;
        kpMCB->lProcessing = FAILURE;
        return;
    }

    /* Copy the input string locally in case the two string */
    /* pointers reference the same variable                 */
    lpszVarStrCopy = CloneString(*lplpszVarStr);
    if(FAILURE == kpMCB->lProcessing)
    {
        free(lpszWork);
        free(lpszSubscript);
        return;
    }

    /* If we have a pointer to an old resolved variable string drop it */
    if(*lplpszResStr)
    {
        free(*lplpszResStr);
        *lplpszResStr = NULL;
    }

    /* Initialise common use local variables */
    lVarStrLen = StrlenNull(lpszVarStrCopy);
    bGotBracket = FALSE;
    bFirstBracket = FALSE;
    lpszWorkCurr = lpszWork;
    lpszSubscriptCurr = lpszSubscript;
    lpszVarStrCurr = lpszVarStrCopy;
    lpszSubscriptCopy = NULL;

    /* Process the string that might contain a variable */
    for(lIdx = 0; lIdx < lVarStrLen; lIdx++)
    {
        if(bFirstBracket)
        {
            /* Got the closing bracket so reached the end of the subscript value */
            if(']' == *lpszVarStrCurr)
            {
                bFirstBracket = FALSE;
                /* Clone the subscript string and resolve its value */
                lpszSubscriptCopy = CloneString(lpszSubscript);
                ResolveValue(lpTIB, &lpszSubscriptCopy, &lpszSubscriptCopy, bDisplayMsg87);

                /* Make sure we have enough buffer left to copy resolved subscript */
                lValueLen = StrlenNull(lpszSubscriptCopy);
                if(lValueLen < (RESOLVE_BUFF_LEN - (lpszWorkCurr - lpszWork)))
                {
                    strcpy(lpszWorkCurr, lpszSubscriptCopy);
                    lpszWorkCurr += lValueLen;
                    free(lpszSubscriptCopy);
                    lpszSubscriptCopy = NULL;
                    *lpszWorkCurr = *lpszVarStrCurr;
                    lpszWorkCurr++;
                    lpszSubscriptCurr = lpszSubscript;
                }
                else
                {
                    LogMsg(85,  lpTIB->lTestNum, lpszSubscript);
                    lpTIB->bCmdResult = FAILURE;
                    free(lpszSubscriptCopy);
                    lpszSubscriptCopy = NULL;
                    break; /* for(lIdx=0; ... */
                }
            }
            else
            {
                *lpszSubscriptCurr = *lpszVarStrCurr;
                lpszSubscriptCurr++;
            }
        }
        else
        {
            if('[' == *lpszVarStrCurr)
            {
                lpszSubscriptCurr = lpszSubscript;
                memset(lpszSubscript, 0x00, RESOLVE_BUFF_LEN + 4);
                bFirstBracket = TRUE;
                bGotBracket = TRUE;
                *lpszWorkCurr = *lpszVarStrCurr;
                lpszWorkCurr++;
            }
            else
            {
                *lpszWorkCurr = *lpszVarStrCurr;
                lpszWorkCurr++;
            }
        }
        lpszVarStrCurr++;
    }

    if(bFirstBracket)
    {
        LogMsg(86,  lpTIB->lTestNum, lpszVarStrCopy);
        lpTIB->bCmdResult = FAILURE;
        *lplpszResStr = CloneString("BADBADBAD\0");
    }
    else
    {
        if(SUCCESS == lpTIB->bCmdResult)
        {
            if(bGotBracket)
            {
                if(bDisplayMsg87)
                {
                    LogMsg(87, lpTIB->lTestNum,
                           lpszVarStrCopy, lpszWork);
                }
            }
            *lplpszResStr = CloneString(lpszWork);
        }
        else
        {
            *lplpszResStr = CloneString("BADBADBAD\0");
        }
    }

    /* Free the local string buffers */
    free(lpszVarStrCopy);
    free(lpszWork);
    free(lpszSubscript);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     SetMQMDFromMIB                                                       */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Set values in the MQMD. If a MIB is available then override any      */
/*     default values if the MIB value has been set.                        */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTIB            Pointer to the test info block                      */
/*     lpMIB            MQMD info block pointer                             */
/*     lpszMsgId        Pointer to override message id if any               */
/*     lpszCorrelId     Pointer to override correlation id if any           */
/*     lpszGroupId      Pointer to override group id if any                 */
/*     lpszMsgSeqNumber Pointer to override group id if any                 */
/*     lpszFormat       Pointer to override format if any                   */
/*     lpMsgDesc        Pointer to MQMD to be initialised                   */
/*     bSetForGet       If TRUE then the MQMD is being set up for an MQGET  */
/*     bDisplayMsg61    Flag controlling display of message 61              */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     SUCCESS or FAILURE                                                   */
/*==========================================================================*/
void
SetMQMDFromMIB(TestInfoBlk* lpTIB, MQMDInfoBlk* lpMIB,
               char* lpszMsgId, char* lpszCorrelId, char* lpszGroupId,
               char* lpszMsgSeqNumber, char* lpszFormat, MQMD* lpMsgDesc,
               long bSetForGet, long bDisplayMsg61)
{
    MQMD       oMQMDDefault = {MQMD_DEFAULT};
    char*      lpszLocalVersion;
    char*      lpszLocalMsgId;
    char*      lpszLocalCorrelId;
    char*      lpszLocalGroupId;
    char*      lpszLocalMsgSeqNumber;
    char*      lpszLocalFormat;
    char*      lpszResVar;
    long       lVersionLen;
    long       lReportLen;
    long       lMsgTypeLen;
    long       lExpiryLen;
    long       lFeedbackLen;
    long       lEncodingLen;
    long       lCCSIDLen;
    long       lFormatLen;
    long       lPriorityLen;
    long       lPersistenceLen;
    long       lMsgIdLen;
    long       lCorrelIdLen;
    long       lBackoutCountLen;
    long       lReplyToQLen;
    long       lReplyToQMgrLen;
    long       lUserIdentifierLen;
    long       lAccountingTokenLen;
    long       lApplIdentityDataLen;
    long       lPutApplTypeLen;
    long       lPutApplNameLen;
    long       lPutDateLen;
    long       lPutTimeLen;
    long       lApplOriginDataLen;
    long       lGroupIdLen;
    long       lMsgSeqNumberLen;
    long       lOffsetLen;
    long       lMsgFlagsLen;
    long       lOriginalLengthLen;
    PutMsgBlk* lpPMB;

    /* Copy the default MQMD to the MQMD we are setting up */
    /* and ensure values we specifically want for MsgTest  */
    memcpy(lpMsgDesc, &oMQMDDefault, sizeof(MQMD));
    lpMsgDesc->Version = MQMD_CURRENT_VERSION;
    lpMsgDesc->Encoding = MQENC_NATIVE;
    lpMsgDesc->CodedCharSetId = MQCCSI_Q_MGR;

    /* Set the local pointers for strings that take overrides */
    lpszLocalVersion = NULL;
    if(lpMIB)
    {
        lpszLocalVersion = lpMIB->lpszVersion;
    }
    /* MsgId */
    lpszLocalMsgId = NULL;
    if(lpszMsgId)
    {
        lpszLocalMsgId = lpszMsgId; /* Override MsgId */
    }
    else
    {
        if(lpMIB)
        {
            lpszLocalMsgId = lpMIB->lpszMsgId;      
        }
    }
    /* CorrelId */
    lpszLocalCorrelId = NULL;
    if(lpszCorrelId)
    {
        lpszLocalCorrelId = lpszCorrelId; /* Override CorrelId */
    }
    else
    {
        if(lpMIB)
        {
            lpszLocalCorrelId = lpMIB->lpszCorrelId;        
        }
    }
    /* GroupId */
    lpszLocalGroupId = NULL;
    if(lpszGroupId)
    {
        lpszLocalGroupId = lpszGroupId; /* Override GroupId */
    }
    else
    {
        if(lpMIB)
        {
            lpszLocalGroupId = lpMIB->lpszGroupId;        
        }
    }
    /* MsgSeqNumber */
    lpszLocalMsgSeqNumber = NULL;
    if(lpszMsgSeqNumber)
    {
        lpszLocalMsgSeqNumber = lpszMsgSeqNumber; /* Override MsgSeqNumber */
    }
    else
    {
        if(lpMIB)
        {
            lpszLocalMsgSeqNumber = lpMIB->lpszMsgSeqNumber;        
        }
    }
    /* Format */
    lpszLocalFormat = NULL;
    if(lpszFormat)
    {
        lpszLocalFormat = lpszFormat; /* Override Format */
    }
    else
    {
        if(lpMIB)
        {
            lpszLocalFormat = lpMIB->lpszFormat;        
        }
    }

    /*------------------------------------------*/
    /* Process fields common to MQGET and MQPUT */
    /*------------------------------------------*/
    lVersionLen = StrlenNull(lpszLocalVersion);
    lMsgIdLen = StrlenNull(lpszLocalMsgId);
    lCorrelIdLen = StrlenNull(lpszLocalCorrelId);
    lGroupIdLen = StrlenNull(lpszLocalGroupId);

    /* Version */
    if(0 < lVersionLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpszLocalVersion, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        TranslateToValue(lpTIB, &lpMsgDesc->Version, NULL, lpszResVar, Q_MQMD);
        FreeIfNotNull((void**)&lpszResVar);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
    }

    /* MsgId, CorrelId and GroupId */
    /* Use the old fashioned method to get any message     */
    memcpy(lpMsgDesc->MsgId, MQMI_NONE, MQ_MSG_ID_LENGTH);
    memcpy(lpMsgDesc->CorrelId, MQCI_NONE, MQ_CORREL_ID_LENGTH);
    memcpy(lpMsgDesc->GroupId, MQGI_NONE, MQ_GROUP_ID_LENGTH);
    if(0 < lMsgIdLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpszLocalMsgId, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        /* If the first two characters are 0x then the MsgId is in hex */
        if(0 == memcmp(lpszResVar, "0x", 2))
        {
            HexToBuff((char*)lpMsgDesc->MsgId,
                      lpszResVar + 2, MQ_MSG_ID_LENGTH);
        }
        else
        {
            TranslateToValue(lpTIB, &lpMsgDesc->MsgId, NULL, lpszResVar, Q_MQMI);
            if(FAILURE == lpTIB->bCmdResult)
            {
                FreeIfNotNull((void**)&lpszResVar);
                return;
            }
        }
        FreeIfNotNull((void**)&lpszResVar);
    }

    if(0 < lCorrelIdLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpszLocalCorrelId, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        /* If the first two characters are 0x then the CorrelId is in hex */
        if(0 == memcmp(lpszResVar, "0x", 2))
        {
            HexToBuff((char*)lpMsgDesc->CorrelId,
                      lpszResVar + 2, MQ_CORREL_ID_LENGTH);
        }
        else
        {
            TranslateToValue(lpTIB, &lpMsgDesc->CorrelId, NULL, lpszResVar, Q_MQCI);
            if(FAILURE == lpTIB->bCmdResult)
            {
                FreeIfNotNull((void**)&lpszResVar);
                return;
            }
        }
        FreeIfNotNull((void**)&lpszResVar);
    }

    if(0 < lGroupIdLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpszLocalGroupId, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        /* If the first two characters are 0x then the GroupId is in hex */
        if(0 == memcmp(lpszResVar, "0x", 2))
        {
            HexToBuff((char*)lpMsgDesc->GroupId,
                      lpszResVar + 2, MQ_GROUP_ID_LENGTH);
        }
        else
        {
            TranslateToValue(lpTIB, &lpMsgDesc->GroupId, NULL, lpszResVar, Q_MQGI);
            if(FAILURE == lpTIB->bCmdResult)
            {
                FreeIfNotNull((void**)&lpszResVar);
                return;
            }
        }
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* If we are setting up an MQMD for MQGET then we are only          */
    /* interested in the Version, MsgId, CorrelId, GroupId and MsgFlags */
    if(bSetForGet)
    {
        return;
    }

    /*---------------------------------------*/
    /* Process fields specific to MQPUT only */
    /*---------------------------------------*/
    /* Format is only applicable for MQPUT but as it can   */
    /* have an override value we need to process it before */
    /* checking the MIB pointer                            */
    lFormatLen = StrlenNull(lpszLocalFormat);
    if(0 < lFormatLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpszLocalFormat, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        TranslateToValue(lpTIB, &lpMsgDesc->Format, NULL, lpszResVar, Q_MQFMT);
        FreeIfNotNull((void**)&lpszResVar);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
    }

    /* If there is no MIB just go with the defaults for everything else */
    if(! lpMIB)
    {
        return;
    }

    lReportLen = StrlenNull(lpMIB->lpszReport);
    lMsgTypeLen = StrlenNull(lpMIB->lpszMsgType);
    lExpiryLen = StrlenNull(lpMIB->lpszExpiry);
    lFeedbackLen = StrlenNull(lpMIB->lpszFeedback);
    lEncodingLen = StrlenNull(lpMIB->lpszEncoding);
    lCCSIDLen = StrlenNull(lpMIB->lpszCodedCharSetId);
    lPriorityLen = StrlenNull(lpMIB->lpszPriority);
    lPersistenceLen = StrlenNull(lpMIB->lpszPersistence);
    lBackoutCountLen = StrlenNull(lpMIB->lpszBackoutCount);
    lReplyToQLen = StrlenNull(lpMIB->lpszReplyToQ);
    lReplyToQMgrLen = StrlenNull(lpMIB->lpszReplyToQMgr);
    lUserIdentifierLen = StrlenNull(lpMIB->lpszUserIdentifier);
    lAccountingTokenLen = StrlenNull(lpMIB->lpszAccountingToken);
    lApplIdentityDataLen = StrlenNull(lpMIB->lpszApplIdentityData);
    lPutApplTypeLen = StrlenNull(lpMIB->lpszPutApplType);
    lPutApplNameLen = StrlenNull(lpMIB->lpszPutApplName);
    lPutDateLen = StrlenNull(lpMIB->lpszPutDate);
    lPutTimeLen = StrlenNull(lpMIB->lpszPutTime);
    lApplOriginDataLen = StrlenNull(lpMIB->lpszApplOriginData);
    lGroupIdLen = StrlenNull(lpMIB->lpszGroupId);
    lMsgSeqNumberLen = StrlenNull(lpMIB->lpszMsgSeqNumber);
    lMsgFlagsLen = StrlenNull(lpMIB->lpszMsgFlags);
    lOffsetLen = StrlenNull(lpMIB->lpszOffset);
    lOriginalLengthLen = StrlenNull(lpMIB->lpszOriginalLength);

    /* Report */
    if(0 < lReportLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszReport, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        /* If the first two characters are 0x the Report value is in hex */
        if(0 == memcmp(lpszResVar, "0x", 2))
        {
            HexToLong(&lpMsgDesc->Report, lpszResVar + 2);
        }
        else
        {
            TranslateToValue(lpTIB, &lpMsgDesc->Report, NULL, lpszResVar, Q_MQRO);
            if(FAILURE == lpTIB->bCmdResult)
            {
                FreeIfNotNull((void**)&lpszResVar);
                return;
            }
        }
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* MsgType */
    if(0 < lMsgTypeLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszMsgType, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        TranslateToValue(lpTIB, &lpMsgDesc->MsgType, NULL, lpszResVar, Q_MQMT);
        FreeIfNotNull((void**)&lpszResVar);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
    }

    /* Expiry */
    if(0 < lExpiryLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszExpiry, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        TranslateToValue(lpTIB, &lpMsgDesc->Expiry, NULL, lpszResVar, Q_MQEI);
        FreeIfNotNull((void**)&lpszResVar);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
    }

    /* Feedback */
    if(0 < lFeedbackLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszFeedback, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        TranslateToValue(lpTIB, &lpMsgDesc->Feedback, NULL, lpszResVar, Q_MQFB);
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* Encoding */
    if(0 < lEncodingLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszEncoding, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        /* If the first two characters are 0x the Encoding value is in hex */
        if(0 == memcmp(lpszResVar, "0x", 2))
        {
            HexToLong(&lpMsgDesc->Encoding, lpszResVar + 2);
        }
        else
        {
            TranslateToValue(lpTIB, &lpMsgDesc->Encoding, NULL, lpszResVar, Q_MQENC);
            if(FAILURE == lpTIB->bCmdResult)
            {
                FreeIfNotNull((void**)&lpszResVar);
                return;
            }
        }
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* CodedCharSetId */
    if(0 < lCCSIDLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszCodedCharSetId, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        /* If the first two characters are 0x the CCSID value is in hex */
        if(0 == memcmp(lpszResVar, "0x", 2))
        {
            HexToLong(&lpMsgDesc->CodedCharSetId, lpszResVar + 2);
        }
        else
        {
            TranslateToValue(lpTIB, &lpMsgDesc->CodedCharSetId, NULL, lpszResVar, Q_MQCCSI);
            if(FAILURE == lpTIB->bCmdResult)
            {
                FreeIfNotNull((void**)&lpszResVar);
                return;
            }
        }
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* Priority */
    if(0 < lPriorityLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszPriority, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        TranslateToValue(lpTIB, &lpMsgDesc->Priority, NULL, lpszResVar, Q_MQPRI);
        if(FAILURE == lpTIB->bCmdResult)
        {
            FreeIfNotNull((void**)&lpszResVar);
            return;
        }
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* Persistence */
    if(0 < lPersistenceLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszPersistence, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        if(0 == memcmp(lpszResVar, kpszTRUE, strlen(kpszTRUE) + 1))
        {
            lpMsgDesc->Persistence = MQPER_PERSISTENT;
        }
        else if(0 == memcmp(lpszResVar, kpszFALSE, strlen(kpszFALSE) + 1))
        {
            lpMsgDesc->Persistence = MQPER_NOT_PERSISTENT;
        }
        else
        {
            TranslateToValue(lpTIB, &lpMsgDesc->Persistence, NULL, lpszResVar, Q_MQPER);
            if(FAILURE == lpTIB->bCmdResult)
            {
                FreeIfNotNull((void**)&lpszResVar);
                return;
            }
        }
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* BackoutCount */
    if(0 < lBackoutCountLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszBackoutCount, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        lpMsgDesc->BackoutCount = ConvNum(lpTIB, lpszResVar, "MQMD.BackoutCount\0");
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* ReplyToQ */
    if(0 < lReplyToQLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszReplyToQ, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        memset(lpMsgDesc->ReplyToQ, 0x00, MQ_Q_NAME_LENGTH);
        strncpy((char*)lpMsgDesc->ReplyToQ, lpszResVar,
                 MQ_Q_NAME_LENGTH);
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* ReplyToQMgr */
    if(0 < lReplyToQMgrLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszReplyToQMgr, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        memset(lpMsgDesc->ReplyToQMgr, 0x00, MQ_Q_MGR_NAME_LENGTH);
        strncpy((char*)lpMsgDesc->ReplyToQMgr, lpszResVar,
                MQ_Q_MGR_NAME_LENGTH);
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* UserIdentifier */
    if(0 < lUserIdentifierLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszUserIdentifier, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        memset(lpMsgDesc->UserIdentifier, 0x00, MQ_USER_ID_LENGTH);
        strncpy((char*)lpMsgDesc->UserIdentifier, lpszResVar,
                 MQ_USER_ID_LENGTH);
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* AccountingToken */
    if(0 < lAccountingTokenLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszAccountingToken, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        /* If the first two characters are 0x then the AccountingToken is in hex */
        if(0 == memcmp(lpszResVar, "0x", 2))
        {
            HexToBuff((char*)lpMsgDesc->AccountingToken,
                      lpszResVar + 2, MQ_ACCOUNTING_TOKEN_LENGTH);
        }
        else
        {
            strncpy((char*)lpMsgDesc->AccountingToken, lpszResVar,
                    MQ_ACCOUNTING_TOKEN_LENGTH);
        }
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* ApplIdentityData */
    if(0 < lApplIdentityDataLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszApplIdentityData, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        memset(lpMsgDesc->ApplIdentityData, 0x00, MQ_APPL_IDENTITY_DATA_LENGTH);
        strncpy((char*)lpMsgDesc->ApplIdentityData, lpszResVar,
                 MQ_APPL_IDENTITY_DATA_LENGTH);
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* PutApplType */
    if(0 < lPutApplTypeLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszPutApplType, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        TranslateToValue(lpTIB, &lpMsgDesc->PutApplType, NULL, lpszResVar, Q_MQAT);
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* PutApplName */
    if(0 < lPutApplNameLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszPutApplName, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        memset(lpMsgDesc->PutApplName, 0x00, MQ_PUT_APPL_NAME_LENGTH);
        strncpy((char*)lpMsgDesc->PutApplName, lpszResVar,
                 MQ_PUT_APPL_NAME_LENGTH);
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* PutDate */
    if(0 < lPutDateLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszPutDate, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        memset(lpMsgDesc->PutDate, 0x00, MQ_PUT_DATE_LENGTH);
        strncpy((char*)lpMsgDesc->PutDate, lpszResVar,
                 MQ_PUT_DATE_LENGTH);
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* PutTime */
    if(0 < lPutTimeLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszPutTime, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        memset(lpMsgDesc->PutTime, 0x00, MQ_PUT_TIME_LENGTH);
        strncpy((char*)lpMsgDesc->PutTime, lpszResVar,
                 MQ_PUT_TIME_LENGTH);
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* ApplOriginData */
    if(0 < lApplOriginDataLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszApplOriginData, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        memset(lpMsgDesc->ApplOriginData, 0x00, MQ_APPL_ORIGIN_DATA_LENGTH);
        strncpy((char*)lpMsgDesc->ApplOriginData, lpszResVar,
                 MQ_APPL_ORIGIN_DATA_LENGTH);
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* MsgSeqNumber */
    if(0 < lMsgSeqNumberLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpszLocalMsgSeqNumber, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        lpMsgDesc->MsgSeqNumber = ConvNum(lpTIB, lpszResVar, "MQMD.MsgSeqNumber\0");
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* MsgFlags */
    if(0 < lMsgFlagsLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszMsgFlags, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        /* If the first two characters are 0x the MsgFlags value is in hex */
        if(0 == memcmp(lpszResVar, "0x", 2))
        {
            HexToLong(&lpMsgDesc->MsgFlags, lpszResVar + 2);
        }
        else
        {
            TranslateToValue(lpTIB, &lpMsgDesc->MsgFlags, NULL, lpszResVar, Q_MQMF);
            if(FAILURE == lpTIB->bCmdResult)
            {
                FreeIfNotNull((void**)&lpszResVar);
                return;
            }
        }
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* If its a PutMsg command and the Group option has been used then reset */
    /* the group related message flags and use the Group Mode setting        */
    if(C_PUTMSG == lpTIB->lpCCBCurr->eCommandType)
    {
        lpPMB = (PutMsgBlk*)lpTIB->lpCCBCurr;
        if(G_NONE != lpPMB->eGroupType)
        {
            /* Turn off group related flags */
            lpMsgDesc->MsgFlags &= ((MQMF_MSG_IN_GROUP | MQMF_LAST_MSG_IN_GROUP) ^ klFFFFFFFF);

            /* Now set the message flags if necessary */
            switch(lpPMB->eGroupType)
            {
            case G_GROUPLAST:
                lpMsgDesc->MsgFlags |= MQMF_LAST_MSG_IN_GROUP;
            case G_GROUPNEXT:
                lpMsgDesc->MsgFlags |= MQMF_MSG_IN_GROUP;
                break;
            case G_GROUPAUTO:
                /* If the current for loop value is equal to the TO value or the */
                /* next list element pointer is NULL then for our purposes this  */
                /* is the last iteration of the loop                             */
                if(lpPMB->lpBreakContinueFSB)
                {
                    lpMsgDesc->MsgFlags |= MQMF_MSG_IN_GROUP;
                    switch(lpPMB->lpBreakContinueFSB->eForType)
                    {
                    case F_FOR:
                        if(lpPMB->lpBreakContinueFSB->lCurrent == lpPMB->lpBreakContinueFSB->lTo)
                        {
                            lpMsgDesc->MsgFlags |= MQMF_LAST_MSG_IN_GROUP;
                        }
                        break;
                    case F_FOREACH:
                        if(NULL == lpPMB->lpBreakContinueFSB->lpCurrLEB->lpLEBNext)
                        {
                            lpMsgDesc->MsgFlags |= MQMF_LAST_MSG_IN_GROUP;
                        }
                        break;
                    }
                }
                else
                {
                    /* No <For> loop so just make it the last message */
                    lpMsgDesc->MsgFlags |= MQMF_LAST_MSG_IN_GROUP;
                    lpMsgDesc->MsgFlags |= MQMF_MSG_IN_GROUP;
                }
                break;
            }
        }
    }

    /* Offset */
    if(0 < lOffsetLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszOffset, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        lpMsgDesc->Offset = ConvNum(lpTIB, lpszResVar, "MQMD.Offset\0");
        FreeIfNotNull((void**)&lpszResVar);
    }

    /* OriginalLength */
    if(0 < lOriginalLengthLen)
    {
        lpszResVar = NULL;
        ResolveValue(lpTIB, &lpszResVar, &lpMIB->lpszOriginalLength, bDisplayMsg61);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
        TranslateToValue(lpTIB, &lpMsgDesc->OriginalLength, NULL, lpszResVar, Q_MQOL);
        FreeIfNotNull((void**)&lpszResVar);
        if(FAILURE == lpTIB->bCmdResult)
        {
            return;
        }
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     CreateMIBFromMQMD                                                    */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Get the values from the MQMD into the MIB.                           */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpMIB    Pointer to the MQMD info block pointe                     */
/*     lpMsgDesc  Pointer to MQMD to be used to create the MIB              */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     SUCCESS or FAILURE                                                   */
/*==========================================================================*/
long
CreateMIBFromMQMD(MQMDInfoBlk** lplpMIB, MQMD* lpMsgDesc)
{
    MQMDInfoBlk* lpMIB;
    char         lpszWorkBuff[WORK_BUFF_LEN + 4];

    /* Initialise local MQMD info block pointer      */
    lpMIB = *lplpMIB;

    /* Do we have an existing MIB */
    if(lpMIB)
    {
        /* If this was a dynamically created MIB free it */
        /* before creating its replacement               */
        if(lpMIB->bDynamicMIB)
        {
            FreeMIB(&lpMIB);
            *lplpMIB = NULL;
        }
    }

    /* Allocate the MIB for holding the MQMD information */
    lpMIB = malloc(sizeof(MQMDInfoBlk));
    if(! lpMIB)
    {
        LogMsg(3, sizeof(MQMDInfoBlk), "MQMDInfoBlk", __LINE__, __FILE__);
        kpMCB->lProcessing = FAILURE;
        return FAILURE;
    }

    /* Set the contents of the MIB pointer we were called with */
    *lplpMIB = lpMIB;

    /* Initialise the MQMD info block */
    memset(lpMIB, 0x00, sizeof(MQMDInfoBlk));
    
    /* Version */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->Version);
    lpMIB->lpszVersion = CloneString(lpszWorkBuff);

    /* Report */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "0x%08x\0", lpMsgDesc->Report);
    lpMIB->lpszReport = CloneString(lpszWorkBuff);

    /* MsgType */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->MsgType);
    lpMIB->lpszMsgType = CloneString(lpszWorkBuff);

    /* Expiry */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->Expiry);
    lpMIB->lpszExpiry = CloneString(lpszWorkBuff);

    /* Feedback */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->Feedback);
    lpMIB->lpszFeedback = CloneString(lpszWorkBuff);

    /* Encoding */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->Encoding);
    lpMIB->lpszEncoding = CloneString(lpszWorkBuff);

    /* CCSID */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->CodedCharSetId);
    lpMIB->lpszCodedCharSetId = CloneString(lpszWorkBuff);

    /* Format */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%.8s\0", lpMsgDesc->Format);
    lpMIB->lpszFormat = CloneString(lpszWorkBuff);

    /* Priority */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->Priority);
    lpMIB->lpszPriority = CloneString(lpszWorkBuff);

    /* Persistence */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->Persistence);
    lpMIB->lpszPersistence = CloneString(lpszWorkBuff);

    /* MsgId */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    GenByte24Str(lpszWorkBuff, (char*)lpMsgDesc->MsgId, TRUE);
    lpMIB->lpszMsgId = CloneString(lpszWorkBuff);

    /* CorrelId */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    GenByte24Str(lpszWorkBuff, (char*)lpMsgDesc->CorrelId, TRUE);
    lpMIB->lpszCorrelId = CloneString(lpszWorkBuff);

    /* BackoutCount */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->BackoutCount);
    lpMIB->lpszBackoutCount = CloneString(lpszWorkBuff);

    /* ReplyToQ */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%.48s\0", lpMsgDesc->ReplyToQ);
    lpMIB->lpszReplyToQ = TrimString(lpszWorkBuff);

    /* ReplyToQMgr */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%.48s\0", lpMsgDesc->ReplyToQMgr);
    lpMIB->lpszReplyToQMgr = TrimString(lpszWorkBuff);

    /* UserIdentifier */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%.12s\0", lpMsgDesc->UserIdentifier);
    lpMIB->lpszUserIdentifier = TrimString(lpszWorkBuff);

    /* AccountingToken */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    GenByte32Str(lpszWorkBuff, (char*)lpMsgDesc->AccountingToken);
    lpMIB->lpszAccountingToken = CloneString(lpszWorkBuff);

    /* ApplIdentityData */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%.32s\0", lpMsgDesc->ApplIdentityData);
    lpMIB->lpszApplIdentityData = TrimString(lpszWorkBuff);

    /* PutApplType */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->PutApplType);
    lpMIB->lpszPutApplType = CloneString(lpszWorkBuff);

    /* PutApplName */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%.28s\0", lpMsgDesc->PutApplName);
    lpMIB->lpszPutApplName = TrimString(lpszWorkBuff);

    /* PutDate */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%.8s\0", lpMsgDesc->PutDate);
    lpMIB->lpszPutDate = TrimString(lpszWorkBuff);

    /* PutTime */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%.8s\0", lpMsgDesc->PutTime);
    lpMIB->lpszPutTime = TrimString(lpszWorkBuff);

    /* ApplOriginData */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%.4s\0", lpMsgDesc->ApplOriginData);
    lpMIB->lpszApplOriginData = TrimString(lpszWorkBuff);

    /* GroupId */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    GenByte24Str(lpszWorkBuff, (char*)lpMsgDesc->GroupId, TRUE);
    lpMIB->lpszGroupId = CloneString(lpszWorkBuff);

    /* MsgSeqNumber */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->MsgSeqNumber);
    lpMIB->lpszMsgSeqNumber = CloneString(lpszWorkBuff);

    /* Offset */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->Offset);
    lpMIB->lpszOffset = CloneString(lpszWorkBuff);

    /* MsgFlags */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "0x%08x\0", lpMsgDesc->MsgFlags);
    lpMIB->lpszMsgFlags = CloneString(lpszWorkBuff);

    /* OriginalLength */
    memset(lpszWorkBuff, 0x00, WORK_BUFF_LEN + 4);
    sprintf(lpszWorkBuff, "%d\0", lpMsgDesc->OriginalLength);
    lpMIB->lpszOriginalLength = CloneString(lpszWorkBuff);

    /* This is a dynamic MIB flag it as such */
    lpMIB->bDynamicMIB = TRUE;

    return SUCCESS;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     GenByte24Str                                                         */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Convert a BYTE24 values to a string. If it contains unprintable      */
/*     characters other than trailing nulls generate the string in hex.     */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpszWorkBuff    Pointer to the buffer to receive the string          */
/*     lpszByte24Value Pointer to the BYTE24 value for conversion           */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     SUCCESS or FAILURE                                                   */
/*==========================================================================*/
void
GenByte24Str(char* lpszWorkBuff, char* lpszByte24Value, long bHexIfSpecialChars)
{
    char         lpszNulls[MQ_MSG_ID_LENGTH];
    char*        lpszCurrChar;
    long         lIdx;
    long         bPrintAsHex;
    long         lForHexConv;

    bPrintAsHex = FALSE;
    lpszCurrChar = lpszByte24Value;
    for(lIdx = 0; lIdx < MQ_MSG_ID_LENGTH; lIdx++)
    {
        if(! isprint(*lpszCurrChar))
        {
            /* If the rest is just nulls there is no need to do a hex print */
            memset(lpszNulls, 0x00, MQ_MSG_ID_LENGTH);
            if(0 == memcmp(lpszCurrChar, lpszNulls, MQ_MSG_ID_LENGTH - lIdx))
            {
                break; /* for(lIdx... */
            }
            else
            {
                bPrintAsHex = TRUE;
                break; /* for(lIdx... */
            }
        }
        if(TRUE == bHexIfSpecialChars)
        {
            if(('%' == *lpszCurrChar) || ('[' == *lpszCurrChar) || (']' == *lpszCurrChar))
            {
                bPrintAsHex = TRUE;
                break; /* for(lIdx... */
            }
        }
        lpszCurrChar++;
    }
    if(bPrintAsHex)
    {
        sprintf(lpszWorkBuff, "0x\0");
        lpszCurrChar = lpszByte24Value;
        for(lIdx = 0; lIdx < MQ_MSG_ID_LENGTH; lIdx++)
        {
            lForHexConv = (char)*lpszCurrChar;
            lForHexConv &= 255; /* Clean up char to long conversion */
            sprintf(lpszWorkBuff + 2 + (lIdx * 2), "%02x\0", lForHexConv);
            lpszCurrChar++;
        }
    }
    else
    {
        sprintf(lpszWorkBuff, "%.24s\0", lpszByte24Value);
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     GenByte32Str                                                         */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Convert a BYTE32 value to a string. If it contains unprintable       */
/*     characters other than trailing nulls generate the string in hex.     */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpszWorkBuff    Pointer to the buffer to receive the string          */
/*     lpszByte32Value Pointer to the BYTE32 value for conversion           */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     SUCCESS or FAILURE                                                   */
/*==========================================================================*/
void
GenByte32Str(char* lpszWorkBuff, char* lpszByte32Value)
{
    char         lpszNulls[MQ_ACCOUNTING_TOKEN_LENGTH];
    char*        lpszCurrChar;
    long         lIdx;
    long         bPrintAsHex;
    long         lForHexConv;

    bPrintAsHex = FALSE;
    lpszCurrChar = lpszByte32Value;
    for(lIdx = 0; lIdx < MQ_ACCOUNTING_TOKEN_LENGTH; lIdx++)
    {
        if(! isprint(*lpszCurrChar))
        {
            /* If the rest is just nulls there is no need to do a hex print */
            memset(lpszNulls, 0x00, MQ_ACCOUNTING_TOKEN_LENGTH);
            if(0 == memcmp(lpszCurrChar, lpszNulls, MQ_ACCOUNTING_TOKEN_LENGTH - lIdx))
            {
                break; /* for(lIdx... */
            }
            else
            {
                bPrintAsHex = TRUE;
                break; /* for(lIdx... */
            }
        }
        lpszCurrChar++;
    }
    if(bPrintAsHex)
    {
        sprintf(lpszWorkBuff, "0x\0");
        lpszCurrChar = lpszByte32Value;
        for(lIdx = 0; lIdx < MQ_ACCOUNTING_TOKEN_LENGTH; lIdx++)
        {
            lForHexConv = (char)*lpszCurrChar;
            lForHexConv &= 255; /* Clean up char to long conversion */
            sprintf(lpszWorkBuff + 2 + (lIdx * 2), "%02x\0", lForHexConv);
            lpszCurrChar++;
        }
    }
    else
    {
        sprintf(lpszWorkBuff, "%.32s\0", lpszByte32Value);
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ConvNum                                                              */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Convert a string to its numeric value                                */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTIB    Pointer to test info block                                  */
/*     kpszNum  Pointer to string to be converted                           */
/*     kpszName Pointer to name of number being converted for error msgs    */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Converted integer value                                              */
/*==========================================================================*/
long
ConvNum(TestInfoBlk* lpTIB, const char* const kpszNum, const char* const kpszName)
{
    long            lIdx;
    char*           lpszName;
    char*           lpszCurr;
    long            lStrLen;

    /* Use UNKNOWN if we dont know the name of the variable */
    if(NULL == kpszName)
    {
        lpszName = kpszUNKNOWN;
    }
    else
    {
        lpszName = (char*)kpszName;
    }

    /* A NULL pointer is bad bad bad. Stop all processing */
    if(NULL == kpszNum)
    {
        if(lpTIB)
        {
            LogMsg(24, "NULL", lpszName);
            lpTIB->bCmdResult = FAILURE;
            kpMCB->lProcessing = FAILURE;
        }
        else
        {
            LogMsg(24, "NULL", lpszName);
            kpMCB->lProcessing = FAILURE;
        }
        return 0;
    }

    /* Point to string to be converted */
    lpszCurr = (char*)kpszNum;
    lStrLen = StrlenNull(kpszNum);

    /* If first character is a minus sign skip past it for numeric checking */
    if('-' == *lpszCurr)
    {
        lpszCurr++;
        lStrLen--;
    }

    /* Dont like empty strings any more than NULL pointers */
    if(0 == lStrLen)
    {
        if(lpTIB)
        {
            LogMsg(24, "EmptyString", lpszName);
            lpTIB->bCmdResult = FAILURE;
            kpMCB->lProcessing = FAILURE;
        }
        else
        {
            LogMsg(24, "EmptyString", lpszName);
            kpMCB->lProcessing = FAILURE;
        }
        return 0;
    }

    /* Check that the string only contains numeric characters */
    for(lIdx = 0; lIdx < lStrLen; lIdx++)
    {
        if(! isdigit(*lpszCurr))
        {
            if(lpTIB)
            {
                LogMsg(198, lpTIB->lTestNum, kpszNum, lpszName);
                lpTIB->bCmdResult = FAILURE;
                kpMCB->lProcessing = FAILURE;
            }
            else
            {
                LogMsg(24, kpszNum, lpszName);
                kpMCB->lProcessing = FAILURE;
            }
            return 0;
        }
        lpszCurr++;
    }

    return atol(kpszNum);
}

/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ConvNumFailReturnsZero                                               */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Convert a string to its numeric value. Result tested by the caller   */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     kpszNum  Pointer to string to be converted                           */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Converted integer value                                              */
/*==========================================================================*/
long
ConvNumFailReturnsZero(const char* const kpszNum)
{
    long            lIdx;
    char*           lpszCurr;
    long            lStrLen;

    /* A NULL pointer is bad bad bad. Stop all processing */
    if(NULL == kpszNum)
    {
        return 0;
    }

    /* Point to string to be converted */
    lpszCurr = (char*)kpszNum;
    lStrLen = StrlenNull(kpszNum);

    /* If first character is a minus sign skip past it for numeric checking */
    if('-' == *lpszCurr)
    {
        lpszCurr++;
        lStrLen--;
    }

    /* Dont like empty strings any more than NULL pointers */
    if(0 == lStrLen)
    {
        return 0;
    }

    /* Check that the string only contains numeric characters */
    for(lIdx = 0; lIdx < lStrLen; lIdx++)
    {
        if(! isdigit(*lpszCurr))
        {
            return 0;
        }
        lpszCurr++;
    }

    return atol(kpszNum);
}

/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ConvDouble                                                           */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Convert a string to its numeric value                                */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTIB       Pointer to test info block                               */
/*     kpszDouble  Pointer to string to be converted                        */
/*     kpszName    Pointer to name of number being converted for error msgs */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Converted double value                                               */
/*==========================================================================*/
double
ConvDouble(TestInfoBlk* lpTIB, const char* const kpszDouble, const char* const kpszName)
{
    long            lIdx;
    long            lStrLen;
    char*           lpszName;
    char*           lpszCurr;
    char*           lpszEndOfScan;

    /* Use UNKNOWN if we dont know the name of the variable */
    if(NULL == kpszName)
    {
        lpszName = kpszUNKNOWN;
    }
    else
    {
        lpszName = (char*)kpszName;
    }

    /* A NULL pointer is bad bad bad. Stop all processing */
    if(NULL == kpszDouble)
    {
        if(lpTIB)
        {
            LogMsg(24, "NULL", lpszName);
            lpTIB->bCmdResult = FAILURE;
            kpMCB->lProcessing = FAILURE;
        }
        else
        {
            LogMsg(24, "NULL", lpszName);
            kpMCB->lProcessing = FAILURE;
        }
        return 0;
    }

    /* Point to string to be converted */
    lpszCurr = (char*)kpszDouble;
    lStrLen = StrlenNull(kpszDouble);

    /* If first character is a minus sign skip past it for numeric checking */
    if('-' == *lpszCurr)
    {
        lpszCurr++;
        lStrLen--;
    }

    /* Dont like empty strings any more than NULL pointers */
    if(0 == lStrLen)
    {
        if(lpTIB)
        {
            LogMsg(24, "EmptyString", lpszName);
            lpTIB->bCmdResult = FAILURE;
            kpMCB->lProcessing = FAILURE;
        }
        else
        {
            LogMsg(24, "EmptyString", lpszName);
            kpMCB->lProcessing = FAILURE;
        }
        return 0;
    }

    /* Check that the string only contains numeric characters */
    /* or a decimal point                                     */
    for(lIdx = 0; lIdx < lStrLen; lIdx++)
    {
        if(! isdigit(*lpszCurr))
        {
            if(kcDECIMALPOINT != *lpszCurr)
            {
                if(lpTIB)
                {
                    lpTIB->bCmdResult = FAILURE;
                }
                else
                {
                    LogMsg(24, kpszDouble, lpszName);
                    kpMCB->lProcessing = FAILURE;
                }
                return 0;
            }
        }
        lpszCurr++;
    }

    return strtod(kpszDouble, &lpszEndOfScan);
}

/*==========================================================================*/
/* FUNCTION:                                                                */
/*     IsNumeric                                                            */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Test if a string is numeric value                                    */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     kpszNum  Pointer to string to be checked                             */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     TRUE or FALSE                                                        */
/*==========================================================================*/
long
IsNumeric(const char* const kpszNum)
{
    long            lIdx;
    char*           lpszCurr;
    long            lStrLen;

    /* A NULL pointer is not numeric */
    if(NULL == kpszNum)
    {
        return FALSE;
    }

    /* Point to string to be checked */
    lpszCurr = (char*)kpszNum;
    lStrLen = StrlenNull(kpszNum);

    /* If first character is a minus sign skip past it for numeric checking */
    if('-' == *lpszCurr)
    {
        lpszCurr++;
        lStrLen--;
    }

    /* Dont like empty strings any more than NULL pointers */
    if(0 == lStrLen)
    {
        return FALSE;
    }

    /* Check that the string only contains numeric characters */
    for(lIdx = 0; lIdx < lStrLen; lIdx++)
    {
        if(! isdigit(*lpszCurr))
        {
            return FALSE;
        }
        lpszCurr++;
    }

    return TRUE;
}

/*==========================================================================*/
/* FUNCTION:                                                                */
/*     IsDouble                                                             */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Test if a string is double numeric value                             */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     kpszNum  Pointer to string to be checked                             */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     TRUE or FALSE                                                        */
/*==========================================================================*/
long
IsDouble(const char* const kpszNum)
{
    long            lIdx;
    char*           lpszCurr;
    long            lStrLen;
    long            lDecimalPointCnt;

    /* A NULL pointer is not numeric */
    if(NULL == kpszNum)
    {
        return FALSE;
    }

    /* Point to string to be checked */
    lpszCurr = (char*)kpszNum;
    lStrLen = StrlenNull(kpszNum);
    lDecimalPointCnt = 0;

    /* If first character is a minus sign skip past it for numeric checking */
    if('-' == *lpszCurr)
    {
        lpszCurr++;
        lStrLen--;
    }

    /* Dont like empty strings any more than NULL pointers */
    if(0 == lStrLen)
    {
        return FALSE;
    }

    /* Check that the string only contains numeric characters or a single decimal point */
    for(lIdx = 0; lIdx < lStrLen; lIdx++)
    {
        if(! isdigit(*lpszCurr))
        {
            if(kcDECIMALPOINT != *lpszCurr)
            {
                return FALSE;
            }
            else
            {
                lDecimalPointCnt++;
                if(1 < lDecimalPointCnt)
                {
                    return FALSE;
                }
            }
        }
        lpszCurr++;
    }

    return TRUE;
}

/*==========================================================================*/
/* FUNCTION:                                                                */
/*     HexToLong                                                            */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Convert a hex string to a long value                                 */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpLong     Long value to contain the return result                   */
/*     kpszHexStr Pointer to hex string to be converted                     */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
HexToLong(long* lpLong, const char* const kpszHexStr)
{
    char        lpszWork[10];
    char        lpszTemp[8];
    long        lHexStrLen;
    long        lIdx;

    /* Initialise work area to zeroes */
    memset(lpszWork, 0x00, 10);
    memcpy(lpszWork, "00000000", 8);

    /* Find out how long the hex string is. Maximum of eight nibbles */
    lHexStrLen = StrlenNull(kpszHexStr);
    if(8 < lHexStrLen)
    {
        lHexStrLen = 8;
    }

    /* Only change the value if the string contains something */
    if(0 < lHexStrLen)
    {
        memcpy(lpszWork + 8 - lHexStrLen, kpszHexStr, lHexStrLen);
        /* If its not a hex digit then set it to zero */
        for(lIdx = 0; lIdx < 8; lIdx++)
        {
            if(! isxdigit(lpszWork[lIdx]))
            {
                lpszWork[lIdx] = '0';
            }
        }
    }

/* For little endian systems flip the bytes */
#ifdef WIN32
#define TJA_LITTLE_ENDIAN
#endif /* WIN32 */

#ifdef TJA_LITTLE_ENDIAN
    memcpy(lpszTemp, lpszWork, 8);
    memcpy(lpszWork + 0, lpszTemp + 6, 2);
    memcpy(lpszWork + 2, lpszTemp + 4, 2);
    memcpy(lpszWork + 4, lpszTemp + 2, 2);
    memcpy(lpszWork + 6, lpszTemp + 0, 2);
#endif


    /* Translate the hex value */
    HexToBuff((char*)lpLong, lpszWork, 4);
}

/*==========================================================================*/
/* FUNCTION:                                                                */
/*     HexToBuff                                                            */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Convert a hex string into a buffer                                   */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpszBuff   Pointer to buffer to receive the converted string         */
/*     kpszHexStr Pointer to hex string to be converted                     */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
HexToBuff(char* lpszBuff, const char* const kpszHexStr, long lMaxLen)
{
    static char hexTab[] =  "\x00\x01\x02\x03\x04\x05\x06\x07\x08"
                            "\x09\x00\x00\x00\x00\x00\x00\x00\x0a"
                            "\x0b\x0c\x0d\x0e\x0f";
    char cLeftNibble;
    char cRightNibble;
    long lHexStrLen;
    long lCopyLen;
    long lIdx;

    memset(lpszBuff, 0x00, lMaxLen);

    lHexStrLen = StrlenNull(kpszHexStr);
    if(0 < lHexStrLen)
    {
        if(0 == (lHexStrLen % 2))
        {
            lCopyLen = lHexStrLen / 2;
        }
        else
        {
            lCopyLen = (lHexStrLen / 2) + 1;
        }
        if(lCopyLen > lMaxLen)
        {
            lCopyLen = lMaxLen;
        }

        for(lIdx = 0; lIdx < lCopyLen; lIdx++)
        {
            cLeftNibble = (char)toupper(kpszHexStr[lIdx * 2]);
            cLeftNibble -= 0x30;
            cLeftNibble = hexTab[cLeftNibble];
            cLeftNibble <<= 4;

            cRightNibble = (char)toupper(kpszHexStr[(lIdx * 2) + 1]);
            if(0x00 != cRightNibble)
            {
                cRightNibble -= 0x30;
                cRightNibble = hexTab[cRightNibble];
            }

            *lpszBuff = (char)(cLeftNibble | cRightNibble);
            lpszBuff++;
        }
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     StrlenNull                                                           */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Return the length of a string whose pointer might be a NULL          */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpszStr   Pointer to the string                                      */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     Length of the string. Zero if null or empty                          */
/*==========================================================================*/
long
StrlenNull(const char* const kpszStr)
{
    if(NULL == kpszStr)
    {
        return 0;
    }

    return strlen(kpszStr);
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FindRC                                                               */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Check the reason code array for the presence of a particular rc      */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lFindRCCount Number of active elements in the array                  */
/*     lFindRC[] The reason code array to be checked                        */
/*     lCurrRC The reason code to be checked for                            */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     TRUE or FALSE                                                        */
/*==========================================================================*/
long
FindRC(long lFindRCCount, long lFindRC[], long lCurrRC)
{
    long lIdx;

    for(lIdx = 0; lIdx < lFindRCCount; lIdx++)
    {
        if(lCurrRC == lFindRC[lIdx])
        {
            return TRUE;
        }
    }

    return FALSE;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     RemoveXmlWhiteSpace                                                  */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Remove whitespace from inbetween XML tags                            */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lplpszXMLBuff Pointer to a pointer to the buffer containing XML data */
/*     lplXMLDataLen Pointer to a long in which to return the new data len  */
/*     lDataToProcessLen Length of XML data to be processed                 */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     TRUE or FALSE                                                        */
/*==========================================================================*/
void
RemoveXmlWhiteSpace(char** lplpszXMLBuff, long* lplXMLDataLen, long lDataToProcessLen)
{
    long    bCheckForWS;
    long    bAllWhiteSpace;
    long    bIsItSpace;
    long    bCDataSection;
    long    bCommentOnly;
    char*   lpszTagEnd;
    char*   lpszFrom;
    char*   lpszTo;
    long    lIdx;

    /* Read the entire file now lets remove any excess whitespace */
    bCheckForWS = TRUE;
    bAllWhiteSpace = TRUE;
    bCDataSection = FALSE;
    bCommentOnly = FALSE;
    lpszTagEnd = *lplpszXMLBuff - 1;
    lpszFrom = *lplpszXMLBuff;
    lpszTo = *lplpszXMLBuff;

    for(lIdx = 0; lIdx < lDataToProcessLen; lIdx++)
    {
        /* Dont attempt to process data in a CDATA section */
        if(bCDataSection)
        {
            if(0 == memcmp("]]>", lpszFrom, 3))
            {
                bCDataSection = FALSE;
            }
        }
        else
        {
            /* Have we found the start of a new tag */
            if('<' == *lpszFrom)
            {
                    /* If everything was whitespace then */
                    /* reset the copy to pointer         */
                    if(bAllWhiteSpace)
                    {
                        lpszTo = lpszTagEnd + 1;
                    }
                    bCheckForWS = FALSE;
                    bAllWhiteSpace = TRUE;
                    bCommentOnly = FALSE;
                    /* '<' Actually a CDATA section ?    */
                    if(0 == memcmp("<![CDATA[", lpszFrom, 9))
                    {
                        bCDataSection = TRUE;
                    }
            }
            /* If checking for nonblank characters   */
            /* have we found one.                    */
            if(bCheckForWS)
            {
                bIsItSpace = isspace(*lpszFrom);
                if(! bIsItSpace)
                {
                    bAllWhiteSpace = FALSE;
                    bCheckForWS = FALSE;
                }
            }
            /* Save the position of the copy to      */
            /* pointer if we find an end tag char    */
            if('>' == *lpszFrom)
            {
                lpszTagEnd = lpszTo;
                bCheckForWS = TRUE;
                bAllWhiteSpace = TRUE;
            }
        }

        /* If its the start of a comment update  */
        /* the pointers and flags                */
        if(0 == memcmp("<!--", lpszFrom, 4))
        {
            memcpy(lpszTagEnd + 1, "<!--", 4);
            lpszTo = lpszTagEnd + 5;
            lpszTagEnd = lpszTo - 1;
            lpszFrom += 4;
            lIdx += 3;
            bCheckForWS = FALSE;
            bAllWhiteSpace = TRUE;
            bCommentOnly = TRUE;
        }
        else if(0 == memcmp("-->", lpszFrom, 3))
        {
            if(bCommentOnly)
            {
                memcpy(lpszTo, "-->", 3);
                lpszTo += 3;
                lpszTagEnd = lpszTo - 1;
                lpszFrom += 3;
                lIdx += 2;
            }
            else
            {
                memcpy(lpszTagEnd + 1, "-->", 3);
                lpszTo = lpszTagEnd + 4;
                lpszTagEnd = lpszTo - 1;
                lpszFrom += 3;
                lIdx += 2;
            }
            bCheckForWS = TRUE;
            bAllWhiteSpace = TRUE;
            bCommentOnly = FALSE;
        }
        else
        {
            /* Copy the current byte */
            *lpszTo = *lpszFrom;
            lpszTo++;
            lpszFrom++;
        }
    }

    /* Calculate the length of the XML scipt */
    if(lpszTagEnd > *lplpszXMLBuff)
    {
        lpszTagEnd++;
        *lplXMLDataLen = lpszTagEnd - *lplpszXMLBuff;
    }
    else
    {
        *lplXMLDataLen = lpszTo - *lplpszXMLBuff;
    }

    /* Overwrite the rest of the file in the buffer with nulls */
    memset(*lplpszXMLBuff + *lplXMLDataLen, 0x00,
           lDataToProcessLen - *lplXMLDataLen);

}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     SetTIBCodes                                                          */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Set the Condition and Reason Code Fields in the TIB                  */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTIB Pointer to the test info block whose codes are to be set       */
/*     lCC   Condition code                                                 */
/*     lRC   Reason code                                                    */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
SetTIBCodes(TestInfoBlk* lpTIB, long lCC, long lRC)
{
    lpTIB->lLastCC = lCC;
    lpTIB->lLastRC = lRC;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     ConvertEscapeSequences                                               */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Convert c-style format string escape sequences to their actual value */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpszConvertStr Pointer to the string to convert                      */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
ConvertEscapeSequences(char* lpszConvertStr)
{
    char* lpszCurrChar;
    char* lpszOutputChar;
    char* lpszEndOfStr;
    char* lpszEscSeqChar;
    long  lMovePtrBy;

    lpszCurrChar = lpszConvertStr;
    lpszOutputChar = lpszConvertStr;
    lpszEndOfStr = lpszOutputChar + StrlenNull(lpszConvertStr);

    while(lpszCurrChar < lpszEndOfStr)
    {
        if('\\' == *lpszCurrChar)
        {
            lMovePtrBy = 1;
            lpszEscSeqChar = lpszCurrChar + 1;
            switch(*lpszEscSeqChar)
            {
            case 'n':
                *lpszCurrChar = '\n';
                *lpszEscSeqChar = '\0';
                break;
            case 't':
                *lpszCurrChar = '\t';
                *lpszEscSeqChar = '\0';
                break;
            case 'a':
                *lpszCurrChar = '\a';
                *lpszEscSeqChar = '\0';
                break;
            case 'b':
                *lpszCurrChar = '\b';
                *lpszEscSeqChar = '\0';
                break;
            case 'f':
                *lpszCurrChar = '\f';
                *lpszEscSeqChar = '\0';
                break;
            case 'r':
                *lpszCurrChar = '\r';
                *lpszEscSeqChar = '\0';
                break;
            case 'v':
                *lpszCurrChar = '\v';
                *lpszEscSeqChar = '\0';
                break;
            case '\\':
                *lpszCurrChar = '\\';
                *lpszEscSeqChar = '\0';
                break;
            case '\"':
                *lpszCurrChar = '\"';
                *lpszEscSeqChar = '\0';
                break;
            case '0':
                *lpszCurrChar = '\0';
                *lpszEscSeqChar = '\0';
                break;
            default:
                /* Unknown escape sequence just output it as is */
                *lpszCurrChar = '\\';
                lMovePtrBy = 0;
            }
            *lpszOutputChar = *lpszCurrChar;
            lpszCurrChar += lMovePtrBy;
        }
        else
        {
            *lpszOutputChar = *lpszCurrChar;
        }
        lpszOutputChar++;
        lpszCurrChar++;
    }
    *lpszOutputChar = '\0';
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     FreeIfNotNull                                                        */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Free the memory pointed to and then set the pointer to NULL          */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     vpvpFreeBlk Pointer to memory block pointer of the memory to free    */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
FreeIfNotNull(void** vpvpFreeBlk)
{
    if(NULL != *vpvpFreeBlk)
    {
        free(*vpvpFreeBlk);
        *vpvpFreeBlk = NULL;
    }
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     CheckAndExtendBuff                                                   */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Check the buffer to see if the extra data can fit in it and if not   */
/*     then extent it                                                       */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpBIB          Pointer to the buffer info block                      */
/*     lOutputDataLen Length of data to be added to the buffer              */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     SUCCESS or FAILURE                                                   */
/*==========================================================================*/
long
CheckAndExtendBuff(BuffInfoBlk* lpBIB, long lOutputDataLen)
{
    long  lOldBuffLen;
    char* lpszOldBuff;

    /* If the data fits return a value of SUCCESS to the caller */
    if((lpBIB->lBuffLen - lpBIB->lDataLen) >= lOutputDataLen)
    {
        return SUCCESS;
    }

    /* Otherwise try to extend the buffer */
    
    /* Hold information about the old buffer */
    lpszOldBuff = lpBIB->lpszBuff;
    lOldBuffLen = lpBIB->lBuffLen;
    
    /* Double the size of the buffer */
    lpBIB->lBuffLen *= 2;
    
    /* If its still not big enough then make it so */
    if(lpBIB->lBuffLen < (lpBIB->lDataLen + lOutputDataLen))
    {
        lpBIB->lBuffLen = (((lpBIB->lDataLen + lOutputDataLen) / 1024) + 1) * 1024;
    }

    /* Allocate a new buffer */
    lpBIB->lpszBuff = malloc(lpBIB->lBuffLen + 8);
    if(NULL == lpBIB->lpszBuff)
    {
        LogMsg(234, lpBIB->lpszBuffName, lpBIB->lBuffLen);
        lpBIB->lpszBuff = lpszOldBuff; /* Point back to old buffer for failure cleanup */
        kpMCB->lProcessing = FAILURE;
        return FAILURE;
    }
    LogMsg(235, lpBIB->lpszBuffName, lOldBuffLen, lpBIB->lBuffLen);

    /* Clear the new buffer and copy the contents of the old one to it */
    memset(lpBIB->lpszBuff, 0x00, lpBIB->lBuffLen + 8);
    memcpy(lpBIB->lpszBuff, lpszOldBuff, lpBIB->lDataLen);

    /* Finally release the old buffer */
    FreeIfNotNull((void**)&lpszOldBuff);

    return SUCCESS;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     BuffConvertToInt                                                     */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Convert a character string to an integer of a specified length and   */
/*     then copy it into the buffer                                         */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpBIB          Pointer to the buffer info block                      */
/*     kpszNumStr     Pointer to a string containing the number             */
/*     lOutputDataLen Length of data to be added to the buffer              */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     SUCCESS or FAILURE                                                   */
/*==========================================================================*/
long
BuffConvertToInt(BuffInfoBlk* lpBIB, const char* const kpszNumStr, long lOutputDataLen)
{
    Int64  i64Num;
    char*  lpszNum;

    /* Check the buffer can fit the data if not then extend it */
    if(FAILURE == CheckAndExtendBuff(lpBIB, lOutputDataLen))
    {
        return FAILURE;
    }

    /* Convert the string to its 64 bit integer representation */
#ifdef WIN32
    i64Num = _atoi64(kpszNumStr);
#else
    i64Num = strtoll(kpszNumStr, NULL, 10);
#endif /* WIN32 */

    lpszNum = (char*)&i64Num;
    /* Depending on whether we are big or little endian move the */
    /* appropriate bytes into the buffer                         */
    if(MQENC_INTEGER_NORMAL == kpMCB->lLocalIntegerEncoding)
    {
        switch(lOutputDataLen)
        {
        case 1:
            memcpy(lpBIB->lpszBuff + lpBIB->lDataLen, lpszNum + 7, 1);
            break;
        case 2:
            memcpy(lpBIB->lpszBuff + lpBIB->lDataLen, lpszNum + 6, 2);
            break;
        case 4:
            memcpy(lpBIB->lpszBuff + lpBIB->lDataLen, lpszNum + 4, 4);
            break;
        case 8:
            memcpy(lpBIB->lpszBuff + lpBIB->lDataLen, lpszNum, 8);
            break;
        }
    }
    else
    {
        switch(lOutputDataLen)
        {
        case 1:
            memcpy(lpBIB->lpszBuff + lpBIB->lDataLen, lpszNum, 1);
            break;
        case 2:
            memcpy(lpBIB->lpszBuff + lpBIB->lDataLen, lpszNum, 2);
            break;
        case 4:
            memcpy(lpBIB->lpszBuff + lpBIB->lDataLen, lpszNum, 4);
            break;
        case 8:
            memcpy(lpBIB->lpszBuff + lpBIB->lDataLen, lpszNum, 8);
            break;
        }
    }
    
    /* Increment the buffers data length */
    lpBIB->lDataLen += lOutputDataLen;

    return SUCCESS;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     PackedToBuff                                                         */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Convert a numeric string into a packed decimal value in the buffer   */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpszBuff   Pointer to buffer to receive the converted string         */
/*     kpszPackedStr Pointer to Packed string to be converted               */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     N/A                                                                  */
/*==========================================================================*/
void
PackedToBuff(char* lpszBuff, const char* const kpszPackedStr, long lMaxLen)
{
    static char PackedTab[] =  "\x00\x01\x02\x03\x04\x05\x06\x07\x08"
                               "\x09\x00\x00\x00\x00\x00\x00\x00\x0a"
                               "\x0b\x0c\x0d\x0e\x0f";
    char* lpszPacked;
    char* lpszCurr;
    char  cNibble;
    long  lPackedStrLen;
    long  lCopyLen;
    long  lIdx;

    memset(lpszBuff, 0x00, lMaxLen);

    lPackedStrLen = StrlenNull(kpszPackedStr);
    if(0 < lPackedStrLen)
    {
        if(0 == (lPackedStrLen % 2))
        {
            lCopyLen = lPackedStrLen / 2;
        }
        else
        {
            lCopyLen = (lPackedStrLen / 2) + 1;
        }
        if(lCopyLen > lMaxLen)
        {
            lCopyLen = lMaxLen;
            lPackedStrLen = lCopyLen * 2;
        }

        lpszCurr = lpszBuff + lMaxLen - 1;
        lpszPacked = (char*)(kpszPackedStr + lPackedStrLen - 1);
        for(lIdx = 0; lIdx < lPackedStrLen; lIdx++)
        {
            cNibble = (char)toupper(*lpszPacked);
            cNibble -= 0x30;
            cNibble = PackedTab[cNibble];
            if(0 == (lIdx % 2))
            {
                *lpszCurr = cNibble;
            }
            else
            {
                cNibble <<= 4;
                *lpszCurr |= cNibble;
                lpszCurr--;
            }
            lpszPacked--;
        }
    }
}

/*==========================================================================*/
/* FUNCTION:                                                                */
/*     EvaluateCondition                                                    */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Evaluate the condition                                               */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTIB          Pointer to the test info block                        */
/*     kpszCond       Pointer to the condition to be evaluated              */
/*     bDisplayMsg255 Flag controlling display of message 255               */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     TRUE or FALSE or UNKNOWN                                             */
/*==========================================================================*/
long
EvaluateCondition(TestInfoBlk* lpTIB, const char* const kpszCond, long bDisplayMsg255)
{
    long                    lCondLen;
    long                    lIdx;
    char*                   lpszWork;
    long                    lWorkLen;
    long                    bInsideApostrophes;
    long                    bCompareAsStrings;
    long                    bCompareAsDoubles;
    char*                   lpszCondCurr;
    char*                   lpszWorkCurr;
    char*                   lpszEvalType;
    char*                   lpszPredicate;
    enum ePredicateTypeEnum ePredicateType;
    char*                   lpszLeftHandSide;
    long                    lLeftHandSideLen;
    long                    lLeftHandSideValue;
    double                  dLeftHandSideValue;
    char*                   lpszRightHandSide;
    long                    lRightHandSideLen;
    long                    lRightHandSideValue;
    double                  dRightHandSideValue;
    long                    lStrcmpResult;
    long                    bReturnValue;
    char*                   lpszEndOfScan;

    /* Allocate a work buffer for condition evaluation */
    lCondLen = StrlenNull(kpszCond);
    lWorkLen = ((lCondLen / 8) + 1) * 8;
    lpszWork = malloc(lWorkLen);
    if(lpszWork)
    {
        memset(lpszWork, 0x00, lWorkLen);
    }
    else
    {
        LogMsg(3, lWorkLen, "EvaluateCondition", __LINE__, __FILE__);
        lpTIB->bCmdResult = FAILURE;
        kpMCB->lProcessing = FAILURE;
        return FALSE;
    }

    /* Remove extraneous whitespace from the condition being evaluated */
    lpszCondCurr = (char*)kpszCond;
    lpszWorkCurr = lpszWork;
    bInsideApostrophes = FALSE;
    bCompareAsStrings = FALSE;
    bCompareAsDoubles = FALSE;
    for(lIdx = 0; lIdx < lCondLen; lIdx++)
    {
        if(bInsideApostrophes)
        {
            /* Got a second apostrophe sign so reached the end of the string */
            if('\'' == *lpszCondCurr)
            {
                /* Predicates should be evaluated using strings */
                bCompareAsStrings = TRUE;
                /* Don't copy apostrophes */
                bInsideApostrophes = FALSE;
            }
            else
            {
                /* Copy the current character to the variable name buffer */
                *lpszWorkCurr = *lpszCondCurr;
                lpszWorkCurr++;
            }
        }
        else
        {
            /* Found an apostrophe */
            if('\'' == *lpszCondCurr)
            {
                /* Set flag to TRUE */
                bInsideApostrophes = TRUE;
            }
            else if(isspace(*lpszCondCurr))
            {
                /* Don't copy whitespace */
            }
            else
            {
                /* Copy anything outside of paired apostrophes to the work buffer */
                *lpszWorkCurr = *lpszCondCurr;
                lpszWorkCurr++;
            }
        }
        lpszCondCurr++;
    }

    /* Determine which of the predicates we are trying to evaluate */
    if(NULL != strstr(lpszWork, kpszEQUALTO))
    {
        lpszPredicate = strstr(lpszWork, kpszEQUALTO);
        ePredicateType = Z_EQUALTO;
    }
    else if(NULL != strstr(lpszWork, kpszNOTEQUALTO))
    {
        lpszPredicate = strstr(lpszWork, kpszNOTEQUALTO);
        ePredicateType = Z_NOTEQUALTO;
    }
    else if(NULL != strstr(lpszWork, kpszGREATER))
    {
        lpszPredicate = strstr(lpszWork, kpszGREATER);
        ePredicateType = Z_GREATER;
    }
    else if(NULL != strstr(lpszWork, kpszLESS))
    {
        lpszPredicate = strstr(lpszWork, kpszLESS);
        ePredicateType = Z_LESS;
    }
    else if(NULL != strstr(lpszWork, kpszGREATEREQUAL))
    {
        lpszPredicate = strstr(lpszWork, kpszGREATEREQUAL);
        ePredicateType = Z_GREATEREQUAL;
    }
    else if(NULL != strstr(lpszWork, kpszLESSEQUAL))
    {
        lpszPredicate = strstr(lpszWork, kpszLESSEQUAL);
        ePredicateType = Z_LESSEQUAL;
    }
    else
    {
        /* Unable to determine how to evaluate the condition */
        if(bDisplayMsg255)
        {
            LogMsg(255, lpTIB->lTestNum, kpszUNKNOWN, lpszWork, kpszUNKNOWN);
        }
        return UNKNOWN;
    }

    /* The left hand side starts at the first byte in the work buffer */
    /* the right hand side after the 4 byte long predicate            */
    lpszLeftHandSide = lpszWork;
    lpszRightHandSide = lpszPredicate + 4;

    /* Put in a terminating Null for the LeftHandSide */
    memset(lpszPredicate, 0x00, 1);

    /* Determine the length of the two strings */
    lLeftHandSideLen = StrlenNull(lpszLeftHandSide);
    lRightHandSideLen = StrlenNull(lpszRightHandSide);

    /* If we are going to do a numeric comparsion i.e. bCompareAsStrings is */
    /* FALSE then check to make sure both sides are numeric as if they are  */
    /* not then its an implicit string compare                              */
    if(FALSE == bCompareAsStrings)
    {
        /* Check the left hand side */
        if(FALSE == IsNumeric(lpszLeftHandSide))
        {
            if(FALSE == IsDouble(lpszLeftHandSide))
            {
                bCompareAsStrings = TRUE;
            }
            else
            {
                bCompareAsDoubles = TRUE;
            }
        }
        /* Check the right hand side */
        if(FALSE == IsNumeric(lpszRightHandSide))
        {
            if(FALSE == IsDouble(lpszRightHandSide))
            {
                bCompareAsStrings = TRUE;
            }
            else
            {
                bCompareAsDoubles = TRUE;
            }
        }
    }

    /* Evaluate the condition */
    bReturnValue = UNKNOWN;
    if(TRUE == bCompareAsStrings)
    {
        lpszEvalType = kpszSTRINGEVAL;
        lStrcmpResult = strcmp(lpszLeftHandSide, lpszRightHandSide);
        switch(ePredicateType)
        {
        case Z_EQUALTO:
            if(lLeftHandSideLen == lRightHandSideLen)
            {
                if(0 == lStrcmpResult)
                {
                    bReturnValue = TRUE;
                }
                else
                {
                    bReturnValue = FALSE;
                }
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        case Z_NOTEQUALTO:
            if(lLeftHandSideLen == lRightHandSideLen)
            {
                if(0 == lStrcmpResult)
                {
                    bReturnValue = FALSE;
                }
                else
                {
                    bReturnValue = TRUE;
                }
            }
            else
            {
                bReturnValue = TRUE;
            }
            break;
        case Z_GREATER:
            if(0 < lStrcmpResult)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        case Z_LESS:
            if(0 > lStrcmpResult)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        case Z_GREATEREQUAL:
            if(0 <= lStrcmpResult)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        case Z_LESSEQUAL:
            if(0 >= lStrcmpResult)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        }
    }
    else if(TRUE == bCompareAsDoubles)
    {
        lpszEvalType = kpszDOUBLEEVAL;
        dLeftHandSideValue = strtod(lpszLeftHandSide, &lpszEndOfScan);
        dRightHandSideValue = strtod(lpszRightHandSide, &lpszEndOfScan);
        switch(ePredicateType)
        {
        case Z_EQUALTO:
            if(dLeftHandSideValue == dRightHandSideValue)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        case Z_NOTEQUALTO:
            if(dLeftHandSideValue != dRightHandSideValue)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        case Z_GREATER:
            if(dLeftHandSideValue > dRightHandSideValue)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        case Z_LESS:
            if(dLeftHandSideValue < dRightHandSideValue)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        case Z_GREATEREQUAL:
            if(dLeftHandSideValue >= dRightHandSideValue)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        case Z_LESSEQUAL:
            if(dLeftHandSideValue <= dRightHandSideValue)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        }
    }
    else
    {
        lpszEvalType = kpszINTEGEREVAL;
        lLeftHandSideValue = atol(lpszLeftHandSide);
        lRightHandSideValue = atol(lpszRightHandSide);
        switch(ePredicateType)
        {
        case Z_EQUALTO:
            if(lLeftHandSideValue == lRightHandSideValue)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        case Z_NOTEQUALTO:
            if(lLeftHandSideValue != lRightHandSideValue)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        case Z_GREATER:
            if(lLeftHandSideValue > lRightHandSideValue)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        case Z_LESS:
            if(lLeftHandSideValue < lRightHandSideValue)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        case Z_GREATEREQUAL:
            if(lLeftHandSideValue >= lRightHandSideValue)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        case Z_LESSEQUAL:
            if(lLeftHandSideValue <= lRightHandSideValue)
            {
                bReturnValue = TRUE;
            }
            else
            {
                bReturnValue = FALSE;
            }
            break;
        }
    }

    /* Restore the first byte of the predicate, log and then return the result */
    if(bDisplayMsg255)
    {
        /* Reinstate the predicate */
        *lpszPredicate = '.';
        if(TRUE == bReturnValue)
        {
            LogMsg(255, lpTIB->lTestNum, lpszEvalType, lpszWork, kpszTRUE);
        }
        else if(FALSE == bReturnValue)
        {
            LogMsg(255, lpTIB->lTestNum, lpszEvalType, lpszWork, kpszFALSE);
        }
        else
        {
            LogMsg(255, lpTIB->lTestNum, kpszUNKNOWN, lpszWork, kpszUNKNOWN);
        }
    }

    return bReturnValue;
}


/*==========================================================================*/
/* FUNCTION:                                                                */
/*     EvaluateMathExpression                                               */
/*                                                                          */
/* PURPOSE:                                                                 */
/*     Evaluate a maths expression                                          */
/*                                                                          */
/* INPUT PARAMETERS                                                         */
/*     lpTIB          Pointer to the test info block                        */
/*     kpszEval       Pointer to the expression to be evaluated             */
/*     eVarType       Result variable type                                  */
/*     vpResult       Pointer to the result                                 */
/*                                                                          */
/* RETURN VALUE                                                             */
/*     TRUE or FALSE or UNKNOWN                                             */
/*==========================================================================*/
void
EvaluateMathExpression(TestInfoBlk* lpTIB, const char* const kpszEval, enum eVariableEnum eVarType, void* vpResult)
{
    long                    lEvalLen;
    long                    lIdx;
    char*                   lpszWork;
    long                    lWorkLen;
    char*                   lpszEvalCurr;
    char*                   lpszWorkCurr;
    char*                   lpszOperator;
    enum eEvalTypeEnum      eEvalType;
    char*                   lpszLeftHandSide;
    long                    lLeftHandSideLen;
    long                    lLeftHandSideValue;
    double                  dLeftHandSideValue;
    char*                   lpszRightHandSide;
    long                    lRightHandSideLen;
    long                    lRightHandSideValue;
    double                  dRightHandSideValue;
    char*                   lpszEndOfScan;
    double*                 lpdResult;
    long*                   lplResult;

    /* Allocate a work buffer for evaluation */
    lEvalLen = StrlenNull(kpszEval);
    lWorkLen = ((lEvalLen / 8) + 1) * 8;
    lpszWork = malloc(lWorkLen);
    if(lpszWork)
    {
        memset(lpszWork, 0x00, lWorkLen);
    }
    else
    {
        LogMsg(3, lWorkLen, "EvaluateBuffer", __LINE__, __FILE__);
        lpTIB->bCmdResult = FAILURE;
        kpMCB->lProcessing = FAILURE;
        return;
    }

    /* Remove extraneous whitespace from the string being evaluated */
    lpszEvalCurr = (char*)kpszEval;
    lpszWorkCurr = lpszWork;
    for(lIdx = 0; lIdx < lEvalLen; lIdx++)
    {
        /* Copy everything that is not whitespace to the work buffer */
        if(0 == isspace(*lpszEvalCurr))
        {
            *lpszWorkCurr = *lpszEvalCurr;
            lpszWorkCurr++;
        }
        lpszEvalCurr++;
    }

    /* Determine which of the Operators we are trying to evaluate */
    if(NULL != strstr(lpszWork, kpszADD))
    {
        lpszOperator = strstr(lpszWork, kpszADD);
        eEvalType = Y_ADD;
    }
    else if(NULL != strstr(lpszWork, kpszSUBTRACT))
    {
        lpszOperator = strstr(lpszWork, kpszSUBTRACT);
        eEvalType = Y_SUBTRACT;
    }
    else if(NULL != strstr(lpszWork, kpszMULTIPLY))
    {
        lpszOperator = strstr(lpszWork, kpszMULTIPLY);
        eEvalType = Y_MULTIPLY;
    }
    else if(NULL != strstr(lpszWork, kpszDIVIDE))
    {
        lpszOperator = strstr(lpszWork, kpszDIVIDE);
        eEvalType = Y_DIVIDE;
    }
    else if(NULL != strstr(lpszWork, kpszREMAINDER))
    {
        lpszOperator = strstr(lpszWork, kpszREMAINDER);
        eEvalType = Y_REM;
    }
    else
    {
        LogMsg(255, lpTIB->lTestNum, kpszUNKNOWN, lpszWork, kpszUNKNOWN);
        return;
    }

    /* The left hand side starts at the first byte in the work buffer */
    /* the right hand side after the 5 byte long Operator            */
    lpszLeftHandSide = lpszWork;
    lpszRightHandSide = lpszOperator + 5;

    /* Put in a terminating Null for the LeftHandSide */
    memset(lpszOperator, 0x00, 1);

    /* Determine the length of the two strings */
    lLeftHandSideLen = StrlenNull(lpszLeftHandSide);
    lRightHandSideLen = StrlenNull(lpszRightHandSide);

    /* Evaluate the string */
    if(V_DOUBLE == eVarType)
    {
        dLeftHandSideValue = strtod(lpszLeftHandSide, &lpszEndOfScan);
        dRightHandSideValue = strtod(lpszRightHandSide, &lpszEndOfScan);
        lpdResult = vpResult;
        switch(eEvalType)
        {
        case Y_ADD:
            *lpdResult = (double)(dLeftHandSideValue + dRightHandSideValue);
            break;
        case Y_SUBTRACT:
            *lpdResult = (double)(dLeftHandSideValue - dRightHandSideValue);
            break;
        case Y_MULTIPLY:
            *lpdResult = (double)(dLeftHandSideValue * dRightHandSideValue);
            break;
        case Y_DIVIDE:
            *lpdResult = (double)(dLeftHandSideValue / dRightHandSideValue);
            break;
        case Y_REM:
            *lpdResult = (double)((long)dLeftHandSideValue % (long)dRightHandSideValue);
            break;
        }
    }
    else
    {
        lLeftHandSideValue = atol(lpszLeftHandSide);
        lRightHandSideValue = atol(lpszRightHandSide);
        lplResult = vpResult;
        switch(eEvalType)
        {
        case Y_ADD:
            *lplResult = lLeftHandSideValue + lRightHandSideValue;
            break;
        case Y_SUBTRACT:
            *lplResult = lLeftHandSideValue - lRightHandSideValue;
            break;
        case Y_MULTIPLY:
            *lplResult = lLeftHandSideValue * lRightHandSideValue;
            break;
        case Y_DIVIDE:
            *lplResult = lLeftHandSideValue / lRightHandSideValue;
            break;
        case Y_REM:
            *lplResult = lLeftHandSideValue % lRightHandSideValue;
            break;
        }
    }
}
